@@ -9,6 +9,7 @@
 #include "mozilla/ComputedStyle.h"
 #include "mozilla/a11y/Accessible.h"
 #include "mozilla/a11y/AccTypes.h"
+#include "mozilla/a11y/CacheConstants.h"
 #include "mozilla/a11y/RelationType.h"
 #include "mozilla/a11y/States.h"
 
@@ -53,9 +54,7 @@ class RemoteAccessible;
 class Relation;
 class RootAccessible;
 class TableAccessible;
-class TableAccessibleBase;
 class TableCellAccessible;
-class TableCellAccessibleBase;
 class TextLeafAccessible;
 class XULLabelAccessible;
 class XULTreeAccessible;
@@ -74,13 +73,16 @@ void TreeSize(const char* aTitle, const char* aMsgText, LocalAccessible* aRoot);
 typedef nsRefPtrHashtable<nsPtrHashKey<const void>, LocalAccessible>
     AccessibleHashtable;
 
-#define NS_ACCESSIBLE_IMPL_IID                       \
-  { /* 133c8bf4-4913-4355-bd50-426bd1d6e1ad */       \
-    0x133c8bf4, 0x4913, 0x4355, {                    \
-      0xbd, 0x50, 0x42, 0x6b, 0xd1, 0xd6, 0xe1, 0xad \
-    }                                                \
-  }
+#define NS_ACCESSIBLE_IMPL_IID                \
+  {/* 133c8bf4-4913-4355-bd50-426bd1d6e1ad */ \
+   0x133c8bf4,                                \
+   0x4913,                                    \
+   0x4355,                                    \
+   {0xbd, 0x50, 0x42, 0x6b, 0xd1, 0xd6, 0xe1, 0xad}}
 
+/**
+ * An accessibility tree node that originated in mDoc's content process.
+ */
 class LocalAccessible : public nsISupports, public Accessible {
  public:
   LocalAccessible(nsIContent* aContent, DocAccessible* aDoc);
@@ -88,7 +90,7 @@ class LocalAccessible : public nsISupports, public Accessible {
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_CLASS(LocalAccessible)
 
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_ACCESSIBLE_IMPL_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_ACCESSIBLE_IMPL_IID)
 
   //////////////////////////////////////////////////////////////////////////////
   // Public methods
@@ -135,10 +137,7 @@ class LocalAccessible : public nsISupports, public Accessible {
     return IsDoc() ? 0 : reinterpret_cast<uintptr_t>(this);
   }
 
-  /**
-   * Return language associated with the accessible.
-   */
-  void Language(nsAString& aLocale);
+  virtual void Language(nsAString& aLocale) override;
 
   /**
    * Get the description of this accessible.
@@ -150,11 +149,6 @@ class LocalAccessible : public nsISupports, public Accessible {
    */
   virtual void Value(nsString& aValue) const override;
 
-  /**
-   * Get help string for the accessible.
-   */
-  void Help(nsString& aHelp) const { aHelp.Truncate(); }
-
   /**
    * Get the name of this accessible.
    */
@@ -178,7 +172,7 @@ class LocalAccessible : public nsISupports, public Accessible {
    * Return accessible role specified by ARIA (see constants in
    * roles).
    */
-  mozilla::a11y::role ARIARole();
+  inline mozilla::a11y::role ARIARole();
 
   /**
    * Returns enumerated accessible role from native markup (see constants in
@@ -265,7 +259,7 @@ class LocalAccessible : public nsISupports, public Accessible {
   /**
    * Set the ARIA role map entry for a new accessible.
    */
-  void SetRoleMapEntry(const nsRoleMapEntry* aRoleMapEntry);
+  inline void SetRoleMapEntry(const nsRoleMapEntry* aRoleMapEntry);
 
   /**
    * Append/insert/remove a child. Return true if operation was successful.
@@ -280,7 +274,8 @@ class LocalAccessible : public nsISupports, public Accessible {
    * then the child is unbound from the document, and false is returned. Make
    * sure to null out any references on the child object as it may be destroyed.
    */
-  bool InsertAfter(LocalAccessible* aNewChild, LocalAccessible* aRefChild);
+  inline bool InsertAfter(LocalAccessible* aNewChild,
+                          LocalAccessible* aRefChild);
 
   virtual bool RemoveChild(LocalAccessible* aChild);
 
@@ -355,7 +350,7 @@ class LocalAccessible : public nsISupports, public Accessible {
   /**
    * Return embedded accessible child at the given index.
    */
-  virtual LocalAccessible* EmbeddedChildAt(uint32_t aIndex) override;
+  virtual Accessible* EmbeddedChildAt(uint32_t aIndex) override;
 
   virtual int32_t IndexOfEmbeddedChild(Accessible* aChild) override;
 
@@ -429,10 +424,8 @@ class LocalAccessible : public nsISupports, public Accessible {
   MOZ_CAN_RUN_SCRIPT
   virtual void ScrollTo(uint32_t aHow) const override;
 
-  /**
-   * Scroll the accessible to the given point.
-   */
-  void ScrollToPoint(uint32_t aCoordinateType, int32_t aX, int32_t aY);
+  virtual void ScrollToPoint(uint32_t aCoordinateType, int32_t aX,
+                             int32_t aY) override;
 
   /**
    * Get a pointer to accessibility interface for this node, which is specific
@@ -442,11 +435,15 @@ class LocalAccessible : public nsISupports, public Accessible {
 
   virtual Maybe<int32_t> GetIntARIAAttr(nsAtom* aAttrName) const override;
 
+  virtual bool GetStringARIAAttr(nsAtom* aAttrName,
+                                 nsAString& aAttrValue) const override;
+
   //////////////////////////////////////////////////////////////////////////////
   // Downcasting and types
 
   inline bool IsAbbreviation() const {
-    return mContent->IsAnyOfHTMLElements(nsGkAtoms::abbr, nsGkAtoms::acronym);
+    return mContent &&
+           mContent->IsAnyOfHTMLElements(nsGkAtoms::abbr, nsGkAtoms::acronym);
   }
 
   ApplicationAccessible* AsApplication();
@@ -464,26 +461,12 @@ class LocalAccessible : public nsISupports, public Accessible {
 
   HTMLImageMapAccessible* AsImageMap();
 
-  RemoteAccessible* Proxy() const {
-    MOZ_ASSERT(IsProxy());
-    return mBits.proxy;
-  }
-
   OuterDocAccessible* AsOuterDoc();
 
   a11y::RootAccessible* AsRoot();
 
-  bool IsSearchbox() const;
-
-  virtual TableAccessible* AsTable() { return nullptr; }
-
-  virtual TableCellAccessible* AsTableCell() { return nullptr; }
-  const TableCellAccessible* AsTableCell() const {
-    return const_cast<LocalAccessible*>(this)->AsTableCell();
-  }
-
-  virtual TableAccessibleBase* AsTableBase() override;
-  virtual TableCellAccessibleBase* AsTableCellBase() override;
+  virtual TableAccessible* AsTable() override;
+  virtual TableCellAccessible* AsTableCell() override;
 
   TextLeafAccessible* AsTextLeaf();
 
@@ -519,34 +502,6 @@ class LocalAccessible : public nsISupports, public Accessible {
    */
   virtual bool IsLink() const override;
 
-  /**
-   * Return true if the link is valid (e. g. points to a valid URL).
-   */
-  inline bool IsLinkValid() {
-    MOZ_ASSERT(IsLink(), "IsLinkValid is called on not hyper link!");
-
-    // XXX In order to implement this we would need to follow every link
-    // Perhaps we can get information about invalid links from the cache
-    // In the mean time authors can use role="link" aria-invalid="true"
-    // to force it for links they internally know to be invalid
-    return (0 == (State() & mozilla::a11y::states::INVALID));
-  }
-
-  /**
-   * Return the number of anchors within the link.
-   */
-  virtual uint32_t AnchorCount();
-
-  /**
-   * Returns an anchor accessible at the given index.
-   */
-  virtual LocalAccessible* AnchorAt(uint32_t aAnchorIndex);
-
-  /**
-   * Returns an anchor URI at the given index.
-   */
-  virtual already_AddRefed<nsIURI> AnchorURIAt(uint32_t aAnchorIndex) const;
-
   //////////////////////////////////////////////////////////////////////////////
   // SelectAccessible
 
@@ -597,7 +552,7 @@ class LocalAccessible : public nsISupports, public Accessible {
   virtual double MinValue() const override;
   virtual double CurValue() const override;
   virtual double Step() const override;
-  virtual bool SetCurValue(double aValue);
+  virtual bool SetCurValue(double aValue) override;
 
   //////////////////////////////////////////////////////////////////////////////
   // Widgets
@@ -636,12 +591,17 @@ class LocalAccessible : public nsISupports, public Accessible {
    */
   virtual LocalAccessible* ContainerWidget() const;
 
-  bool IsActiveDescendant(LocalAccessible** aWidget = nullptr) const;
+  /**
+   * Accessible's element ID is referenced as a aria-activedescendant in the
+   * document. This method is only used for ID changes and therefore does not
+   * need to work for direct element references via ariaActiveDescendantElement.
+   */
+  bool IsActiveDescendantId(LocalAccessible** aWidget = nullptr) const;
 
   /**
    * Return true if the accessible is defunct.
    */
-  bool IsDefunct() const;
+  inline bool IsDefunct() const;
 
   /**
    * Return false if the accessible is no longer in the document.
@@ -665,12 +625,12 @@ class LocalAccessible : public nsISupports, public Accessible {
   /**
    * Return true if native markup has a numeric value.
    */
-  bool NativeHasNumericValue() const;
+  inline bool NativeHasNumericValue() const;
 
   /**
    * Return true if ARIA specifies support for a numeric value.
    */
-  bool ARIAHasNumericValue() const;
+  inline bool ARIAHasNumericValue() const;
 
   /**
    * Return true if the accessible has a numeric value.
@@ -759,14 +719,18 @@ class LocalAccessible : public nsISupports, public Accessible {
   virtual bool IsRemote() const override { return false; }
 
   already_AddRefed<AccAttributes> BundleFieldsForCache(
-      uint64_t aCacheDomain, CacheUpdateType aUpdateType);
+      uint64_t aCacheDomain, CacheUpdateType aUpdateType,
+      uint64_t aInitialDomains = CacheDomain::None);
 
   /**
    * Push fields to cache.
    * aCacheDomain - describes which fields to bundle and ultimately send
    * aUpdate - describes whether this is an initial or subsequent update
+   * aAppendEventData - don't send the event now; append it to the mutation
+   *                    events list on the DocAccessibleChild
    */
-  void SendCache(uint64_t aCacheDomain, CacheUpdateType aUpdate);
+  void SendCache(uint64_t aCacheDomain, CacheUpdateType aUpdate,
+                 bool aAppendEventData = false);
 
   void MaybeQueueCacheUpdateForStyleChanges();
 
@@ -778,6 +742,8 @@ class LocalAccessible : public nsISupports, public Accessible {
 
   virtual void DOMNodeID(nsString& aID) const override;
 
+  virtual void DOMNodeClass(nsString& aClass) const override;
+
   virtual void LiveRegionAttributes(nsAString* aLive, nsAString* aRelevant,
                                     Maybe<bool>* aAtomic,
                                     nsAString* aBusy) const override;
@@ -799,6 +765,8 @@ class LocalAccessible : public nsISupports, public Accessible {
    */
   void NativeDescription(nsString& aDescription) const;
 
+  uint64_t ExplicitState() const;
+
   /**
    * Return object attributes provided by native markup. It doesn't take into
    * account ARIA.
@@ -885,13 +853,19 @@ class LocalAccessible : public nsISupports, public Accessible {
    */
   mozilla::a11y::role ARIATransformRole(mozilla::a11y::role aRole) const;
 
+  /**
+   * Return the minimum role that should be used as a last resort if the element
+   * does not have a more specific role.
+   */
+  mozilla::a11y::role GetMinimumRole(mozilla::a11y::role aRole) const;
+
   //////////////////////////////////////////////////////////////////////////////
   // Name helpers
 
   /**
    * Returns the accessible name specified by ARIA.
    */
-  void ARIAName(nsString& aName) const;
+  ENameValueFlag ARIAName(nsString& aName) const;
 
   /**
    * Returns the accessible description specified by ARIA.
@@ -929,18 +903,15 @@ class LocalAccessible : public nsISupports, public Accessible {
    *  invoke action of mozilla accessibles direclty (see bug 277888 for
    * details).
    *
-   * @param  aContent      [in, optional] element to click
    * @param  aActionIndex  [in, optional] index of accessible action
    */
-  void DoCommand(nsIContent* aContent = nullptr,
-                 uint32_t aActionIndex = 0) const;
+  void DoCommand(uint32_t aActionIndex = 0) const;
 
   /**
    * Dispatch click event.
    */
   MOZ_CAN_RUN_SCRIPT
-  virtual void DispatchClickEvent(nsIContent* aContent,
-                                  uint32_t aActionIndex) const;
+  virtual void DispatchClickEvent(uint32_t aActionIndex) const;
 
   //////////////////////////////////////////////////////////////////////////////
   // Helpers
@@ -974,7 +945,7 @@ class LocalAccessible : public nsISupports, public Accessible {
 
   // Data Members
   // mContent can be null in a DocAccessible if the document has no body or
-  // root element.
+  // root element, or if the initial tree hasn't been constructed yet.
   nsCOMPtr<nsIContent> mContent;
   RefPtr<DocAccessible> mDoc;
 
@@ -1036,10 +1007,7 @@ class LocalAccessible : public nsISupports, public Accessible {
 
   friend class EmbeddedObjCollector;
 
-  union {
-    AccGroupInfo* groupInfo;
-    RemoteAccessible* proxy;
-  } mutable mBits;
+  mutable AccGroupInfo* mGroupInfo;
   friend class AccGroupInfo;
 
  private:
@@ -1053,9 +1021,20 @@ class LocalAccessible : public nsISupports, public Accessible {
    * OOP iframe docs and tab documents.
    */
   nsIFrame* FindNearestAccessibleAncestorFrame();
-};
 
-NS_DEFINE_STATIC_IID_ACCESSOR(LocalAccessible, NS_ACCESSIBLE_IMPL_IID)
+  /*
+   * This function assumes that the current role is not valid. It searches for a
+   * fallback role in the role attribute string, and returns it. If there is no
+   * valid fallback role in the role attribute string, the function returns the
+   * native role. The aRolesToSkip parameter will cause the function to skip any
+   * roles found in the role attribute string when searching for the next valid
+   * role.
+   */
+  role FindNextValidARIARole(
+      std::initializer_list<nsStaticAtom*> aRolesToSkip) const;
+
+  LocalAccessible* GetPopoverTargetDetailsRelation() const;
+};
 
 ////////////////////////////////////////////////////////////////////////////////
 // LocalAccessible downcasting method
