@@ -1580,13 +1580,6 @@ uint64_t RemoteAccessible::State() {
             mCachedFields->GetAttribute<uint64_t>(CacheKey::State)) {
       VERIFY_CACHE(CacheDomain::State);
       state = *rawState;
-      // Handle states that are derived from other states.
-      if (!(state & states::UNAVAILABLE)) {
-        state |= states::ENABLED | states::SENSITIVE;
-      }
-      if (state & states::EXPANDABLE && !(state & states::EXPANDED)) {
-        state |= states::COLLAPSED;
-      }
     }
 
     ApplyImplicitState(state);
@@ -2337,6 +2330,18 @@ Maybe<int32_t> RemoteAccessible::GetIntARIAAttr(nsAtom* aAttrName) const {
   return Nothing();
 }
 
+bool RemoteAccessible::GetStringARIAAttr(nsAtom* aAttrName,
+                                         nsAString& aAttrValue) const {
+  if (RequestDomainsIfInactive(CacheDomain::ARIA)) {
+    return false;
+  }
+  if (auto attrs = GetCachedARIAAttributes()) {
+    return attrs->GetAttribute(aAttrName, aAttrValue);
+  }
+
+  return false;
+}
+
 void RemoteAccessible::Language(nsAString& aLocale) {
   if (RequestDomainsIfInactive(CacheDomain::Text)) {
     return;
