@@ -5,27 +5,30 @@
 
 #include "XULMenuAccessible.h"
 
-#include "Accessible-inl.h"
+#include "LocalAccessible-inl.h"
+#include "XULMenuBarElement.h"
+#include "XULMenuParentElement.h"
+#include "XULPopupElement.h"
+#include "mozilla/Assertions.h"
 #include "nsAccessibilityService.h"
 #include "nsAccUtils.h"
 #include "DocAccessible.h"
-#include "Role.h"
+#include "mozilla/a11y/Role.h"
 #include "States.h"
 #include "XULFormControlAccessible.h"
 
-#include "nsIMutableArray.h"
+#include "nsIContentInlines.h"
 #include "nsIDOMXULContainerElement.h"
+#include "nsIDOMXULSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
-#include "nsIDOMXULMultSelectCntrlEl.h"
-#include "nsIServiceManager.h"
-#include "nsIPresShell.h"
 #include "nsIContent.h"
-#include "nsMenuBarFrame.h"
 #include "nsMenuPopupFrame.h"
 
 #include "mozilla/Preferences.h"
 #include "mozilla/LookAndFeel.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/XULButtonElement.h"
 #include "mozilla/dom/KeyboardEventBinding.h"
 
 using namespace mozilla;
@@ -37,25 +40,22 @@ using namespace mozilla::a11y;
 
 XULMenuitemAccessible::XULMenuitemAccessible(nsIContent* aContent,
                                              DocAccessible* aDoc)
-    : AccessibleWrap(aContent, aDoc) {
-  mStateFlags |= eNoXBLKids;
-}
+    : AccessibleWrap(aContent, aDoc) {}
 
 uint64_t XULMenuitemAccessible::NativeState() const {
-  uint64_t state = Accessible::NativeState();
+  uint64_t state = LocalAccessible::NativeState();
 
   // Has Popup?
   if (mContent->NodeInfo()->Equals(nsGkAtoms::menu, kNameSpaceID_XUL)) {
-    state |= states::HASPOPUP;
-    if (mContent->AsElement()->HasAttr(kNameSpaceID_None, nsGkAtoms::open))
+    state |= states::HASPOPUP | states::EXPANDABLE;
+    if (mContent->AsElement()->HasAttr(nsGkAtoms::open)) {
       state |= states::EXPANDED;
-    else
-      state |= states::COLLAPSED;
+    }
   }
 
   // Checkable/checked?
-  static Element::AttrValuesArray strings[] = {nsGkAtoms::radio,
-                                               nsGkAtoms::checkbox, nullptr};
+  static dom::Element::AttrValuesArray strings[] = {
+      nsGkAtoms::radio, nsGkAtoms::checkbox, nullptr};
 
   if (mContent->AsElement()->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::type,
                                              strings, eCaseMatters) >= 0) {
@@ -65,8 +65,9 @@ uint64_t XULMenuitemAccessible::NativeState() const {
     // Checked?
     if (mContent->AsElement()->AttrValueIs(kNameSpaceID_None,
                                            nsGkAtoms::checked, nsGkAtoms::_true,
-                                           eCaseMatters))
+                                           eCaseMatters)) {
       state |= states::CHECKED;
+    }
   }
 
   // Combo box listitem
@@ -81,7 +82,7 @@ uint64_t XULMenuitemAccessible::NativeState() const {
 
     // Is collapsed?
     bool isCollapsed = false;
-    Accessible* parent = Parent();
+    LocalAccessible* parent = LocalParent();
     if (parent && parent->State() & states::INVISIBLE) isCollapsed = true;
 
     if (isSelected) {
@@ -90,7 +91,7 @@ uint64_t XULMenuitemAccessible::NativeState() const {
       // Selected and collapsed?
       if (isCollapsed) {
         // Set selected option offscreen/invisible according to combobox state
-        Accessible* grandParent = parent->Parent();
+        LocalAccessible* grandParent = parent->LocalParent();
         if (!grandParent) return state;
         NS_ASSERTION(grandParent->IsCombobox(),
                      "grandparent of combobox listitem is not combobox");
@@ -100,8 +101,8 @@ uint64_t XULMenuitemAccessible::NativeState() const {
                  (grandParentState & states::INVISIBLE) |
                  (grandParentState & states::OPAQUE1);
       }  // isCollapsed
-    }    // isSelected
-  }      // ROLE_COMBOBOX_OPTION
+    }  // isSelected
+  }  // ROLE_COMBOBOX_OPTION
 
   return state;
 }
@@ -109,12 +110,11 @@ uint64_t XULMenuitemAccessible::NativeState() const {
 uint64_t XULMenuitemAccessible::NativeInteractiveState() const {
   if (NativelyUnavailable()) {
     // Note: keep in sinc with nsXULPopupManager::IsValidMenuItem() logic.
+    auto* button = dom::XULButtonElement::FromNode(GetContent());
     bool skipNavigatingDisabledMenuItem = true;
-    nsMenuFrame* menuFrame = do_QueryFrame(GetFrame());
-    if (!menuFrame || !menuFrame->IsOnMenuBar()) {
-      skipNavigatingDisabledMenuItem =
-          LookAndFeel::GetInt(
-              LookAndFeel::eIntID_SkipNavigatingDisabledMenuItem, 0) != 0;
+    if (!button || !button->IsOnMenuBar()) {
+      skipNavigatingDisabledMenuItem = LookAndFeel::GetInt(
+          LookAndFeel::IntID::SkipNavigatingDisabledMenuItem);
     }
 
     if (skipNavigatingDisabledMenuItem) return states::UNAVAILABLE;
@@ -126,13 +126,12 @@ uint64_t XULMenuitemAccessible::NativeInteractiveState() const {
 }
 
 ENameValueFlag XULMenuitemAccessible::NativeName(nsString& aName) const {
-  mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::label, aName);
+  mContent->AsElement()->GetAttr(nsGkAtoms::label, aName);
   return eNameOK;
 }
 
-void XULMenuitemAccessible::Description(nsString& aDescription) {
-  mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::description,
-                                 aDescription);
+void XULMenuitemAccessible::Description(nsString& aDescription) const {
+  mContent->AsElement()->GetAttr(nsGkAtoms::description, aDescription);
 }
 
 KeyBinding XULMenuitemAccessible::AccessKey() const {
@@ -143,13 +142,12 @@ KeyBinding XULMenuitemAccessible::AccessKey() const {
   // We do not use nsCoreUtils::GetAccesskeyFor() because accesskeys for
   // menu are't registered by EventStateManager.
   nsAutoString accesskey;
-  mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::accesskey,
-                                 accesskey);
+  mContent->AsElement()->GetAttr(nsGkAtoms::accesskey, accesskey);
   if (accesskey.IsEmpty()) return KeyBinding();
 
   uint32_t modifierKey = 0;
 
-  Accessible* parentAcc = Parent();
+  LocalAccessible* parentAcc = LocalParent();
   if (parentAcc) {
     if (parentAcc->NativeRole() == roles::MENUBAR) {
       // If top level menu item, add Alt+ or whatever modifier text to string
@@ -167,10 +165,8 @@ KeyBinding XULMenuitemAccessible::AccessKey() const {
           modifierKey = KeyBinding::kAlt;
           break;
         case dom::KeyboardEvent_Binding::DOM_VK_META:
-          modifierKey = KeyBinding::kMeta;
-          break;
         case dom::KeyboardEvent_Binding::DOM_VK_WIN:
-          modifierKey = KeyBinding::kOS;
+          modifierKey = KeyBinding::kMeta;
           break;
       }
     }
@@ -181,19 +177,19 @@ KeyBinding XULMenuitemAccessible::AccessKey() const {
 
 KeyBinding XULMenuitemAccessible::KeyboardShortcut() const {
   nsAutoString keyElmId;
-  mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::key, keyElmId);
+  mContent->AsElement()->GetAttr(nsGkAtoms::key, keyElmId);
   if (keyElmId.IsEmpty()) return KeyBinding();
 
-  Element* keyElm = mContent->OwnerDoc()->GetElementById(keyElmId);
+  dom::Element* keyElm = mContent->OwnerDoc()->GetElementById(keyElmId);
   if (!keyElm) return KeyBinding();
 
   uint32_t key = 0;
 
   nsAutoString keyStr;
-  keyElm->GetAttr(kNameSpaceID_None, nsGkAtoms::key, keyStr);
+  keyElm->GetAttr(nsGkAtoms::key, keyStr);
   if (keyStr.IsEmpty()) {
     nsAutoString keyCodeStr;
-    keyElm->GetAttr(kNameSpaceID_None, nsGkAtoms::keycode, keyCodeStr);
+    keyElm->GetAttr(nsGkAtoms::keycode, keyCodeStr);
     nsresult errorCode;
     key = keyStr.ToInteger(&errorCode, /* aRadix = */ 10);
     if (NS_FAILED(errorCode)) {
@@ -204,15 +200,14 @@ KeyBinding XULMenuitemAccessible::KeyboardShortcut() const {
   }
 
   nsAutoString modifiersStr;
-  keyElm->GetAttr(kNameSpaceID_None, nsGkAtoms::modifiers, modifiersStr);
+  keyElm->GetAttr(nsGkAtoms::modifiers, modifiersStr);
 
   uint32_t modifierMask = 0;
-  if (modifiersStr.Find("shift") != -1) modifierMask |= KeyBinding::kShift;
-  if (modifiersStr.Find("alt") != -1) modifierMask |= KeyBinding::kAlt;
-  if (modifiersStr.Find("meta") != -1) modifierMask |= KeyBinding::kMeta;
-  if (modifiersStr.Find("os") != -1) modifierMask |= KeyBinding::kOS;
-  if (modifiersStr.Find("control") != -1) modifierMask |= KeyBinding::kControl;
-  if (modifiersStr.Find("accel") != -1) {
+  if (modifiersStr.Find(u"shift") != -1) modifierMask |= KeyBinding::kShift;
+  if (modifiersStr.Find(u"alt") != -1) modifierMask |= KeyBinding::kAlt;
+  if (modifiersStr.Find(u"meta") != -1) modifierMask |= KeyBinding::kMeta;
+  if (modifiersStr.Find(u"control") != -1) modifierMask |= KeyBinding::kControl;
+  if (modifiersStr.Find(u"accel") != -1) {
     modifierMask |= KeyBinding::AccelModifier();
   }
 
@@ -223,45 +218,42 @@ role XULMenuitemAccessible::NativeRole() const {
   nsCOMPtr<nsIDOMXULContainerElement> xulContainer = Elm()->AsXULContainer();
   if (xulContainer) return roles::PARENT_MENUITEM;
 
-  if (mParent && mParent->Role() == roles::COMBOBOX_LIST)
+  LocalAccessible* widget = ContainerWidget();
+  if (widget && widget->Role() == roles::COMBOBOX_LIST) {
     return roles::COMBOBOX_OPTION;
+  }
 
   if (mContent->AsElement()->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
-                                         nsGkAtoms::radio, eCaseMatters))
+                                         nsGkAtoms::radio, eCaseMatters)) {
     return roles::RADIO_MENU_ITEM;
+  }
 
   if (mContent->AsElement()->AttrValueIs(kNameSpaceID_None, nsGkAtoms::type,
-                                         nsGkAtoms::checkbox, eCaseMatters))
+                                         nsGkAtoms::checkbox, eCaseMatters)) {
     return roles::CHECK_MENU_ITEM;
+  }
 
   return roles::MENUITEM;
 }
 
-int32_t XULMenuitemAccessible::GetLevelInternal() {
+int32_t XULMenuitemAccessible::GetLevel(bool aFast) const {
   return nsAccUtils::GetLevelForXULContainerItem(mContent);
 }
 
-bool XULMenuitemAccessible::DoAction(uint8_t index) const {
-  if (index == eAction_Click) {  // default action
-    DoCommand();
-    return true;
-  }
-
-  return false;
-}
-
 void XULMenuitemAccessible::ActionNameAt(uint8_t aIndex, nsAString& aName) {
   if (aIndex == eAction_Click) aName.AssignLiteral("click");
 }
 
-uint8_t XULMenuitemAccessible::ActionCount() const { return 1; }
+bool XULMenuitemAccessible::HasPrimaryAction() const { return true; }
 
 ////////////////////////////////////////////////////////////////////////////////
 // XULMenuitemAccessible: Widgets
 
 bool XULMenuitemAccessible::IsActiveWidget() const {
   // Parent menu item is a widget, it's active when its popup is open.
-  nsIContent* menuPopupContent = mContent->GetFirstChild();
+  // Typically the <menupopup> is included in the document markup, and
+  // <menu> prepends content in front of it.
+  nsIContent* menuPopupContent = mContent->GetLastChild();
   if (menuPopupContent) {
     nsMenuPopupFrame* menuPopupFrame =
         do_QueryFrame(menuPopupContent->GetPrimaryFrame());
@@ -273,7 +265,7 @@ bool XULMenuitemAccessible::IsActiveWidget() const {
 bool XULMenuitemAccessible::AreItemsOperable() const {
   // Parent menu item is a widget, its items are operable when its popup is
   // open.
-  nsIContent* menuPopupContent = mContent->GetFirstChild();
+  nsIContent* menuPopupContent = mContent->GetLastChild();
   if (menuPopupContent) {
     nsMenuPopupFrame* menuPopupFrame =
         do_QueryFrame(menuPopupContent->GetPrimaryFrame());
@@ -282,19 +274,12 @@ bool XULMenuitemAccessible::AreItemsOperable() const {
   return false;
 }
 
-Accessible* XULMenuitemAccessible::ContainerWidget() const {
-  nsMenuFrame* menuFrame = do_QueryFrame(GetFrame());
-  if (menuFrame) {
-    nsMenuParent* menuParent = menuFrame->GetMenuParent();
-    if (menuParent) {
-      if (menuParent->IsMenuBar())  // menubar menu
-        return mParent;
-
-      // a menupoup or parent menu item
-      if (menuParent->IsMenu()) return mParent;
-
-      // otherwise it's different kind of popups (like panel or tooltip), it
-      // shouldn't be a real case.
+LocalAccessible* XULMenuitemAccessible::ContainerWidget() const {
+  if (auto* button = dom::XULButtonElement::FromNode(GetContent())) {
+    if (auto* popup = button->GetMenuParent()) {
+      // We use GetAccessibleOrContainer instead of just GetAccessible because
+      // we strip menupopups from the tree for ATK.
+      return mDoc->GetAccessibleOrContainer(popup);
     }
   }
   return nullptr;
@@ -320,14 +305,7 @@ ENameValueFlag XULMenuSeparatorAccessible::NativeName(nsString& aName) const {
 
 role XULMenuSeparatorAccessible::NativeRole() const { return roles::SEPARATOR; }
 
-bool XULMenuSeparatorAccessible::DoAction(uint8_t index) const { return false; }
-
-void XULMenuSeparatorAccessible::ActionNameAt(uint8_t aIndex,
-                                              nsAString& aName) {
-  aName.Truncate();
-}
-
-uint8_t XULMenuSeparatorAccessible::ActionCount() const { return 0; }
+bool XULMenuSeparatorAccessible::HasPrimaryAction() const { return false; }
 
 ////////////////////////////////////////////////////////////////////////////////
 // XULMenupopupAccessible
@@ -336,38 +314,37 @@ uint8_t XULMenuSeparatorAccessible::ActionCount() const { return 0; }
 XULMenupopupAccessible::XULMenupopupAccessible(nsIContent* aContent,
                                                DocAccessible* aDoc)
     : XULSelectControlAccessible(aContent, aDoc) {
-  nsMenuPopupFrame* menuPopupFrame = do_QueryFrame(GetFrame());
-  if (menuPopupFrame && menuPopupFrame->IsMenu()) mType = eMenuPopupType;
+  if (nsMenuPopupFrame* menuPopupFrame = do_QueryFrame(GetFrame())) {
+    if (menuPopupFrame->GetPopupType() == widget::PopupType::Menu) {
+      mType = eMenuPopupType;
+    }
+  }
 
   // May be the anonymous <menupopup> inside <menulist> (a combobox)
-  nsIContent* parent = mContent->GetFlattenedTreeParent();
+  auto* parent = mContent->GetParentElement();
   nsCOMPtr<nsIDOMXULSelectControlElement> selectControl =
-      parent && parent->AsElement() ? parent->AsElement()->AsXULSelectControl()
-                                    : nullptr;
+      parent ? parent->AsXULSelectControl() : nullptr;
   if (selectControl) {
-    mSelectControl = parent->AsElement();
+    mSelectControl = parent;
   } else {
     mSelectControl = nullptr;
     mGenericTypes &= ~eSelect;
   }
-
-  mStateFlags |= eNoXBLKids;
 }
 
 uint64_t XULMenupopupAccessible::NativeState() const {
-  uint64_t state = Accessible::NativeState();
+  uint64_t state = LocalAccessible::NativeState();
 
 #ifdef DEBUG
   // We are onscreen if our parent is active
-  bool isActive =
-      mContent->AsElement()->HasAttr(kNameSpaceID_None, nsGkAtoms::menuactive);
+  nsMenuPopupFrame* menuPopupFrame = do_QueryFrame(GetFrame());
+  bool isActive = menuPopupFrame ? menuPopupFrame->IsOpen() : false;
   if (!isActive) {
-    Accessible* parent = Parent();
+    LocalAccessible* parent = LocalParent();
     if (parent) {
       nsIContent* parentContent = parent->GetContent();
       if (parentContent && parentContent->IsElement())
-        isActive = parentContent->AsElement()->HasAttr(kNameSpaceID_None,
-                                                       nsGkAtoms::open);
+        isActive = parentContent->AsElement()->HasAttr(nsGkAtoms::open);
     }
   }
 
@@ -375,7 +352,9 @@ uint64_t XULMenupopupAccessible::NativeState() const {
                "XULMenupopup doesn't have INVISIBLE when it's inactive");
 #endif
 
-  if (state & states::INVISIBLE) state |= states::OFFSCREEN | states::COLLAPSED;
+  if (state & states::INVISIBLE) {
+    state |= states::OFFSCREEN;
+  }
 
   return state;
 }
@@ -384,7 +363,7 @@ ENameValueFlag XULMenupopupAccessible::NativeName(nsString& aName) const {
   nsIContent* content = mContent;
   while (content && aName.IsEmpty()) {
     if (content->IsElement()) {
-      content->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::label, aName);
+      content->AsElement()->GetAttr(nsGkAtoms::label, aName);
     }
     content = content->GetFlattenedTreeParent();
   }
@@ -393,20 +372,19 @@ ENameValueFlag XULMenupopupAccessible::NativeName(nsString& aName) const {
 }
 
 role XULMenupopupAccessible::NativeRole() const {
-  // If accessible is not bound to the tree (this happens while children are
-  // cached) return general role.
-  if (mParent) {
-    if (mParent->IsCombobox() || mParent->IsAutoComplete())
-      return roles::COMBOBOX_LIST;
+  nsMenuPopupFrame* menuPopupFrame = do_QueryFrame(GetFrame());
+  if (menuPopupFrame && menuPopupFrame->IsContextMenu()) {
+    return roles::MENUPOPUP;
+  }
 
-    if (mParent->Role() == roles::PUSHBUTTON) {
-      // Some widgets like the search bar have several popups, owned by buttons.
-      Accessible* grandParent = mParent->Parent();
-      if (grandParent && grandParent->IsAutoComplete())
-        return roles::COMBOBOX_LIST;
+  if (mParent) {
+    if (mParent->IsCombobox()) {
+      return roles::COMBOBOX_LIST;
     }
   }
 
+  // If accessible is not bound to the tree (this happens while children are
+  // cached) return general role.
   return roles::MENUPOPUP;
 }
 
@@ -426,36 +404,38 @@ bool XULMenupopupAccessible::AreItemsOperable() const {
   return menuPopupFrame && menuPopupFrame->IsOpen();
 }
 
-Accessible* XULMenupopupAccessible::ContainerWidget() const {
+LocalAccessible* XULMenupopupAccessible::ContainerWidget() const {
   DocAccessible* document = Document();
 
   nsMenuPopupFrame* menuPopupFrame = do_QueryFrame(GetFrame());
-  while (menuPopupFrame) {
-    Accessible* menuPopup =
-        document->GetAccessible(menuPopupFrame->GetContent());
-    if (!menuPopup)  // shouldn't be a real case
-      return nullptr;
-
-    nsMenuFrame* menuFrame = do_QueryFrame(menuPopupFrame->GetParent());
-    if (!menuFrame)  // context menu or popups
-      return nullptr;
-
-    nsMenuParent* menuParent = menuFrame->GetMenuParent();
-    if (!menuParent)  // menulist or menubutton
-      return menuPopup->Parent();
-
-    if (menuParent->IsMenuBar()) {  // menubar menu
-      nsMenuBarFrame* menuBarFrame = static_cast<nsMenuBarFrame*>(menuParent);
-      return document->GetAccessible(menuBarFrame->GetContent());
-    }
-
-    // different kind of popups like panel or tooltip
-    if (!menuParent->IsMenu()) return nullptr;
-
-    menuPopupFrame = static_cast<nsMenuPopupFrame*>(menuParent);
+  MOZ_ASSERT(menuPopupFrame);
+  if (!menuPopupFrame) {
+    return nullptr;
   }
 
-  MOZ_ASSERT_UNREACHABLE("Shouldn't be a real case.");
+  auto* cur = dom::XULPopupElement::FromNode(GetContent());
+  while (cur) {
+    auto* menu = cur->GetContainingMenu();
+    if (!menu) {
+      // <panel> / <tooltip> / etc.
+      return nullptr;
+    }
+    dom::XULMenuParentElement* parent = menu->GetMenuParent();
+    if (!parent) {
+      LocalAccessible* menuPopup = document->GetAccessible(cur);
+      MOZ_ASSERT(menuPopup);
+      return menuPopup ? menuPopup->LocalParent() : nullptr;
+    }
+
+    if (parent->IsMenuBar()) {
+      return document->GetAccessible(parent);
+    }
+
+    cur = dom::XULPopupElement::FromNode(parent);
+    MOZ_ASSERT(cur, "Should be a popup");
+  }
+
+  MOZ_ASSERT_UNREACHABLE("How did we get out of the loop without returning?");
   return nullptr;
 }
 
@@ -478,24 +458,21 @@ role XULMenubarAccessible::NativeRole() const { return roles::MENUBAR; }
 // XULMenubarAccessible: Widgets
 
 bool XULMenubarAccessible::IsActiveWidget() const {
-  nsMenuBarFrame* menuBarFrame = do_QueryFrame(GetFrame());
-  return menuBarFrame && menuBarFrame->IsActive();
+  auto* menuBar = dom::XULMenuBarElement::FromNode(GetContent());
+  return menuBar && menuBar->IsActive();
 }
 
 bool XULMenubarAccessible::AreItemsOperable() const { return true; }
 
-Accessible* XULMenubarAccessible::CurrentItem() const {
-  nsMenuBarFrame* menuBarFrame = do_QueryFrame(GetFrame());
-  if (menuBarFrame) {
-    nsMenuFrame* menuFrame = menuBarFrame->GetCurrentMenuItem();
-    if (menuFrame) {
-      nsIContent* menuItemNode = menuFrame->GetContent();
-      return mDoc->GetAccessible(menuItemNode);
-    }
+LocalAccessible* XULMenubarAccessible::CurrentItem() const {
+  auto* content = dom::XULMenuParentElement::FromNode(GetContent());
+  MOZ_ASSERT(content);
+  if (!content || !content->GetActiveMenuChild()) {
+    return nullptr;
   }
-  return nullptr;
+  return mDoc->GetAccessible(content->GetActiveMenuChild());
 }
 
-void XULMenubarAccessible::SetCurrentItem(const Accessible* aItem) {
+void XULMenubarAccessible::SetCurrentItem(const LocalAccessible* aItem) {
   NS_ERROR("XULMenubarAccessible::SetCurrentItem not implemented");
 }
