@@ -36,6 +36,9 @@ const { Sanitizer } = ChromeUtils.importESModule(
 const { NewTabUtils } = ChromeUtils.importESModule(
   "resource://gre/modules/NewTabUtils.sys.mjs"
 );
+const { CookieXPCShellUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/CookieXPCShellUtils.sys.mjs"
+);
 
 ExtensionTestUtils.init(this);
 AddonTestUtils.init(this);
@@ -91,7 +94,7 @@ function addTestCookie(isSessionCookie) {
   gCookieCounter++;
   let name = `Cookie name: ${gCookieCounter}`;
 
-  Services.cookies.add(
+  const cv = Services.cookies.add(
     COOKIE_HOST,
     COOKIE_PATH,
     name,
@@ -101,9 +104,10 @@ function addTestCookie(isSessionCookie) {
     isSessionCookie,
     Date.now() / 1000 + 1,
     COOKIE_ORIGIN_ATTRIBUTES,
-    Ci.nsICookie.SAMESITE_NONE,
+    Ci.nsICookie.SAMESITE_UNSET,
     Ci.nsICookie.SCHEME_HTTP
   );
+  Assert.equal(cv.result, Ci.nsICookieValidation.eOK);
 
   return name;
 }
@@ -250,6 +254,18 @@ async function expectNoRegeneration(taskFn, msg) {
   sandbox.restore();
 }
 
+add_setup(() => {
+  CookieXPCShellUtils.createServer({ hosts: ["example.com"] });
+  Services.prefs.setBoolPref("dom.security.https_first", false);
+
+  // Allow all cookies.
+  Services.prefs.setIntPref("network.cookie.cookieBehavior", 0);
+  Services.prefs.setBoolPref(
+    "network.cookieJarSettings.unblocked_for_testing",
+    true
+  );
+});
+
 /**
  * Tests that backup regeneration occurs on the page-removed PlacesObserver
  * event that indicates manual user deletion of a page from history.
@@ -585,6 +601,28 @@ add_task(async function test_cookies_removed() {
   }
 });
 
+/**
+ * Tests that backup regeneration does not occur if a cookie is removed due
+ * to expiry.
+ */
+add_task(async function test_cookies_not_removed_expiry() {
+  await expectNoRegeneration(async () => {
+    const COOKIE = "test=test";
+    await CookieXPCShellUtils.setCookieToDocument(
+      "http://example.com/",
+      COOKIE
+    );
+
+    // Now expire that cookie by using the same value, but setting the expires
+    // directive to sometime wayyyyy in the past.
+    const EXPIRE = `${COOKIE}; expires=Thu, 01 Jan 1970 00:00:00 UTC; path=/`;
+    await CookieXPCShellUtils.setCookieToDocument(
+      "http://example.com/",
+      EXPIRE
+    );
+  }, "Saw no regeneration on cookie expiry.");
+});
+
 /**
  * Tests that backup regeneration occurs when newtab links are blocked.
  */
