@@ -119,6 +119,8 @@ async function withWindowOverflowed(
 
   await ensureMaximizedWindow(win);
 
+  info(`Window maximized`);
+
   // The OverflowableToolbar operates asynchronously at times, so we will
   // poll a widget's overflowedItem attribute to detect whether or not the
   // widgets have finished being moved. We'll use the first widget that
@@ -227,13 +229,16 @@ async function withWindowOverflowed(
   };
   CustomizableUI.addListener(listener);
   // Start all the extensions sequentially.
+  info(`starting extensions...`);
   for (const extension of extensions) {
     await extension.startup();
   }
+  info(`waiting for menu-created messages`);
   await Promise.all([
     extWithMenuBrowserAction.awaitMessage("menu-created"),
     extWithSubMenuBrowserAction.awaitMessage("menu-created"),
   ]);
+  info(`waiting for browser action widgets`);
   await listener.promise;
   CustomizableUI.removeListener(listener);
 
@@ -243,6 +248,7 @@ async function withWindowOverflowed(
     info("Running beforeOverflowed task");
     await beforeOverflowed(extensionIDs);
   } finally {
+    info(`beforeOverflowed finished`);
     // The beforeOverflowed task may have moved some items out from the navbar,
     // so only listen for overflows for items still in there.
     const browserActionIDs = extensionIDs.map(id =>
@@ -262,8 +268,11 @@ async function withWindowOverflowed(
         return this._deferred.promise;
       },
 
-      onWidgetOverflow() {
+      onWidgetOverflow(widget) {
         this._remainingOverflowables--;
+        info(
+          `onWidgetOverflow: ${this._remainingOverflowables} remaining after ${widget.id}`
+        );
         if (!this._remainingOverflowables) {
           this._deferred.resolve();
         }
@@ -271,7 +280,11 @@ async function withWindowOverflowed(
     };
     CustomizableUI.addListener(widgetOverflowListener);
 
-    win.resizeTo(OVERFLOW_WINDOW_WIDTH_PX, win.outerHeight);
+    info(
+      `Resizing to overflow window width (current width: ${win.innerWidth})`
+    );
+    await ensureWindowInnerDimensions(win, OVERFLOW_WINDOW_WIDTH_PX, null);
+    info("Waiting for widget to overflow");
     await widgetOverflowListener.promise;
     CustomizableUI.removeListener(widgetOverflowListener);
 
@@ -292,6 +305,7 @@ async function withWindowOverflowed(
       info("Running whenOverflowed task");
       await whenOverflowed(defaultList, unifiedExtensionList, extensionIDs);
     } finally {
+      info("whenOverflowed finished, maximizing again");
       await ensureMaximizedWindow(win);
 
       // Notably, we don't wait for the nav-bar to not have the "overflowing"
@@ -333,11 +347,7 @@ async function verifyExtensionWidget(widget, win = window) {
     "expected no .toolbarbutton-1 CSS class on the action button in the panel"
   );
 
-  let menuButton = widget.lastElementChild;
-  Assert.ok(
-    menuButton.classList.contains("unified-extensions-item-menu-button"),
-    "expected class on the button"
-  );
+  let menuButton = widget.querySelector(".unified-extensions-item-menu-button");
   ok(
     menuButton.classList.contains("subviewbutton"),
     "expected the .subviewbutton CSS class on the menu button in the panel"
@@ -1033,21 +1043,10 @@ add_task(async function test_overflow_with_a_second_window() {
   );
 
   // Make sure the first window is the active window.
-  let windowActivePromise = new Promise(resolve => {
-    if (Services.focus.activeWindow == win) {
-      resolve();
-    } else {
-      win.addEventListener(
-        "activate",
-        () => {
-          resolve();
-        },
-        { once: true }
-      );
-    }
-  });
-  win.focus();
-  await windowActivePromise;
+  info("second window is maximized, trying to make first window active");
+  await SimpleTest.promiseFocus(win);
+
+  info("first window focused");
 
   let extensionWidgetID;
   let aNode;
