@@ -7,8 +7,6 @@ const { topChromeWindow } = window.browsingContext;
 const lazy = {};
 ChromeUtils.defineESModuleGetters(lazy, {
   GenAI: "resource:///modules/GenAI.sys.mjs",
-  LightweightThemeConsumer:
-    "resource://gre/modules/LightweightThemeConsumer.sys.mjs",
   SpecialMessageActions:
     "resource://messaging-system/lib/SpecialMessageActions.sys.mjs",
 });
@@ -16,6 +14,15 @@ const { XPCOMUtils } = ChromeUtils.importESModule(
   "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
 
+// Define actions for onboarding and chatbot
+const ACTIONS = Object.freeze({
+  CHATBOT_PERSIST: "chatbot:persist",
+  CHATBOT_REVERT: "chatbot:revert",
+  CHATBOT_SELECT: "chatbot:select",
+  CHATBOT_SUPPORT: "chatbot:support",
+  OPEN_URL: "OPEN_URL",
+});
+
 XPCOMUtils.defineLazyPreferenceGetter(
   lazy,
   "providerPref",
@@ -28,6 +35,97 @@ XPCOMUtils.defineLazyPreferenceGetter(
   "shortcutsPref",
   "browser.ml.chat.shortcuts"
 );
+XPCOMUtils.defineLazyPreferenceGetter(
+  lazy,
+  "sidebarRevampPref",
+  "sidebar.revamp"
+);
+XPCOMUtils.defineLazyPreferenceGetter(
+  lazy,
+  "onboardingConfig",
+  "browser.ml.chat.onboarding.config",
+  JSON.stringify({
+    id: "chatbot",
+    template: "multistage",
+    transitions: true,
+    screens: [
+      {
+        id: "chat_pick",
+        content: {
+          fullscreen: true,
+          hide_secondary_section: "responsive",
+          narrow: true,
+          position: "split",
+
+          title: {
+            fontWeight: 400,
+            string_id: "genai-onboarding-header",
+          },
+          cta_paragraph: {
+            text: {
+              string_id: "genai-onboarding-description",
+              string_name: "learn-more",
+            },
+            action: {
+              type: ACTIONS.CHATBOT_SUPPORT,
+            },
+          },
+          above_button_content: [
+            // Placeholder to inject on provider change
+            {
+              text: " ",
+              type: "text",
+            },
+          ],
+          primary_button: {
+            action: {
+              navigate: true,
+              type: ACTIONS.CHATBOT_PERSIST,
+            },
+            label: { string_id: "genai-onboarding-primary" },
+          },
+          additional_button: {
+            action: { dismiss: true, type: ACTIONS.CHATBOT_REVERT },
+            label: { string_id: "genai-onboarding-secondary" },
+            style: "link",
+          },
+          progress_bar: true,
+        },
+      },
+      {
+        id: "chat_suggest",
+        content: {
+          fullscreen: true,
+          hide_secondary_section: "responsive",
+          narrow: true,
+          position: "split",
+
+          title: {
+            fontWeight: 400,
+            string_id: "genai-onboarding-select-header",
+          },
+          subtitle: { string_id: "genai-onboarding-select-description" },
+          above_button_content: [
+            {
+              height: "172px",
+              type: "image",
+              width: "307px",
+            },
+            {
+              text: " ",
+              type: "text",
+            },
+          ],
+          primary_button: {
+            action: { navigate: true },
+            label: { string_id: "genai-onboarding-select-primary" },
+          },
+          progress_bar: true,
+        },
+      },
+    ],
+  })
+);
 
 ChromeUtils.defineLazyGetter(
   lazy,
@@ -62,10 +160,13 @@ function request(url = lazy.providerPref) {
 
 function renderChat() {
   const browser = document.createXULElement("browser");
+  const browserContainer = document.getElementById("browser-container");
   browser.setAttribute("disableglobalhistory", "true");
-  browser.setAttribute("type", "content");
+  browser.setAttribute("maychangeremoteness", "true");
+  browser.setAttribute("nodefaultsrc", "true");
   browser.setAttribute("remote", "true");
-  return document.body.appendChild(browser);
+  browser.setAttribute("type", "content");
+  return browserContainer.appendChild(browser);
 }
 
 async function renderProviders() {
@@ -185,15 +286,19 @@ function renderMore() {
       ],
     ].forEach(([type, l10n, command, checked]) => {
       const item = menu.appendChild(topDoc.createXULElement(type));
-      if (type == "menuitem") {
-        document.l10n.setAttributes(item, ...l10n);
-        item.addEventListener("command", () => {
-          command();
-          Glean.genaiChatbot.sidebarMoreMenuClick.record({
-            action: command.name,
-            provider: providerId,
-          });
+      if (type != "menuitem") {
+        return;
+      }
+      document.l10n.setAttributes(item, ...l10n);
+      item.addEventListener("command", () => {
+        command();
+        Glean.genaiChatbot.sidebarMoreMenuClick.record({
+          action: command.name,
+          provider: providerId,
         });
+      });
+      if (checked !== undefined) {
+        item.setAttribute("type", "checkbox");
         if (checked) {
           item.setAttribute("checked", true);
         }
@@ -228,7 +333,6 @@ addEventListener("change", handleChange);
 // Expose a promise for loading and rendering the chat browser element
 var browserPromise = new Promise((resolve, reject) => {
   addEventListener("load", async () => {
-    new lazy.LightweightThemeConsumer(document);
     try {
       node.chat = renderChat();
       node.provider = await renderProviders();
@@ -240,6 +344,17 @@ var browserPromise = new Promise((resolve, reject) => {
           provider: lazy.GenAI.getProviderId(),
         });
       });
+      document
+        .getElementById("summarize-button")
+        .addEventListener("click", async () => {
+          const badgeKey = "browser.ml.chat.page.footerBadge";
+          const newBadgePref = Services.prefs.getBoolPref(badgeKey);
+
+          if (newBadgePref) {
+            Services.prefs.setBoolPref(badgeKey, false);
+          }
+          await lazy.GenAI.summarizeCurrentPage(topChromeWindow, "footer");
+        });
     } catch (ex) {
       console.error("Failed to render on load", ex);
       reject(ex);
@@ -249,6 +364,7 @@ var browserPromise = new Promise((resolve, reject) => {
       opened: true,
       provider: lazy.GenAI.getProviderId(),
       reason: "load",
+      version: lazy.sidebarRevampPref ? "new" : "old",
     });
   });
 });
@@ -259,6 +375,7 @@ addEventListener("unload", () => {
     opened: false,
     provider: lazy.GenAI.getProviderId(),
     reason: "unload",
+    version: lazy.sidebarRevampPref ? "new" : "old",
   });
 });
 
@@ -268,13 +385,6 @@ addEventListener("unload", () => {
  * @param {number} length optional show fewer screens
  */
 function showOnboarding(length) {
-  const ACTIONS = Object.freeze({
-    OPEN_URL: "OPEN_URL",
-    CHATBOT_SELECT: "chatbot:select",
-    CHATBOT_PERSIST: "chatbot:persist",
-    CHATBOT_REVERT: "chatbot:revert",
-  });
-
   // Insert onboarding container and render with script
   const root = document.createElement("div");
   root.id = "multi-stage-message-root";
@@ -302,109 +412,39 @@ function showOnboarding(length) {
         closeSidebar();
       }
       root.remove();
+
+      // Indicate onboarding finished and allow another
+      showOnboarding.resolve();
+      onboardingPromise = new Promise(resolve => {
+        showOnboarding.resolve = resolve;
+      });
     },
     AWGetFeatureConfig() {
-      return {
-        id: "chatbot",
-        template: "multistage",
-        transitions: true,
-        screens: [
-          {
-            id: "chat_pick",
-            content: {
-              fullscreen: true,
-              hide_secondary_section: "responsive",
-              narrow: true,
-              position: "split",
-
-              title: {
-                fontWeight: 400,
-                string_id: "genai-onboarding-header",
-              },
-              cta_paragraph: {
-                text: {
-                  string_id: "genai-onboarding-description",
-                  string_name: "learn-more",
-                },
-                action: {
-                  data: {
-                    args: lazy.supportLink,
-                    where: "tabshifted",
-                  },
-                  type: ACTIONS.OPEN_URL,
-                },
-              },
-              tiles: {
-                action: { picker: "<event>" },
-                data: [...providerConfigs.values()].map(config => ({
-                  action: { type: ACTIONS.CHATBOT_SELECT, config },
-                  id: config.id,
-                  label: config.name,
-                  tooltip: { string_id: config.tooltipId },
-                })),
-                // Default to nothing selected
-                selected: " ",
-                type: "single-select",
-              },
-              above_button_content: [
-                // Placeholder to inject on provider change
-                {
-                  text: " ",
-                  type: "text",
-                },
-              ],
-              primary_button: {
-                action: {
-                  navigate: true,
-                  type: ACTIONS.CHATBOT_PERSIST,
-                },
-                label: { string_id: "genai-onboarding-primary" },
-              },
-              additional_button: {
-                action: { dismiss: true, type: ACTIONS.CHATBOT_REVERT },
-                label: { string_id: "genai-onboarding-secondary" },
-                style: "link",
-              },
-              progress_bar: true,
-            },
-          },
-          {
-            id: "chat_suggest",
-            content: {
-              fullscreen: true,
-              hide_secondary_section: "responsive",
-              narrow: true,
-              position: "split",
-
-              title: {
-                fontWeight: 400,
-                string_id: "genai-onboarding-select-header",
-              },
-              subtitle: { string_id: "genai-onboarding-select-description" },
-              above_button_content: [
-                {
-                  height: "172px",
-                  type: "image",
-                  width: "307px",
-                },
-                {
-                  text: " ",
-                  type: "text",
-                },
-              ],
-              primary_button: {
-                action: { navigate: true },
-                label: { string_id: "genai-onboarding-select-primary" },
-              },
-              progress_bar: true,
-            },
-          },
-        ].slice(0, length),
+      const onboarding = JSON.parse(lazy.onboardingConfig);
+      const providerTiles = {
+        action: { picker: "<event>" },
+        data: [...providerConfigs.values()].map(config => ({
+          action: { type: ACTIONS.CHATBOT_SELECT, config },
+          id: config.id,
+          label: config.name,
+          tooltip: { string_id: config.tooltipId },
+        })),
+        // Default to nothing selected
+        selected: " ",
+        type: "single-select",
       };
+      // Insert provider tiles on the first screen
+      onboarding.screens[0].content.tiles = providerTiles;
+      // Remove extra screens if any
+      onboarding.screens = onboarding.screens.slice(0, length);
+      return onboarding;
     },
     AWGetInstalledAddons() {},
     AWGetSelectedTheme() {
-      document.querySelector(".primary").disabled = true;
+      const primary = document.querySelector(".primary");
+      if (primary) {
+        primary.disabled = true;
+      }
 
       // Specially handle links to open out of the sidebar
       const handleLink = ev => {
@@ -415,7 +455,7 @@ function showOnboarding(length) {
         }
       };
       const links = document.querySelector(".link-paragraph");
-      links.addEventListener("click", handleLink);
+      links?.addEventListener("click", handleLink);
 
       [...document.querySelectorAll("fieldset label")].forEach(label => {
         // Add content that is hidden with 0 height until selected
@@ -532,7 +572,7 @@ function showOnboarding(length) {
 
           // Update potentially multiple links for the provider
           const links = document.querySelector(".link-paragraph");
-          if (links.dataset.l10nId != config.linksId) {
+          if (links && links.dataset.l10nId != config.linksId) {
             links.innerHTML = "";
             for (let i = 1; i <= 3; i++) {
               const link = links.appendChild(document.createElement("a"));
@@ -545,7 +585,15 @@ function showOnboarding(length) {
 
           break;
         }
+        case ACTIONS.CHATBOT_SUPPORT:
+          openLink(lazy.supportLink);
+          break;
       }
     },
   });
 }
+
+// Expose a promise for onboarding finishing
+var onboardingPromise = new Promise(resolve => {
+  showOnboarding.resolve = resolve;
+});
