@@ -12,10 +12,12 @@
   const lazy = {};
 
   ChromeUtils.defineESModuleGetters(lazy, {
+    BrowserSearchTelemetry:
+      "moz-src:///browser/components/search/BrowserSearchTelemetry.sys.mjs",
     BrowserUtils: "resource://gre/modules/BrowserUtils.sys.mjs",
     FormHistory: "resource://gre/modules/FormHistory.sys.mjs",
     SearchSuggestionController:
-      "resource://gre/modules/SearchSuggestionController.sys.mjs",
+      "moz-src:///toolkit/components/search/SearchSuggestionController.sys.mjs",
   });
 
   /**
@@ -40,7 +42,7 @@
         <html:input class="searchbar-textbox" is="autocomplete-input" type="search" data-l10n-id="searchbar-input" autocompletepopup="PopupSearchAutoComplete" autocompletesearch="search-autocomplete" autocompletesearchparam="searchbar-history" maxrows="10" completeselectedindex="true" minresultsforpopup="0"/>
         <menupopup class="textbox-contextmenu"></menupopup>
         <hbox class="search-go-container" align="center">
-          <image class="search-go-button urlbar-icon" role="button" keyNav="false" hidden="true" onclick="handleSearchCommand(event);" data-l10n-id="searchbar-submit"></image>
+          <image class="search-go-button urlbar-icon" role="button" keyNav="false" hidden="true" data-l10n-id="searchbar-submit"></image>
         </hbox>
       `;
     }
@@ -128,7 +130,7 @@
               this._textbox.popup.updateHeader();
               // Refresh the display (updating icon, etc)
               this.updateDisplay();
-              BrowserSearch.updateOpenSearchBadge();
+              OpenSearchManager.updateOpenSearchBadge(window);
             })
             .catch(status =>
               console.error(
@@ -157,6 +159,10 @@
         },
         { capture: true, once: true }
       );
+
+      this.querySelector(".search-go-button").addEventListener("click", event =>
+        this.handleSearchCommand(event)
+      );
     }
 
     async getEngines() {
@@ -324,9 +330,8 @@
       let selectedIndex = this.telemetrySelectedIndex;
       let isOneOff = false;
 
-      BrowserSearchTelemetry.recordSearchSuggestionSelectionMethod(
+      lazy.BrowserSearchTelemetry.recordSearchSuggestionSelectionMethod(
         aEvent,
-        "searchbar",
         selectedIndex
       );
 
@@ -383,13 +388,6 @@
 
       this.telemetrySelectedIndex = -1;
 
-      BrowserSearchTelemetry.recordSearch(
-        gBrowser.selectedBrowser,
-        engine,
-        "searchbar",
-        details
-      );
-
       // Record when the user uses the search bar
       Services.prefs.setStringPref(
         "browser.search.widget.lastUsed",
@@ -408,6 +406,28 @@
           params[key] = aParams[key];
         }
       }
+
+      if (aWhere == "tab") {
+        gBrowser.tabContainer.addEventListener(
+          "TabOpen",
+          event =>
+            lazy.BrowserSearchTelemetry.recordSearch(
+              event.target.linkedBrowser,
+              engine,
+              "searchbar",
+              details
+            ),
+          { once: true }
+        );
+      } else {
+        lazy.BrowserSearchTelemetry.recordSearch(
+          gBrowser.selectedBrowser,
+          engine,
+          "searchbar",
+          details
+        );
+      }
+
       openTrustedLinkIn(submission.uri.spec, aWhere, params);
     }
 
@@ -499,6 +519,7 @@
         this._needBrowserFocusAtEnterKeyUp = true;
       }
 
+      lazy.BrowserSearchTelemetry.recordSearchForm(engine, "searchbar");
       openTrustedLinkIn(searchForm, where, params);
     }
 
@@ -774,6 +795,7 @@
           } else {
             this.textbox.select();
           }
+          aEvent.preventDefault();
           return true;
         }
         return false;
