@@ -7,6 +7,11 @@
 // This is loaded into chrome windows with the subscript loader. Wrap in
 // a block to prevent accidentally leaking globals onto `window`.
 {
+  const lazy = {};
+  ChromeUtils.defineESModuleGetters(lazy, {
+    TabMetrics: "moz-src:///browser/components/tabbrowser/TabMetrics.sys.mjs",
+  });
+
   class MozTabbrowserTab extends MozElements.MozTab {
     static markup = `
       <stack class="tab-stack" flex="1">
@@ -23,19 +28,14 @@
             <image class="tab-sharing-icon-overlay" role="presentation"/>
             <image class="tab-icon-overlay" role="presentation"/>
           </stack>
+          <html:moz-button type="icon ghost" size="small" class="tab-audio-button" tabindex="-1"></html:moz-button>
           <vbox class="tab-label-container"
-                onoverflow="this.setAttribute('textoverflow', 'true');"
-                onunderflow="this.removeAttribute('textoverflow');"
                 align="start"
                 pack="center"
                 flex="1">
             <label class="tab-text tab-label" role="presentation"/>
             <hbox class="tab-secondary-label">
-              <label class="tab-icon-sound-label tab-icon-sound-playing-label" data-l10n-id="browser-tab-audio-playing2" role="presentation"/>
-              <label class="tab-icon-sound-label tab-icon-sound-muted-label" data-l10n-id="browser-tab-audio-muted2" role="presentation"/>
-              <label class="tab-icon-sound-label tab-icon-sound-blocked-label" data-l10n-id="browser-tab-audio-blocked" role="presentation"/>
               <label class="tab-icon-sound-label tab-icon-sound-pip-label" data-l10n-id="browser-tab-audio-pip" role="presentation"/>
-              <label class="tab-icon-sound-label tab-icon-sound-tooltip-label" role="presentation"/>
             </hbox>
           </vbox>
           <image class="tab-close-button close-icon" role="button" data-l10n-id="tabbrowser-close-tabs-button" data-l10n-args='{"tabCount": 1}' keyNav="false"/>
@@ -54,6 +54,7 @@
       this.addEventListener("mouseup", this);
       this.addEventListener("click", this);
       this.addEventListener("dblclick", this, true);
+      this.addEventListener("animationstart", this);
       this.addEventListener("animationend", this);
       this.addEventListener("focus", this);
       this.addEventListener("AriaFocus", this);
@@ -85,32 +86,42 @@
         ".tab-group-line": "selected=visuallyselected,multiselected",
         ".tab-loading-burst": "pinned,bursting,notselectedsinceload",
         ".tab-content":
-          "pinned,selected=visuallyselected,titlechanged,attention",
+          "pinned,selected=visuallyselected,multiselected,titlechanged,attention",
         ".tab-icon-stack":
-          "sharing,pictureinpicture,crashed,busy,soundplaying,soundplaying-scheduledremoval,pinned,muted,blocked,selected=visuallyselected,activemedia-blocked,indicator-replaces-favicon",
+          "sharing,pictureinpicture,crashed,busy,soundplaying,soundplaying-scheduledremoval,pinned,muted,blocked,selected=visuallyselected,activemedia-blocked",
         ".tab-throbber":
           "fadein,pinned,busy,progress,selected=visuallyselected",
         ".tab-icon-pending":
           "fadein,pinned,busy,progress,selected=visuallyselected,pendingicon",
         ".tab-icon-image":
-          "src=image,triggeringprincipal=iconloadingprincipal,requestcontextid,fadein,pinned,selected=visuallyselected,busy,crashed,sharing,pictureinpicture",
+          "src=image,triggeringprincipal=iconloadingprincipal,requestcontextid,fadein,pinned,selected=visuallyselected,busy,crashed,sharing,pictureinpicture,pending,discarded",
         ".tab-sharing-icon-overlay": "sharing,selected=visuallyselected,pinned",
         ".tab-icon-overlay":
-          "sharing,pictureinpicture,crashed,busy,soundplaying,soundplaying-scheduledremoval,pinned,muted,blocked,selected=visuallyselected,activemedia-blocked,indicator-replaces-favicon",
+          "sharing,pictureinpicture,crashed,busy,soundplaying,soundplaying-scheduledremoval,pinned,muted,blocked,selected=visuallyselected,activemedia-blocked",
+        ".tab-audio-button":
+          "crashed,soundplaying,soundplaying-scheduledremoval,pinned,muted,activemedia-blocked",
         ".tab-label-container":
           "pinned,selected=visuallyselected,labeldirection",
         ".tab-label":
           "text=label,accesskey,fadein,pinned,selected=visuallyselected,attention",
         ".tab-label-container .tab-secondary-label":
-          "soundplaying,soundplaying-scheduledremoval,pinned,muted,blocked,selected=visuallyselected,activemedia-blocked,pictureinpicture",
+          "pinned,blocked,selected=visuallyselected,pictureinpicture",
         ".tab-close-button": "fadein,pinned,selected=visuallyselected",
       };
     }
 
+    #lastGroup;
     connectedCallback() {
+      this.#updateOnTabGrouped();
+      this.#lastGroup = this.group;
+
       this.initialize();
     }
 
+    disconnectedCallback() {
+      this.#updateOnTabUngrouped();
+    }
+
     initialize() {
       if (this._initialized) {
         return;
@@ -125,10 +136,34 @@
       if (!("_lastAccessed" in this)) {
         this.updateLastAccessed();
       }
+
+      let labelContainer = this.querySelector(".tab-label-container");
+      labelContainer.addEventListener("overflow", this);
+      labelContainer.addEventListener("underflow", this);
+
+      // Tabs in the tab strip default to being at the top level (level 1)
+      // Tabs in tab groups are one level down (level 2); this tab will
+      // update its value when it moves in and out of tab groups.
+      this.setAttribute("aria-level", 1);
     }
 
+    #elementIndex;
+    get elementIndex() {
+      if (!this.visible) {
+        throw new Error("Tab is not visible, so does not have an elementIndex");
+      }
+      // Make sure the index is up to date.
+      this.container.ariaFocusableItems;
+      return this.#elementIndex;
+    }
+
+    set elementIndex(index) {
+      this.#elementIndex = index;
+    }
+
+    #owner;
     get owner() {
-      let owner = this._owner?.deref();
+      let owner = this.#owner?.deref();
       if (owner && !owner.closing) {
         return owner;
       }
@@ -136,11 +171,7 @@
     }
 
     set owner(owner) {
-      if (owner) {
-        this._owner = new WeakRef(owner);
-      } else {
-        this._owner = null;
-      }
+      this.#owner = owner ? new WeakRef(owner) : null;
     }
 
     get container() {
@@ -156,15 +187,6 @@
       gBrowser._tabAttrModified(this, ["attention"]);
     }
 
-    set undiscardable(val) {
-      if (val == this.hasAttribute("undiscardable")) {
-        return;
-      }
-
-      this.toggleAttribute("undiscardable", val);
-      gBrowser._tabAttrModified(this, ["undiscardable"]);
-    }
-
     set _visuallySelected(val) {
       if (val == this.hasAttribute("visuallyselected")) {
         return;
@@ -195,15 +217,16 @@
       return this.hasAttribute("pinned");
     }
 
-    get visible() {
+    get isOpen() {
       return (
-        this.isConnected &&
-        !this.hidden &&
-        !this.closing &&
-        !this.group?.collapsed
+        this.isConnected && !this.closing && this != FirefoxViewHandler.tab
       );
     }
 
+    get visible() {
+      return this.isOpen && !this.hidden && !this.group?.collapsed;
+    }
+
     get hidden() {
       // This getter makes `hidden` read-only
       return super.hidden;
@@ -239,6 +262,15 @@
       return this.hasAttribute("undiscardable");
     }
 
+    set undiscardable(val) {
+      if (val == this.hasAttribute("undiscardable")) {
+        return;
+      }
+
+      this.toggleAttribute("undiscardable", val);
+      gBrowser._tabAttrModified(this, ["undiscardable"]);
+    }
+
     get isEmpty() {
       // Determines if a tab is "empty", usually used in the context of determining
       // if it's ok to close the tab.
@@ -307,10 +339,18 @@
       return this.overlayIcon?.matches(":hover");
     }
 
+    get _overAudioButton() {
+      return this.audioButton?.matches(":hover");
+    }
+
     get overlayIcon() {
       return this.querySelector(".tab-icon-overlay");
     }
 
+    get audioButton() {
+      return this.querySelector(".tab-audio-button");
+    }
+
     get throbber() {
       return this.querySelector(".tab-throbber");
     }
@@ -357,9 +397,9 @@
       }
 
       const diff_in_msec = Date.now() - this._lastUnloaded;
-      Services.telemetry
-        .getHistogramById("TAB_UNLOAD_TO_RELOAD")
-        .add(diff_in_msec / 1000);
+      Glean.browserEngagement.tabUnloadToReload.accumulateSingleSample(
+        diff_in_msec / 1000
+      );
       Glean.browserEngagement.tabReloadCount.add(1);
       delete this._lastUnloaded;
     }
@@ -368,24 +408,6 @@
       if (event.target.classList.contains("tab-close-button")) {
         this.mOverCloseButton = true;
       }
-      if (this._overPlayingIcon) {
-        const selectedTabs = gBrowser.selectedTabs;
-        const contextTabInSelection = selectedTabs.includes(this);
-        const affectedTabsLength = contextTabInSelection
-          ? selectedTabs.length
-          : 1;
-        let stringID;
-        if (this.hasAttribute("activemedia-blocked")) {
-          stringID = "browser-tab-unblock";
-        } else {
-          stringID = this.linkedBrowser.audioMuted
-            ? "browser-tab-unmute"
-            : "browser-tab-mute";
-        }
-        this.setSecondaryTabTooltipLabel(stringID, {
-          count: affectedTabsLength,
-        });
-      }
 
       if (!this.visible) {
         return;
@@ -406,9 +428,6 @@
       if (event.target.classList.contains("tab-close-button")) {
         this.mOverCloseButton = false;
       }
-      if (event.target == this.overlayIcon) {
-        this.setSecondaryTabTooltipLabel(null);
-      }
 
       // If the new target is not part of this tab then this is a mouseleave event.
       if (!this.contains(event.relatedTarget)) {
@@ -448,7 +467,8 @@
         this.style.MozUserFocus = "ignore";
       } else if (
         event.target.classList.contains("tab-close-button") ||
-        event.target.classList.contains("tab-icon-overlay")
+        event.target.classList.contains("tab-icon-overlay") ||
+        event.target.classList.contains("tab-audio-button")
       ) {
         eventMaySelectTab = false;
       }
@@ -513,14 +533,18 @@
       if (
         gBrowser.multiSelectedTabsCount > 0 &&
         !event.target.classList.contains("tab-close-button") &&
-        !event.target.classList.contains("tab-icon-overlay")
+        !event.target.classList.contains("tab-icon-overlay") &&
+        !event.target.classList.contains("tab-audio-button")
       ) {
         // Tabs were previously multi-selected and user clicks on a tab
         // without holding Ctrl/Cmd Key
         gBrowser.clearMultiSelectedTabs();
       }
 
-      if (event.target.classList.contains("tab-icon-overlay")) {
+      if (
+        event.target.classList.contains("tab-icon-overlay") ||
+        event.target.classList.contains("tab-audio-button")
+      ) {
         if (this.activeMediaBlocked) {
           if (this.multiselected) {
             gBrowser.resumeDelayedMediaOnMultiSelectedTabs(this);
@@ -539,11 +563,18 @@
 
       if (event.target.classList.contains("tab-close-button")) {
         if (this.multiselected) {
-          gBrowser.removeMultiSelectedTabs();
+          gBrowser.removeMultiSelectedTabs(
+            lazy.TabMetrics.userTriggeredContext(
+              lazy.TabMetrics.METRIC_SOURCE.TAB_STRIP
+            )
+          );
         } else {
           gBrowser.removeTab(this, {
             animate: true,
             triggeringEvent: event,
+            ...lazy.TabMetrics.userTriggeredContext(
+              lazy.TabMetrics.METRIC_SOURCE.TAB_STRIP
+            ),
           });
         }
         // This enables double-click protection for the tab container
@@ -576,6 +607,21 @@
       }
     }
 
+    on_animationstart(event) {
+      if (!event.animationName.startsWith("tab-throbber-animation")) {
+        return;
+      }
+      // The animation is on a pseudo-element so we need to use `subtree: true`
+      // to get our hands on it.
+      for (let animation of event.target.getAnimations({ subtree: true })) {
+        if (animation.animationName === event.animationName) {
+          // Ensure all tab throbber animations are synchronized by sharing an
+          // start time.
+          animation.startTime = 0;
+        }
+      }
+    }
+
     on_animationend(event) {
       if (event.target.classList.contains("tab-loading-burst")) {
         this.removeAttribute("bursting");
@@ -608,27 +654,6 @@
       this.dispatchEvent(new CustomEvent("TabHoverEnd", { bubbles: true }));
     }
 
-    setSecondaryTabTooltipLabel(l10nID, l10nArgs) {
-      this.querySelector(".tab-secondary-label").toggleAttribute(
-        "showtooltip",
-        l10nID
-      );
-
-      const tooltipEl = this.querySelector(".tab-icon-sound-tooltip-label");
-
-      if (l10nArgs) {
-        tooltipEl.setAttribute("data-l10n-args", JSON.stringify(l10nArgs));
-      } else {
-        tooltipEl.removeAttribute("data-l10n-args");
-      }
-      if (l10nID) {
-        tooltipEl.setAttribute("data-l10n-id", l10nID);
-      } else {
-        tooltipEl.removeAttribute("data-l10n-id");
-      }
-      // TODO(Itiel): Maybe simplify this when bug 1830989 lands
-    }
-
     resumeDelayedMedia() {
       if (this.activeMediaBlocked) {
         this.removeAttribute("activemedia-blocked");
@@ -693,6 +718,53 @@
     on_AriaFocus() {
       this.updateA11yDescription();
     }
+
+    on_overflow(event) {
+      event.currentTarget.toggleAttribute("textoverflow", true);
+    }
+
+    on_underflow(event) {
+      event.currentTarget.removeAttribute("textoverflow");
+    }
+
+    #updateOnTabGrouped() {
+      if (this.group && this.#lastGroup != this.group) {
+        // Trigger TabGrouped on the tab group, not the tab itself. This is a
+        // bit unorthodox, but fixes bug1964152 where tab group events are not
+        // fired correctly when tabs change windows (because the tab is
+        // detached from the DOM at time of the event).
+        this.group.dispatchEvent(
+          new CustomEvent("TabGrouped", {
+            bubbles: true,
+            detail: this,
+          })
+        );
+        this.setAttribute("aria-level", 2);
+      }
+    }
+
+    #updateOnTabUngrouped() {
+      if (this.#lastGroup && this.#lastGroup != this.group) {
+        // Trigger TabUngrouped on the tab group, not the tab itself. This is a
+        // bit unorthodox, but fixes bug1964152 where tab group events are not
+        // fired correctly when tabs change windows (because the tab is
+        // detached from the DOM at time of the event).
+        this.#lastGroup.dispatchEvent(
+          new CustomEvent("TabUngrouped", {
+            bubbles: true,
+            detail: this,
+          })
+        );
+        // Tab could have moved to be ungrouped (level 1)
+        // or to a different group (level 2).
+        this.setAttribute("aria-level", this.group ? 2 : 1);
+        // `posinset` and `setsize` only need to be set explicitly
+        // on grouped tabs so that a11y tools can tell users that a
+        // given tab is "2 of 7" in the group, for example.
+        this.removeAttribute("aria-posinset");
+        this.removeAttribute("aria-setsize");
+      }
+    }
   }
 
   customElements.define("tabbrowser-tab", MozTabbrowserTab, {
