@@ -13,7 +13,7 @@
 ChromeUtils.defineESModuleGetters(this, {
   UrlbarProviderClipboard:
     "resource:///modules/UrlbarProviderClipboard.sys.mjs",
-  SearchUtils: "resource://gre/modules/SearchUtils.sys.mjs",
+  SearchUtils: "moz-src:///toolkit/components/search/SearchUtils.sys.mjs",
 });
 
 // This test has many subtests and can time out in verify mode.
@@ -457,52 +457,6 @@ add_task(async function selected_result_unit() {
   await SpecialPowers.popPrefEnv();
 });
 
-add_task(async function selected_result_rs_adm_sponsored() {
-  const cleanupQuickSuggest = await ensureQuickSuggestInit({
-    prefs: [["quicksuggest.rustEnabled", false]],
-  });
-
-  await doTest(async () => {
-    await openPopup("amp");
-    await selectRowByURL("https://example.com/amp");
-    await doEnter();
-
-    assertEngagementTelemetry([
-      {
-        selected_result: "rs_adm_sponsored",
-        selected_position: 2,
-        provider: "UrlbarProviderQuickSuggest",
-        results: "search_engine,rs_adm_sponsored",
-      },
-    ]);
-  });
-
-  await cleanupQuickSuggest();
-});
-
-add_task(async function selected_result_rs_adm_nonsponsored() {
-  const cleanupQuickSuggest = await ensureQuickSuggestInit({
-    prefs: [["quicksuggest.rustEnabled", false]],
-  });
-
-  await doTest(async () => {
-    await openPopup("wikipedia");
-    await selectRowByURL("https://example.com/wikipedia");
-    await doEnter();
-
-    assertEngagementTelemetry([
-      {
-        selected_result: "rs_adm_nonsponsored",
-        selected_position: 2,
-        provider: "UrlbarProviderQuickSuggest",
-        results: "search_engine,rs_adm_nonsponsored",
-      },
-    ]);
-  });
-
-  await cleanupQuickSuggest();
-});
-
 add_task(async function selected_result_input_field() {
   const expected = [
     {
@@ -878,9 +832,7 @@ add_task(async function selected_result_addons() {
 });
 
 add_task(async function selected_result_rust_adm_sponsored() {
-  const cleanupQuickSuggest = await ensureQuickSuggestInit({
-    prefs: [["quicksuggest.rustEnabled", true]],
-  });
+  const cleanupQuickSuggest = await ensureQuickSuggestInit();
 
   await doTest(async () => {
     await openPopup("amp");
@@ -901,9 +853,7 @@ add_task(async function selected_result_rust_adm_sponsored() {
 });
 
 add_task(async function selected_result_rust_adm_nonsponsored() {
-  const cleanupQuickSuggest = await ensureQuickSuggestInit({
-    prefs: [["quicksuggest.rustEnabled", true]],
-  });
+  const cleanupQuickSuggest = await ensureQuickSuggestInit();
 
   await doTest(async () => {
     await openPopup("wikipedia");
@@ -943,9 +893,47 @@ add_task(async function selected_result_action() {
         selected_position: 2,
         provider: "UrlbarProviderGlobalActions",
         results: "search_engine,action",
-        actions: "none",
+        actions: "none,settings",
       },
     ]);
   });
   await SpecialPowers.popPrefEnv();
 });
+
+add_task(async function selected_result_semantic() {
+  const historyUrl = "https://www.example.com/semantic/";
+  let defaultEngine = await Services.search.getDefault();
+  const searchUrl = defaultEngine.getSubmission("semantic", null, "keyword").uri
+    .spec;
+  await doTestWithSemantic(
+    [
+      {
+        id: 1,
+        title: "Test Page",
+        url: historyUrl,
+        frecency: 100,
+      },
+      {
+        id: 2,
+        title: "Test SERP",
+        url: searchUrl,
+        frecency: 10,
+      },
+    ],
+    async () => {
+      // Must be longer than `browser.urlbar.suggest.semanticHistory.minLength`.
+      await openPopup("testing semantic");
+      await selectRowByURL(historyUrl);
+      await doEnter();
+
+      assertEngagementTelemetry([
+        {
+          selected_result: "history_semantic",
+          selected_position: 2,
+          provider: "SemanticHistorySearch",
+          results: "search_engine,history_semantic,history_semantic_serp",
+        },
+      ]);
+    }
+  );
+});
