@@ -108,10 +108,6 @@ add_task(function makeResultGroups_true() {
                   availableSpan: 3,
                   group: UrlbarUtils.RESULT_GROUP.INPUT_HISTORY,
                 },
-                {
-                  availableSpan: 2,
-                  group: UrlbarUtils.RESULT_GROUP.HISTORY_SEMANTIC,
-                },
                 {
                   flexChildren: true,
                   children: [
@@ -122,6 +118,7 @@ add_task(function makeResultGroups_true() {
                     {
                       flex: 2,
                       group: UrlbarUtils.RESULT_GROUP.GENERAL,
+                      orderBy: "frecency",
                     },
                     {
                       flex: 2,
@@ -189,10 +186,6 @@ add_task(function makeResultGroups_false() {
                   availableSpan: 3,
                   group: UrlbarUtils.RESULT_GROUP.INPUT_HISTORY,
                 },
-                {
-                  availableSpan: 2,
-                  group: UrlbarUtils.RESULT_GROUP.HISTORY_SEMANTIC,
-                },
                 {
                   flexChildren: true,
                   children: [
@@ -203,6 +196,7 @@ add_task(function makeResultGroups_false() {
                     {
                       flex: 2,
                       group: UrlbarUtils.RESULT_GROUP.GENERAL,
+                      orderBy: "frecency",
                     },
                     {
                       flex: 2,
