@@ -3,9 +3,6 @@
 
 "use strict";
 
-/* eslint-env browser */
-/* eslint no-unused-vars: [2, {"vars": "local"}] */
-
 // Load the shared-head file first.
 Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/shared/test/shared-head.js",
@@ -21,7 +18,7 @@ Services.scriptloader.loadSubScript(
 );
 
 // Make sure the ADB addon is removed and ADB is stopped when the test ends.
-registerCleanupFunction(async function() {
+registerCleanupFunction(async function () {
   // Reset the selected tool in case we opened about:devtools-toolbox to
   // avoid side effects between tests.
   Services.prefs.clearUserPref("devtools.toolbox.selectedTool");
@@ -97,13 +94,21 @@ async function openAboutDevtoolsToolbox(
   ok(inspectButton, `Inspect button for ${targetText} appeared`);
   inspectButton.click();
   const onToolboxReady = gDevTools.once("toolbox-ready");
-  await Promise.all([
-    waitForAboutDebuggingRequests(win.AboutDebugging.store),
-    shouldWaitToolboxReady ? onToolboxReady : Promise.resolve(),
-  ]);
 
-  // WebExtension open a toolbox in a dedicated window
-  if (isWebExtension) {
+  info("Wait for about debugging requests to settle");
+  await waitForAboutDebuggingRequests(win.AboutDebugging.store);
+
+  if (shouldWaitToolboxReady) {
+    info("Wait for onToolboxReady");
+    await onToolboxReady;
+  }
+
+  const { runtimes } = win.AboutDebugging.store.getState();
+  const isOnThisFirefox = runtimes.selectedRuntimeId === "this-firefox";
+  const isLocalWebExtension = isWebExtension && isOnThisFirefox;
+
+  // Local WebExtension toolboxes open in a dedicated window
+  if (isLocalWebExtension) {
     const toolbox = await onToolboxReady;
     // For some reason the test helpers prevents the toolbox from being automatically focused on opening,
     // whereas it is IRL.
@@ -128,12 +133,14 @@ async function openAboutDevtoolsToolbox(
     };
   }
 
+  info("Wait until a new tab is opened");
   await waitUntil(() => tab.nextElementSibling);
 
   info("Wait for about:devtools-toolbox tab will be selected");
   const devtoolsTab = tab.nextElementSibling;
   await waitUntil(() => gBrowser.selectedTab === devtoolsTab);
   const devtoolsBrowser = gBrowser.selectedBrowser;
+  info("Wait for about:devtools-toolbox tab to have the expected URL");
   await waitUntil(() =>
     devtoolsBrowser.contentWindow.location.href.startsWith(
       "about:devtools-toolbox?"
@@ -164,7 +171,11 @@ async function closeAboutDevtoolsToolbox(
   const devtoolsBrowser = devtoolsTab.linkedBrowser;
   const devtoolsWindow = devtoolsBrowser.contentWindow;
   const toolbox = getToolbox(devtoolsWindow);
-  await toolbox.commands.client.waitForRequestsToSettle();
+
+  info("Wait for requests to settle");
+  await toolbox.commands.client.waitForRequestsToSettle({
+    ignoreOrphanedFronts: true,
+  });
 
   info("Close about:devtools-toolbox page");
   const onToolboxDestroyed = gDevTools.once("toolbox-destroyed");
@@ -353,6 +364,13 @@ async function connectToRuntime(deviceName, document) {
   await waitUntil(() => !sidebarItem.querySelector(".qa-connect-button"));
 }
 
+async function waitForRuntimePage(name, document) {
+  await waitUntil(() => {
+    const runtimeInfo = document.querySelector(".qa-runtime-name");
+    return runtimeInfo && runtimeInfo.textContent.includes(name);
+  });
+}
+
 async function selectRuntime(deviceName, name, document) {
   const sidebarItem = findSidebarItemByText(deviceName, document);
   const store = document.defaultView.AboutDebugging.store;
@@ -360,10 +378,7 @@ async function selectRuntime(deviceName, name, document) {
 
   sidebarItem.querySelector(".qa-sidebar-link").click();
 
-  await waitUntil(() => {
-    const runtimeInfo = document.querySelector(".qa-runtime-name");
-    return runtimeInfo && runtimeInfo.textContent.includes(name);
-  });
+  await waitForRuntimePage(name, document);
 
   info("Wait for SELECT_PAGE_SUCCESS to be dispatched");
   await onSelectPageSuccess;
@@ -501,5 +516,66 @@ function clickOnAddonWidget(addonId) {
   ok(browserActionEl, "Got the browserAction button from the browser UI");
 
   info("Show the web extension popup");
-  browserActionEl.firstElementChild.click();
+  browserActionEl
+    .querySelector(".unified-extensions-item-action-button")
+    .click();
 }
+
+// Create basic addon data as the DevToolsClient would return it.
+function createAddonData({
+  id,
+  name,
+  isSystem = false,
+  hidden = false,
+  temporary = false,
+}) {
+  return {
+    actor: `actorid-${id}`,
+    hidden,
+    iconURL: `moz-extension://${id}/icon-url.png`,
+    id,
+    manifestURL: `moz-extension://${id}/manifest-url.json`,
+    name,
+    isSystem,
+    temporarilyInstalled: temporary,
+    debuggable: true,
+  };
+}
+
+async function connectToLocalFirefox({ runtimeId, runtimeName, deviceName }) {
+  // This is a client to the current Firefox.
+  const clientWrapper = await createLocalClientWrapper();
+
+  // enable USB devices mocks
+  const mocks = new Mocks();
+  const usbClient = mocks.createUSBRuntime(runtimeId, {
+    deviceName,
+    name: runtimeName,
+    clientWrapper,
+  });
+
+  // Wrap a disconnect helper for convenience for the caller.
+  const disconnect = doc =>
+    disconnectFromLocalFirefox({
+      doc,
+      runtimeId,
+      deviceName,
+      mocks,
+    });
+
+  return { disconnect, mocks, usbClient };
+}
+/* exported connectToLocalFirefox */
+
+async function disconnectFromLocalFirefox({
+  doc,
+  mocks,
+  runtimeId,
+  deviceName,
+}) {
+  info("Remove USB runtime");
+  mocks.removeUSBRuntime(runtimeId);
+  mocks.emitUSBUpdate();
+  await waitUntilUsbDeviceIsUnplugged(deviceName, doc);
+}
+/* exported disconnectFromLocalFirefox */
