@@ -11,8 +11,6 @@ import { connect } from "devtools/client/shared/vendor/react-redux";
 
 import { getLineText, isLineBlackboxed } from "./../../utils/source";
 import { createLocation } from "./../../utils/location";
-import { getIndentation } from "../../utils/indentation";
-import { features } from "../../utils/prefs";
 import { markerTypes } from "../../constants";
 import { asSettled, isFulfilled, isRejected } from "../../utils/async-value";
 
@@ -46,35 +44,20 @@ import Breakpoints from "./Breakpoints";
 import ColumnBreakpoints from "./ColumnBreakpoints";
 import DebugLine from "./DebugLine";
 import HighlightLine from "./HighlightLine";
-import EmptyLines from "./EmptyLines";
 import ConditionalPanel from "./ConditionalPanel";
 import InlinePreviews from "./InlinePreviews";
 import Exceptions from "./Exceptions";
-import BlackboxLines from "./BlackboxLines";
 
 import {
   fromEditorLine,
-  showSourceText,
-  setDocument,
-  resetLineNumberFormat,
   getEditor,
   removeEditor,
-  lineAtHeight,
   toSourceLine,
-  getDocument,
   toEditorPosition,
-  getSourceLocationFromMouseEvent,
-  hasDocument,
   onMouseOver,
-  startOperation,
-  endOperation,
 } from "../../utils/editor/index";
 
-import {
-  resizeToggleButton,
-  getLineNumberWidth,
-  resizeBreakpointGutter,
-} from "../../utils/ui";
+import { updateEditorSizeCssVariables } from "../../utils/ui";
 
 const { debounce } = require("resource://devtools/shared/debounce.js");
 const classnames = require("resource://devtools/client/shared/classnames.js");
@@ -149,27 +132,7 @@ class Editor extends PureComponent {
       editor = this.setupEditor();
     }
 
-    if (!features.codemirrorNext) {
-      const shouldUpdateSize =
-        nextProps.startPanelSize !== this.props.startPanelSize ||
-        nextProps.endPanelSize !== this.props.endPanelSize;
-
-      startOperation();
-      if (shouldUpdateSize) {
-        editor.codeMirror.setSize();
-      }
-      this.setTextContent(nextProps, editor, prevEditor);
-      endOperation();
-
-      if (this.props.selectedSource != nextProps.selectedSource) {
-        this.props.updateViewport();
-        resizeBreakpointGutter(editor.codeMirror);
-        resizeToggleButton(getLineNumberWidth(editor.codeMirror));
-      }
-    } else {
-      // For codemirror 6
-      this.setTextContent(nextProps, editor, prevEditor);
-    }
+    this.setTextContent(nextProps, editor, prevEditor);
   }
 
   async setTextContent(nextProps, editor, prevEditor) {
@@ -199,20 +162,14 @@ class Editor extends PureComponent {
     // Also if scope mapping is on, the babel parser worker
     // will be used instead (for scope mapping) as the preview data relies
     // on it for original variable mapping.
-    if (
-      nextProps.isPaused &&
-      features.codemirrorNext &&
-      !nextProps.isOriginalSourceAndMapScopesEnabled
-    ) {
+    if (nextProps.isPaused && !nextProps.isOriginalSourceAndMapScopesEnabled) {
       this.props.setInScopeLines(editor);
     }
   }
 
   onEditorUpdated = viewUpdate => {
     if (viewUpdate.docChanged || viewUpdate.geometryChanged) {
-      resizeToggleButton(
-        viewUpdate.view.dom.querySelector(".cm-gutters").clientWidth
-      );
+      updateEditorSizeCssVariables(viewUpdate.view.dom);
       this.props.updateViewport();
     } else if (viewUpdate.selectionSet) {
       this.onCursorChange();
@@ -231,63 +188,32 @@ class Editor extends PureComponent {
       editor.appendToLocalElement(mountEl);
     }
 
-    if (!features.codemirrorNext) {
-      const { codeMirror } = editor;
+    editor.setUpdateListener(this.onEditorUpdated);
+    editor.setGutterEventListeners({
+      click: (event, cm, line) => {
+        // Ignore clicks on the code folding button
+        if (
+          event.target.className.includes("cm6-dt-foldgutter__toggle-button")
+        ) {
+          return;
+        }
+        // Clicking any where on the fold gutter (except on a code folding button)
+        // should toggle the breakpoint for this line, if possible.
+        if (event.target.className.includes("cm-foldGutter")) {
+          this.props.toggleBreakpointAtLine(line);
+          return;
+        }
+        this.onGutterClick(cm, line, null, event);
+      },
+      contextmenu: (event, cm, line) => this.openMenu(event, line),
+    });
+    editor.addEditorDOMEventListeners({
+      click: (event, cm, line, column) => this.onClick(event, line, column),
+      contextmenu: (event, cm, line, column) =>
+        this.openMenu(event, line, column),
+      mouseover: onMouseOver(editor),
+    });
 
-      this.abortController = new window.AbortController();
-
-      // CodeMirror refreshes its internal state on window resize, but we need to also
-      // refresh it when the side panels are resized.
-      // We could have a ResizeObserver instead, but we wouldn't be able to differentiate
-      // between window resize and side panel resize and as a result, might refresh
-      // codeMirror twice, which is wasteful.
-      window.document
-        .querySelector(".editor-pane")
-        .addEventListener("resizeend", () => codeMirror.refresh(), {
-          signal: this.abortController.signal,
-        });
-
-      codeMirror.on("gutterClick", this.onGutterClick);
-      codeMirror.on("cursorActivity", this.onCursorChange);
-
-      const codeMirrorWrapper = codeMirror.getWrapperElement();
-      // Set code editor wrapper to be focusable
-      codeMirrorWrapper.tabIndex = 0;
-      codeMirrorWrapper.addEventListener("click", e => this.onClick(e));
-      codeMirrorWrapper.addEventListener("mouseover", onMouseOver(editor));
-      codeMirrorWrapper.addEventListener("contextmenu", event =>
-        this.openMenu(event)
-      );
-
-      codeMirror.on("scroll", this.onEditorScroll);
-      this.onEditorScroll();
-    } else {
-      editor.setUpdateListener(this.onEditorUpdated);
-      editor.setGutterEventListeners({
-        click: (event, cm, line) => {
-          // Ignore clicks on the code folding button
-          if (
-            event.target.className.includes("cm6-dt-foldgutter__toggle-button")
-          ) {
-            return;
-          }
-          // Clicking any where on the fold gutter (except on a code folding button)
-          // should toggle the breakpoint for this line, if possible.
-          if (event.target.className.includes("cm-foldGutter")) {
-            this.props.toggleBreakpointAtLine(line);
-            return;
-          }
-          this.onGutterClick(cm, line, null, event);
-        },
-        contextmenu: (event, cm, line) => this.openMenu(event, line),
-      });
-      editor.addEditorDOMEventListeners({
-        click: (event, cm, line, column) => this.onClick(event, line, column),
-        contextmenu: (event, cm, line, column) =>
-          this.openMenu(event, line, column),
-        mouseover: onMouseOver(editor),
-      });
-    }
     this.setState({ editor });
     // Used for tests
     Object.defineProperty(window, "codeMirrorSourceEditorTestInstance", {
@@ -339,68 +265,65 @@ class Editor extends PureComponent {
       return;
     }
 
-    // Sets the breakables lines for codemirror 6
-    if (features.codemirrorNext) {
-      const shouldUpdateBreakableLines =
-        prevProps.breakableLines.size !== this.props.breakableLines.size ||
-        prevProps.selectedSource?.id !== selectedSource.id ||
-        // Make sure we update after the editor has loaded
-        (!prevState.editor && !!editor);
-
-      if (shouldUpdateBreakableLines) {
-        editor.setLineGutterMarkers([
-          {
-            id: markerTypes.EMPTY_LINE_MARKER,
-            lineClassName: "empty-line",
-            condition: line => {
-              const lineNumber = fromEditorLine(selectedSource, line);
-              return !breakableLines.has(lineNumber);
-            },
-          },
-        ]);
-      }
+    const shouldUpdateBreakableLines =
+      prevProps.breakableLines.size !== this.props.breakableLines.size ||
+      prevProps.selectedSource?.id !== selectedSource.id ||
+      // Make sure we update after the editor has loaded
+      (!prevState.editor && !!editor);
 
+    if (shouldUpdateBreakableLines) {
       editor.setLineGutterMarkers([
         {
-          id: markerTypes.BLACKBOX_LINE_GUTTER_MARKER,
-          lineClassName: "blackboxed-line",
+          id: markerTypes.EMPTY_LINE_MARKER,
+          lineClassName: "empty-line",
           condition: line => {
             const lineNumber = fromEditorLine(selectedSource, line);
-            return isLineBlackboxed(
-              blackboxedRanges[selectedSource.url],
-              lineNumber,
-              isSourceOnIgnoreList
-            );
+            return !breakableLines.has(lineNumber);
           },
         },
       ]);
+    }
 
-      if (
-        prevProps.selectedSource?.id !== selectedSource.id ||
-        prevProps.blackboxedRanges[selectedSource.url]?.length !==
-          blackboxedRanges[selectedSource.url]?.length ||
-        (!prevState.editor && !!editor)
-      ) {
-        if (blackboxedRanges[selectedSource.url] == undefined) {
-          editor.removeLineContentMarker(markerTypes.BLACKBOX_LINE_MARKER);
-          return;
-        }
+    editor.setLineGutterMarkers([
+      {
+        id: markerTypes.BLACKBOX_LINE_GUTTER_MARKER,
+        lineClassName: "blackboxed-line",
+        condition: line => {
+          const lineNumber = fromEditorLine(selectedSource, line);
+          return isLineBlackboxed(
+            blackboxedRanges[selectedSource.url],
+            lineNumber,
+            isSourceOnIgnoreList
+          );
+        },
+      },
+    ]);
 
-        const lines = [];
-        for (const range of blackboxedRanges[selectedSource.url]) {
-          for (let line = range.start.line; line <= range.end.line; line++) {
-            lines.push({ line });
-          }
-        }
-
-        editor.setLineContentMarker({
-          id: markerTypes.BLACKBOX_LINE_MARKER,
-          lineClassName: "blackboxed-line",
-          // If the the whole source is blackboxed, lets just mark all positions.
-          shouldMarkAllLines: !blackboxedRanges[selectedSource.url].length,
-          lines,
-        });
+    if (
+      prevProps.selectedSource?.id !== selectedSource.id ||
+      prevProps.blackboxedRanges[selectedSource.url]?.length !==
+        blackboxedRanges[selectedSource.url]?.length ||
+      (!prevState.editor && !!editor)
+    ) {
+      if (blackboxedRanges[selectedSource.url] == undefined) {
+        editor.removeLineContentMarker(markerTypes.BLACKBOX_LINE_MARKER);
+        return;
       }
+
+      const lines = [];
+      for (const range of blackboxedRanges[selectedSource.url]) {
+        for (let line = range.start.line; line <= range.end.line; line++) {
+          lines.push({ line });
+        }
+      }
+
+      editor.setLineContentMarker({
+        id: markerTypes.BLACKBOX_LINE_MARKER,
+        lineClassName: "blackboxed-line",
+        // If the the whole source is blackboxed, lets just mark all positions.
+        shouldMarkAllLines: !blackboxedRanges[selectedSource.url].length,
+        lines,
+      });
     }
   }
 
@@ -418,9 +341,6 @@ class Editor extends PureComponent {
     }
 
     if (editor) {
-      if (!features.codemirrorNext) {
-        editor.codeMirror.off("scroll", this.onEditorScroll);
-      }
       editor.destroy();
       this.setState({ editor: null });
       removeEditor();
@@ -503,19 +423,8 @@ class Editor extends PureComponent {
    * split console. Restore it here, but preventDefault if and only if there
    * is a multiselection.
    */
-  onEscape = e => {
-    if (!this.state.editor) {
-      return;
-    }
+  onEscape() {}
 
-    if (!features.codemirrorNext) {
-      const { codeMirror } = this.state.editor;
-      if (codeMirror.listSelections().length > 1) {
-        codeMirror.execCommand("singleSelection");
-        e.preventDefault();
-      }
-    }
-  };
   // Note: The line is optional, if not passed it fallsback to lineAtHeight.
   openMenu(event, line, ch) {
     event.stopPropagation();
@@ -541,20 +450,14 @@ class Editor extends PureComponent {
 
     const target = event.target;
     const { id: sourceId } = selectedSource;
-    if (!features.codemirrorNext) {
-      line = line ?? lineAtHeight(editor, selectedSource, event);
-    }
 
     if (typeof line != "number") {
       return;
     }
 
     if (
-      // handles codemirror 6
       target.classList.contains("cm-gutter") ||
-      target.classList.contains("cm-gutterElement") ||
-      // handles codemirror 5
-      target.classList.contains("CodeMirror-linenumber")
+      target.classList.contains("cm-gutterElement")
     ) {
       const location = createLocation({
         line,
@@ -583,16 +486,11 @@ class Editor extends PureComponent {
       return;
     }
 
-    let location;
-    if (features.codemirrorNext) {
-      location = createLocation({
-        source: selectedSource,
-        line: fromEditorLine(selectedSource, line),
-        column: editor.isWasm ? 0 : ch + 1,
-      });
-    } else {
-      location = getSourceLocationFromMouseEvent(editor, selectedSource, event);
-    }
+    const location = createLocation({
+      source: selectedSource,
+      line: fromEditorLine(selectedSource, line),
+      column: editor.isWasm ? 0 : ch + 1,
+    });
 
     const lineObject = editor.getSelectionCursor();
     this.props.showEditorContextMenu(event, editor, lineObject, location);
@@ -691,23 +589,15 @@ class Editor extends PureComponent {
   onClick(e, line, ch) {
     const { selectedSource, updateCursorPosition, jumpToMappedLocation } =
       this.props;
+
     const { editor } = this.state;
 
     if (selectedSource) {
-      let sourceLocation;
-      if (features.codemirrorNext) {
-        sourceLocation = createLocation({
-          source: selectedSource,
-          line: fromEditorLine(selectedSource, line),
-          column: editor.isWasm ? 0 : ch + 1,
-        });
-      } else {
-        sourceLocation = getSourceLocationFromMouseEvent(
-          this.state.editor,
-          selectedSource,
-          e
-        );
-      }
+      const sourceLocation = createLocation({
+        source: selectedSource,
+        line: fromEditorLine(selectedSource, line),
+        column: editor.isWasm ? 0 : ch + 1,
+      });
 
       if (e.metaKey && e.altKey) {
         jumpToMappedLocation(sourceLocation);
@@ -735,15 +625,8 @@ class Editor extends PureComponent {
   }
 
   scrollToLocation(nextProps, editor) {
-    const { selectedLocation, selectedSource } = nextProps;
-
-    let { line, column } = toEditorPosition(selectedLocation);
-
-    if (selectedSource && hasDocument(selectedSource.id)) {
-      const doc = getDocument(selectedSource.id);
-      const lineText = doc.getLine(line);
-      column = Math.max(column, getIndentation(lineText));
-    }
+    const { selectedLocation } = nextProps;
+    const { line, column } = toEditorPosition(selectedLocation);
     return editor.scrollTo(line, column);
   }
 
@@ -756,9 +639,6 @@ class Editor extends PureComponent {
 
     // check if we previously had a selected source
     if (!selectedSource) {
-      if (!features.codemirrorNext) {
-        this.clearEditor();
-      }
       return;
     }
 
@@ -777,25 +657,10 @@ class Editor extends PureComponent {
       return;
     }
 
-    if (!features.codemirrorNext) {
-      showSourceText(editor, selectedSource, selectedSourceTextContent);
-    } else {
-      await editor.setText(
-        selectedSourceTextContent.value.value,
-        selectedSource.id
-      );
-    }
-  }
-
-  clearEditor() {
-    const { editor } = this.state;
-    if (!editor) {
-      return;
-    }
-
-    const doc = editor.createDocument("", { name: "text" });
-    editor.replaceDocument(doc);
-    resetLineNumberFormat(editor);
+    await editor.setText(
+      selectedSourceTextContent.value.value,
+      selectedSource.id
+    );
   }
 
   showErrorMessage(msg) {
@@ -810,31 +675,11 @@ class Editor extends PureComponent {
     } else {
       error = L10N.getFormatStr("errorLoadingText3", msg);
     }
-    if (!features.codemirrorNext) {
-      const doc = editor.createDocument(error, { name: "text" });
-      editor.replaceDocument(doc);
-      resetLineNumberFormat(editor);
-    } else {
-      editor.setText(error);
-    }
+    editor.setText(error);
   }
 
   showLoadingMessage(editor) {
-    if (!features.codemirrorNext) {
-      // Create the "loading message" document only once
-      let doc = getDocument("loading");
-      if (!doc) {
-        doc = editor.createDocument(L10N.getStr("loadingText"), {
-          name: "text",
-        });
-        setDocument("loading", doc);
-      }
-      // `createDocument` won't be used right away in the editor, we still need to
-      // explicitely update it
-      editor.replaceDocument(doc);
-    } else {
-      editor.setText(L10N.getStr("loadingText"));
-    }
+    editor.setText(L10N.getStr("loadingText"));
   }
 
   getInlineEditorStyles() {
@@ -860,9 +705,6 @@ class Editor extends PureComponent {
       isTraceSelected,
       inlinePreviewEnabled,
       highlightedLineRange,
-      blackboxedRanges,
-      isSourceOnIgnoreList,
-      selectedSourceIsBlackBoxed,
       isOriginalSourceAndMapScopesEnabled,
       selectedSourceTextContent,
     } = this.props;
@@ -872,71 +714,15 @@ class Editor extends PureComponent {
       return null;
     }
 
-    if (features.codemirrorNext) {
-      // Only load the sub components if the content has loaded without issues.
-      if (
-        selectedSourceTextContent &&
-        !isFulfilled(selectedSourceTextContent)
-      ) {
-        return null;
-      }
-
-      return React.createElement(
-        React.Fragment,
-        null,
-        React.createElement(Breakpoints, { editor }),
-        (isPaused || isTraceSelected) &&
-          selectedSource.isOriginal &&
-          !selectedSource.isPrettyPrinted &&
-          !isOriginalSourceAndMapScopesEnabled
-          ? null
-          : React.createElement(Preview, {
-              editor,
-              editorRef: this.$editorWrapper,
-            }),
-        React.createElement(DebugLine, { editor, selectedSource }),
-        React.createElement(HighlightLine, { editor }),
-        React.createElement(Exceptions, { editor }),
-        conditionalPanelLocation
-          ? React.createElement(ConditionalPanel, {
-              editor,
-              selectedSource,
-            })
-          : null,
-        (isPaused || isTraceSelected) &&
-          inlinePreviewEnabled &&
-          (!selectedSource.isOriginal ||
-            selectedSource.isPrettyPrinted ||
-            isOriginalSourceAndMapScopesEnabled)
-          ? React.createElement(InlinePreviews, {
-              editor,
-            })
-          : null,
-        highlightedLineRange
-          ? React.createElement(HighlightLines, {
-              editor,
-              range: highlightedLineRange,
-            })
-          : null,
-        React.createElement(ColumnBreakpoints, {
-          editor,
-        })
-      );
-    }
-
-    if (!getDocument(selectedSource.id)) {
+    // Only load the sub components if the content has loaded without issues.
+    if (selectedSourceTextContent && !isFulfilled(selectedSourceTextContent)) {
       return null;
     }
-    return div(
+
+    return React.createElement(
+      React.Fragment,
       null,
-      React.createElement(DebugLine, null),
-      React.createElement(HighlightLine, null),
-      React.createElement(EmptyLines, {
-        editor,
-      }),
-      React.createElement(Breakpoints, {
-        editor,
-      }),
+      React.createElement(Breakpoints, { editor }),
       (isPaused || isTraceSelected) &&
         selectedSource.isOriginal &&
         !selectedSource.isPrettyPrinted &&
@@ -946,39 +732,33 @@ class Editor extends PureComponent {
             editor,
             editorRef: this.$editorWrapper,
           }),
+      React.createElement(DebugLine, { editor, selectedSource }),
+      React.createElement(HighlightLine, { editor }),
+      React.createElement(Exceptions, { editor }),
+      conditionalPanelLocation
+        ? React.createElement(ConditionalPanel, {
+            editor,
+            selectedSource,
+          })
+        : null,
+      (isPaused || isTraceSelected) &&
+        inlinePreviewEnabled &&
+        (!selectedSource.isOriginal ||
+          selectedSource.isPrettyPrinted ||
+          isOriginalSourceAndMapScopesEnabled)
+        ? React.createElement(InlinePreviews, {
+            editor,
+          })
+        : null,
       highlightedLineRange
         ? React.createElement(HighlightLines, {
             editor,
             range: highlightedLineRange,
           })
         : null,
-      isSourceOnIgnoreList || selectedSourceIsBlackBoxed
-        ? React.createElement(BlackboxLines, {
-            editor,
-            selectedSource,
-            isSourceOnIgnoreList,
-            blackboxedRangesForSelectedSource:
-              blackboxedRanges[selectedSource.url],
-          })
-        : null,
-      React.createElement(Exceptions, null),
-      conditionalPanelLocation
-        ? React.createElement(ConditionalPanel, {
-            editor,
-          })
-        : null,
       React.createElement(ColumnBreakpoints, {
         editor,
-      }),
-      (isPaused || isTraceSelected) &&
-        inlinePreviewEnabled &&
-        (!selectedSource.isOriginal ||
-          (selectedSource.isOriginal && selectedSource.isPrettyPrinted) ||
-          isOriginalSourceAndMapScopesEnabled)
-        ? React.createElement(InlinePreviews, {
-            editor,
-          })
-        : null
+      })
     );
   }
 
