@@ -1,14 +1,13 @@
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
  http://creativecommons.org/publicdomain/zero/1.0/ */
-/* eslint no-unused-vars: [2, {"vars": "local"}] */
-/* import-globals-from ../../test/head.js */
+
 "use strict";
 
 // Import the inspector's head.js first (which itself imports shared-head.js).
 Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/inspector/test/head.js",
-  this);
+  this
+);
 
 registerCleanupFunction(() => {
   Services.prefs.clearUserPref("devtools.defaultColorUnit");
@@ -23,6 +22,21 @@ function fireCopyEvent(element) {
   element.dispatchEvent(evt);
 }
 
+/**
+ * Return all the computed items in the computed view
+ *
+ * @param {CssComputedView} view
+ *        The instance of the computed view panel
+ * @returns {Array<Element>}
+ */
+function getComputedViewProperties(view) {
+  return Array.from(
+    view.styleDocument.querySelectorAll(
+      "#computed-container .computed-property-view"
+    )
+  );
+}
+
 /**
  * Get references to the name and value span nodes corresponding to a given
  * property name in the computed-view
@@ -35,13 +49,12 @@ function fireCopyEvent(element) {
  */
 function getComputedViewProperty(view, name) {
   let prop;
-  for (const property of view.styleDocument.querySelectorAll(
-      "#computed-container .computed-property-view")) {
+  for (const property of getComputedViewProperties(view)) {
     const nameSpan = property.querySelector(".computed-property-name");
     const valueSpan = property.querySelector(".computed-property-value");
 
     if (nameSpan.firstChild.textContent === name) {
-      prop = {nameSpan: nameSpan, valueSpan: valueSpan};
+      prop = { nameSpan, valueSpan };
       break;
     }
   }
@@ -60,7 +73,7 @@ function getComputedViewProperty(view, name) {
 function getComputedViewPropertyView(view, name) {
   let propView;
   for (const propertyView of view.propertyViews) {
-    if (propertyView._propertyInfo.name === name) {
+    if (propertyView.propertyInfo.name === name) {
       propView = propertyView;
       break;
     }
@@ -69,12 +82,11 @@ function getComputedViewPropertyView(view, name) {
 }
 
 /**
- * Get a reference to the computed-property-content element for a given property name in
+ * Get a reference to the matched rules element for a given property name in
  * the computed-view.
- * A computed-property-content element always follows (nextSibling) the property itself
+ * A matched rule element is inside the property element (<li>) itself
  * and is only shown when the twisty icon is expanded on the property.
- * A computed-property-content element contains matched rules, with selectors,
- * properties, values and stylesheet links
+ * It contains matched rules, with selectors, properties, values and stylesheet links.
  *
  * @param {CssComputedView} view
  *        The instance of the computed view panel
@@ -83,15 +95,17 @@ function getComputedViewPropertyView(view, name) {
  * @return {Promise} A promise that resolves to the property matched rules
  * container
  */
-var getComputedViewMatchedRules = async function(view, name) {
+var getComputedViewMatchedRules = async function (view, name) {
   let expander;
-  let propertyContent;
+  let matchedRulesEl;
   for (const property of view.styleDocument.querySelectorAll(
-      "#computed-container .computed-property-view")) {
+    "#computed-container .computed-property-view"
+  )) {
     const nameSpan = property.querySelector(".computed-property-name");
     if (nameSpan.firstChild.textContent === name) {
       expander = property.querySelector(".computed-expandable");
-      propertyContent = property.nextSibling;
+      matchedRulesEl = property.querySelector(".matchedselectors");
+
       break;
     }
   }
@@ -101,9 +115,11 @@ var getComputedViewMatchedRules = async function(view, name) {
     const onExpand = view.inspector.once("computed-view-property-expanded");
     expander.click();
     await onExpand;
+
+    await waitFor(() => expander.hasAttribute("open"));
   }
 
-  return propertyContent;
+  return matchedRulesEl;
 };
 
 /**
@@ -116,9 +132,8 @@ var getComputedViewMatchedRules = async function(view, name) {
  *        The name of the property to retrieve
  * @return {String} The property value
  */
-function getComputedViewPropertyValue(view, name, propertyName) {
-  return getComputedViewProperty(view, name, propertyName)
-    .valueSpan.textContent;
+function getComputedViewPropertyValue(view, name) {
+  return getComputedViewProperty(view, name).valueSpan.textContent;
 }
 
 /**
@@ -136,7 +151,7 @@ function expandComputedViewPropertyByIndex(view, index) {
   info("Expanding property " + index + " in the computed view");
   const expandos = view.styleDocument.querySelectorAll(".computed-expandable");
   if (!expandos.length || !expandos[index]) {
-    return promise.reject();
+    return Promise.reject();
   }
 
   const onExpand = view.inspector.once("computed-view-property-expanded");
@@ -154,7 +169,9 @@ function expandComputedViewPropertyByIndex(view, index) {
  * @return {DOMNode} The link at the given index, if one exists, null otherwise
  */
 function getComputedViewLinkByIndex(view, index) {
-  const links = view.styleDocument.querySelectorAll(".rule-link .computed-link");
+  const links = view.styleDocument.querySelectorAll(
+    ".rule-link .computed-link"
+  );
   return links[index];
 }
 
@@ -180,12 +197,16 @@ function selectAllText(view) {
 async function copyAllAndCheckClipboard(view, expectedPattern) {
   selectAllText(view);
   const contentDoc = view.styleDocument;
-  const prop = contentDoc.querySelector("#computed-container .computed-property-view");
+  const prop = contentDoc.querySelector(
+    "#computed-container .computed-property-view"
+  );
 
   try {
     info("Trigger a copy event and wait for the clipboard content");
-    await waitForClipboardPromise(() => fireCopyEvent(prop),
-                                  () => checkClipboard(expectedPattern));
+    await waitForClipboardPromise(
+      () => fireCopyEvent(prop),
+      () => checkClipboard(expectedPattern)
+    );
   } catch (e) {
     failClipboardCheck(expectedPattern);
   }
@@ -208,7 +229,8 @@ async function copySomeTextAndCheckClipboard(view, positions, expectedPattern) {
 
   const contentDocument = view.styleDocument;
   const props = contentDocument.querySelectorAll(
-    "#computed-container .computed-property-view");
+    "#computed-container .computed-property-view"
+  );
 
   info("Create the text selection range");
   const range = contentDocument.createRange();
@@ -218,15 +240,17 @@ async function copySomeTextAndCheckClipboard(view, positions, expectedPattern) {
 
   try {
     info("Trigger a copy event and wait for the clipboard content");
-    await waitForClipboardPromise(() => fireCopyEvent(props[0]),
-                                  () => checkClipboard(expectedPattern));
+    await waitForClipboardPromise(
+      () => fireCopyEvent(props[0]),
+      () => checkClipboard(expectedPattern)
+    );
   } catch (e) {
     failClipboardCheck(expectedPattern);
   }
 }
 
 function checkClipboard(expectedPattern) {
-  const actual = SpecialPowers.getClipboardData("text/unicode");
+  const actual = SpecialPowers.getClipboardData("text/plain");
   const expectedRegExp = new RegExp(expectedPattern, "g");
   return expectedRegExp.test(actual);
 }
@@ -238,15 +262,17 @@ function failClipboardCheck(expectedPattern) {
   expectedPattern = expectedPattern.replace(/\\\(/g, "(");
   expectedPattern = expectedPattern.replace(/\\\)/g, ")");
 
-  let actual = SpecialPowers.getClipboardData("text/unicode");
+  let actual = SpecialPowers.getClipboardData("text/plain");
 
   // Trim the right hand side of our strings. This is because expectedPattern
   // accounts for windows sometimes adding a newline to our copied data.
   expectedPattern = expectedPattern.trimRight();
   actual = actual.trimRight();
 
-  dump("TEST-UNEXPECTED-FAIL | Clipboard text does not match expected ... " +
-    "results (escaped for accurate comparison):\n");
+  dump(
+    "TEST-UNEXPECTED-FAIL | Clipboard text does not match expected ... " +
+      "results (escaped for accurate comparison):\n"
+  );
   info("Actual: " + escape(actual));
   info("Expected: " + escape(expectedPattern));
 }
