@@ -137,6 +137,18 @@ InspectorSearch.prototype = {
       this._onSearch(event.shiftKey);
       event.preventDefault();
     }
+
+    // The search box is a `<input type="search">`, which would clear on pressing Escape.
+    // Prevent the clear action from happening when the autocomplete popup is visible and
+    // only hide it. The preventDefault has to be called during keydown and not keypress
+    // to avoid the input clearance.
+    if (
+      event.keyCode === KeyCodes.DOM_VK_ESCAPE &&
+      this.autocompleter.searchPopup?.isOpen
+    ) {
+      event.preventDefault();
+      this.autocompleter.hidePopup();
+    }
   },
 
   findNext() {
@@ -202,6 +214,11 @@ class SelectorAutocompleter extends EventEmitter {
     ID: "id",
     TAG: "tag",
     ATTRIBUTE: "attribute",
+    // This is for pseudo classes (e.g. `:active`, `:not()`). We keep it as "pseudo" as
+    // the server handles both pseudo elements and pseudo classes under the same type
+    PSEUDO_CLASS: "pseudo",
+    // This is for pseudo element (e.g. `::selection`)
+    PSEUDO_ELEMENT: "pseudo-element",
   };
 
   // The current state of the query.
@@ -263,6 +280,10 @@ class SelectorAutocompleter extends EventEmitter {
             this.#state = this.States.ID;
           } else if (lastChar === "[") {
             this.#state = this.States.ATTRIBUTE;
+          } else if (lastChar === ":") {
+            this.#state = this.States.PSEUDO_CLASS;
+          } else if (lastChar === ")") {
+            this.#state = null;
           } else {
             this.#state = this.States.TAG;
           }
@@ -278,6 +299,10 @@ class SelectorAutocompleter extends EventEmitter {
               this.#state = this.States.ID;
             } else if (lastChar === "[") {
               this.#state = this.States.ATTRIBUTE;
+            } else if (lastChar === ":") {
+              this.#state = this.States.PSEUDO_CLASS;
+            } else if (lastChar === ")") {
+              this.#state = null;
             } else {
               this.#state = this.States.CLASS;
             }
@@ -294,6 +319,10 @@ class SelectorAutocompleter extends EventEmitter {
               this.#state = this.States.CLASS;
             } else if (lastChar === "[") {
               this.#state = this.States.ATTRIBUTE;
+            } else if (lastChar === ":") {
+              this.#state = this.States.PSEUDO_CLASS;
+            } else if (lastChar === ")") {
+              this.#state = null;
             } else {
               this.#state = this.States.ID;
             }
@@ -309,11 +338,34 @@ class SelectorAutocompleter extends EventEmitter {
               this.#state = this.States.CLASS;
             } else if (lastChar === "#") {
               this.#state = this.States.ID;
+            } else if (lastChar === ":") {
+              this.#state = this.States.PSEUDO_CLASS;
+            } else if (lastChar === ")") {
+              this.#state = null;
             } else {
               this.#state = this.States.ATTRIBUTE;
             }
           }
           break;
+
+        case this.States.PSEUDO_CLASS:
+          if (lastChar === ":" && secondLastChar === ":") {
+            // We don't support searching for pseudo elements, so bail out when we
+            // see `::`
+            this.#state = this.States.PSEUDO_ELEMENT;
+            return this.#state;
+          }
+
+          if (lastChar === "(") {
+            this.#state = null;
+          } else if (lastChar === ".") {
+            this.#state = this.States.CLASS;
+          } else if (lastChar === "#") {
+            this.#state = this.States.ID;
+          } else {
+            this.#state = this.States.PSEUDO_CLASS;
+          }
+          break;
       }
     }
     return this.#state;
@@ -406,16 +458,20 @@ class SelectorAutocompleter extends EventEmitter {
   /**
    * Populates the suggestions list and show the suggestion popup.
    *
+   * @param {Array} suggestions: List of suggestions
+   * @param {String|null} popupState: One of SelectorAutocompleter.States
    * @return {Promise} promise that will resolve when the autocomplete popup is fully
    * displayed or hidden.
    */
-  #showPopup(list, popupState) {
+  #showPopup(suggestions, popupState) {
     let total = 0;
     const query = this.searchBox.value;
     const items = [];
 
-    for (let [value, , state] of list) {
-      if (query.match(/[\s>+~]$/)) {
+    for (let [value, state] of suggestions) {
+      if (popupState === this.States.PSEUDO_CLASS) {
+        value = query.substring(0, query.lastIndexOf(":")) + value;
+      } else if (query.match(/[\s>+~]$/)) {
         // for cases like 'div ', 'div >', 'div+' or 'div~'
         value = query + value;
       } else if (query.match(/[\s>+~][\.#a-zA-Z][^\s>+~\.#\[]*$/)) {
@@ -485,12 +541,19 @@ class SelectorAutocompleter extends EventEmitter {
     const originalQuery = this.searchBox.value;
 
     const state = this.state;
-    let firstPart = "";
+    let completing = "";
 
-    if (query.endsWith("*") || state === this.States.ATTRIBUTE) {
-      // Hide the popup if the query ends with * (because we don't want to
-      // suggest all nodes) or if it is an attribute selector (because
-      // it would give a lot of useless results).
+    if (
+      // Hide the popup if:
+      // - the query is empty
+      !query ||
+      // - the query ends with * (because we don't want to suggest all nodes)
+      query.endsWith("*") ||
+      // - if it is an attribute selector (because it would give a lot of useless results).
+      state === this.States.ATTRIBUTE ||
+      // - if it is a pseudo element selector (we don't support it, see Bug 1097991)
+      state === this.States.PSEUDO_ELEMENT
+    ) {
       this.hidePopup();
       this.emitForTests("processing-done", { query: originalQuery });
       return;
@@ -505,16 +568,20 @@ class SelectorAutocompleter extends EventEmitter {
       // - 'div.foo ~ s' returns 's'
       // - 'div.foo x-el_1' returns 'x-el_1'
       const matches = query.match(/[\s>+~]?(?<tag>[a-zA-Z0-9_-]*)$/);
-      firstPart = matches.groups.tag;
-      query = query.slice(0, query.length - firstPart.length);
+      completing = matches.groups.tag;
+      query = query.slice(0, query.length - completing.length);
     } else if (state === this.States.CLASS) {
       // gets the class that is being completed. For ex. '.foo.b' returns 'b'
-      firstPart = query.match(/\.([^\.]*)$/)[1];
-      query = query.slice(0, query.length - firstPart.length - 1);
+      completing = query.match(/\.([^\.]*)$/)[1];
+      query = query.slice(0, query.length - completing.length - 1);
     } else if (state === this.States.ID) {
       // gets the id that is being completed. For ex. '.foo#b' returns 'b'
-      firstPart = query.match(/#([^#]*)$/)[1];
-      query = query.slice(0, query.length - firstPart.length - 1);
+      completing = query.match(/#([^#]*)$/)[1];
+      query = query.slice(0, query.length - completing.length - 1);
+    } else if (state === this.States.PSEUDO_CLASS) {
+      // The getSuggestionsForQuery expects a pseudo element without the : prefix
+      completing = query.substring(query.lastIndexOf(":") + 1);
+      query = "";
     }
     // TODO: implement some caching so that over the wire request is not made
     // everytime.
@@ -525,19 +592,25 @@ class SelectorAutocompleter extends EventEmitter {
     let suggestions =
       await this.inspector.commands.inspectorCommand.getSuggestionsForQuery(
         query,
-        firstPart,
+        completing,
         state
       );
 
     if (state === this.States.CLASS) {
-      firstPart = "." + firstPart;
+      completing = "." + completing;
     } else if (state === this.States.ID) {
-      firstPart = "#" + firstPart;
+      completing = "#" + completing;
+    } else if (state === this.States.PSEUDO_CLASS) {
+      completing = ":" + completing;
+      // Remove pseudo-element suggestions, since the search does not work with them (Bug 1097991)
+      suggestions = suggestions.filter(
+        suggestion => !suggestion[0].startsWith("::")
+      );
     }
 
     // If there is a single tag match and it's what the user typed, then
     // don't need to show a popup.
-    if (suggestions.length === 1 && suggestions[0][0] === firstPart) {
+    if (suggestions.length === 1 && suggestions[0][0] === completing) {
       suggestions = [];
     }
 
