@@ -179,7 +179,6 @@ function Inspector(toolbox, commands) {
   this.onSidebarSelect = this.onSidebarSelect.bind(this);
   this.onSidebarShown = this.onSidebarShown.bind(this);
   this.onSidebarToggle = this.onSidebarToggle.bind(this);
-  this.onReflowInSelection = this.onReflowInSelection.bind(this);
   this.listenForSearchEvents = this.listenForSearchEvents.bind(this);
 
   this.prefObserver = new PrefObserver("devtools.");
@@ -199,6 +198,13 @@ Inspector.prototype = {
    * @param {Object} options
    * @param {NodeFront|undefined} options.defaultStartupNode: Optional node front that
    *        will be selected when the first root node is available.
+   * @param {ElementIdentifier|undefined} options.defaultStartupNodeDomReference: Optional
+   *        element identifier whose matching node front will be selected when the first
+   *        root node is available.
+   *        Will be ignored if defaultStartupNode is passed.
+   * @param {String|undefined} options.defaultStartupNodeSelectionReason: Optional string
+   *        that will be used as a reason for the node selection when either
+   *        defaultStartupNode or defaultStartupNodeDomReference is passed
    * @returns {Inspector}
    */
   async init(options = {}) {
@@ -220,9 +226,13 @@ Inspector.prototype = {
     // iframe if it had already been initialized.
     this.setupSplitter();
 
-    // Optional NodeFront set on inspector startup, to be selected once the first root
+    // Optional NodeFront/ElementIdentifier set on inspector startup, to be selected once the first root
     // node is available.
     this._defaultStartupNode = options.defaultStartupNode;
+    this._defaultStartupNodeDomReference =
+      options.defaultStartupNodeDomReference;
+    this._defaultStartupNodeSelectionReason =
+      options.defaultStartupNodeSelectionReason;
 
     // NodeFront for the DOM Element selected when opening the inspector, or after each
     // navigation (i.e. each time a new Root Node is available)
@@ -241,6 +251,7 @@ Inspector.prototype = {
       // To observe CSS change before opening changes view.
       TYPES.CSS_CHANGE,
       TYPES.DOCUMENT_EVENT,
+      TYPES.REFLOW,
     ];
     // The root node is retrieved from onTargetSelected which is now called
     // on startup as well as on any navigation (= new top level target).
@@ -367,6 +378,16 @@ Inspector.prototype = {
       ) {
         this._onWillNavigate();
       }
+
+      if (resource.resourceType === this.toolbox.resourceCommand.TYPES.REFLOW) {
+        this.emit("reflow");
+        if (resource.targetFront === this.selection?.nodeFront?.targetFront) {
+          // This event will be fired whenever a reflow is detected in the target front of the
+          // selected node front (so when a reflow is detected inside any of the windows that
+          // belong to the BrowsingContext where the currently selected node lives).
+          this.emit("reflow-in-selected-target");
+        }
+      }
     }
 
     return Promise.all(rootNodeAvailablePromises);
@@ -389,8 +410,11 @@ Inspector.prototype = {
       }
 
       this.selection.setNodeFront(defaultNode, {
-        reason: "inspector-default-selection",
+        reason:
+          this._defaultStartupNodeSelectionReason ??
+          "inspector-default-selection",
       });
+      this._defaultStartupNodeSelectionReason = null;
 
       await this._initMarkupView();
 
@@ -600,9 +624,11 @@ Inspector.prototype = {
    *        The current root node front for the top walker.
    */
   async _getDefaultNodeForSelection(rootNodeFront) {
+    let node;
     if (this._defaultStartupNode) {
-      const node = this._defaultStartupNode;
+      node = this._defaultStartupNode;
       this._defaultStartupNode = null;
+      this._defaultStartupNodeDomReference = null;
       return node;
     }
 
@@ -610,11 +636,36 @@ Inspector.prototype = {
     const pendingSelectionUnique = Symbol("pending-selection");
     this._pendingSelectionUnique = pendingSelectionUnique;
 
+    if (this._defaultStartupNodeDomReference) {
+      const domReference = this._defaultStartupNodeDomReference;
+      // nullify before calling the async getNodeActorFromContentDomReference so calls
+      // made to getDefaultNodeForSelection while the promise is pending will be properly
+      // ignored with the check on pendingSelectionUnique
+      this._defaultStartupNode = null;
+      this._defaultStartupNodeDomReference = null;
+
+      try {
+        node =
+          await this.inspectorFront.getNodeActorFromContentDomReference(
+            domReference
+          );
+      } catch (e) {
+        console.warn(
+          "Couldn't retrieve node front from dom reference",
+          domReference
+        );
+      }
+    }
+
     if (this._pendingSelectionUnique !== pendingSelectionUnique) {
       // If this method was called again while waiting, bail out.
       return null;
     }
 
+    if (node) {
+      return node;
+    }
+
     const walker = rootNodeFront.walkerFront;
     const cssSelectors = this.selectionCssSelectors;
     // Try to find a default node using three strategies:
@@ -634,7 +685,7 @@ Inspector.prototype = {
 
     // Try all default node selectors until a valid node is found.
     for (const selector of defaultNodeSelectors) {
-      const node = await selector();
+      node = await selector();
       if (this._pendingSelectionUnique !== pendingSelectionUnique) {
         // If this method was called again while waiting, bail out.
         return null;
@@ -1588,7 +1639,6 @@ Inspector.prototype = {
 
     this.updateAddElementButton();
     this.updateSelectionCssSelectors();
-    this.trackReflowsInSelection();
 
     const selfUpdate = this.updating("inspector-panel");
     executeSoon(() => {
@@ -1601,56 +1651,6 @@ Inspector.prototype = {
     });
   },
 
-  /**
-   * Starts listening for reflows in the targetFront of the currently selected nodeFront.
-   */
-  async trackReflowsInSelection() {
-    this.untrackReflowsInSelection();
-    if (!this.selection.nodeFront) {
-      return;
-    }
-
-    if (this._destroyed) {
-      return;
-    }
-
-    try {
-      await this.commands.resourceCommand.watchResources(
-        [this.commands.resourceCommand.TYPES.REFLOW],
-        {
-          onAvailable: this.onReflowInSelection,
-        }
-      );
-    } catch (e) {
-      // it can happen that watchResources fails as the client closes while we're processing
-      // some asynchronous call.
-      // In order to still get valid exceptions, we re-throw the exception if the inspector
-      // isn't destroyed.
-      if (!this._destroyed) {
-        throw e;
-      }
-    }
-  },
-
-  /**
-   * Stops listening for reflows.
-   */
-  untrackReflowsInSelection() {
-    this.commands.resourceCommand.unwatchResources(
-      [this.commands.resourceCommand.TYPES.REFLOW],
-      {
-        onAvailable: this.onReflowInSelection,
-      }
-    );
-  },
-
-  onReflowInSelection() {
-    // This event will be fired whenever a reflow is detected in the target front of the
-    // selected node front (so when a reflow is detected inside any of the windows that
-    // belong to the BrowsingContext when the currently selected node lives).
-    this.emit("reflow-in-selected-target");
-  },
-
   /**
    * Delay the "inspector-updated" notification while a tool
    * is updating itself.  Returns a function that must be
@@ -1788,7 +1788,6 @@ Inspector.prototype = {
     resourceCommand.unwatchResources(this._watchedResources, {
       onAvailable: this.onResourceAvailable,
     });
-    this.untrackReflowsInSelection();
 
     this._InspectorTabPanel = null;
     this._TabBar = null;
@@ -1853,7 +1852,9 @@ Inspector.prototype = {
   },
 
   stopEyeDropperListeners() {
-    this.toolbox.tellRDMAboutPickerState(false, PICKER_TYPES.EYEDROPPER);
+    this.toolbox
+      .tellRDMAboutPickerState(false, PICKER_TYPES.EYEDROPPER)
+      .catch(console.error);
     this.inspectorFront.off("color-pick-canceled", this.onEyeDropperDone);
     this.inspectorFront.off("color-picked", this.onEyeDropperDone);
     this.off("new-root", this.onEyeDropperDone);
