@@ -4,62 +4,61 @@
 
 "use strict";
 
-const Services = require("Services");
-const EventEmitter = require("devtools/shared/old-event-emitter");
+const EventEmitter = require("resource://devtools/shared/event-emitter.js");
 const isOSX = Services.appinfo.OS === "Darwin";
-const {KeyCodes} = require("devtools/client/shared/keycodes");
+const { KeyCodes } = require("resource://devtools/client/shared/keycodes.js");
 
 // List of electron keys mapped to DOM API (DOM_VK_*) key code
 const ElectronKeysMapping = {
-  "F1": "DOM_VK_F1",
-  "F2": "DOM_VK_F2",
-  "F3": "DOM_VK_F3",
-  "F4": "DOM_VK_F4",
-  "F5": "DOM_VK_F5",
-  "F6": "DOM_VK_F6",
-  "F7": "DOM_VK_F7",
-  "F8": "DOM_VK_F8",
-  "F9": "DOM_VK_F9",
-  "F10": "DOM_VK_F10",
-  "F11": "DOM_VK_F11",
-  "F12": "DOM_VK_F12",
-  "F13": "DOM_VK_F13",
-  "F14": "DOM_VK_F14",
-  "F15": "DOM_VK_F15",
-  "F16": "DOM_VK_F16",
-  "F17": "DOM_VK_F17",
-  "F18": "DOM_VK_F18",
-  "F19": "DOM_VK_F19",
-  "F20": "DOM_VK_F20",
-  "F21": "DOM_VK_F21",
-  "F22": "DOM_VK_F22",
-  "F23": "DOM_VK_F23",
-  "F24": "DOM_VK_F24",
-  "Space": "DOM_VK_SPACE",
-  "Backspace": "DOM_VK_BACK_SPACE",
-  "Delete": "DOM_VK_DELETE",
-  "Insert": "DOM_VK_INSERT",
-  "Return": "DOM_VK_RETURN",
-  "Enter": "DOM_VK_RETURN",
-  "Up": "DOM_VK_UP",
-  "Down": "DOM_VK_DOWN",
-  "Left": "DOM_VK_LEFT",
-  "Right": "DOM_VK_RIGHT",
-  "Home": "DOM_VK_HOME",
-  "End": "DOM_VK_END",
-  "PageUp": "DOM_VK_PAGE_UP",
-  "PageDown": "DOM_VK_PAGE_DOWN",
-  "Escape": "DOM_VK_ESCAPE",
-  "Esc": "DOM_VK_ESCAPE",
-  "Tab": "DOM_VK_TAB",
-  "VolumeUp": "DOM_VK_VOLUME_UP",
-  "VolumeDown": "DOM_VK_VOLUME_DOWN",
-  "VolumeMute": "DOM_VK_VOLUME_MUTE",
-  "PrintScreen": "DOM_VK_PRINTSCREEN",
+  F1: "DOM_VK_F1",
+  F2: "DOM_VK_F2",
+  F3: "DOM_VK_F3",
+  F4: "DOM_VK_F4",
+  F5: "DOM_VK_F5",
+  F6: "DOM_VK_F6",
+  F7: "DOM_VK_F7",
+  F8: "DOM_VK_F8",
+  F9: "DOM_VK_F9",
+  F10: "DOM_VK_F10",
+  F11: "DOM_VK_F11",
+  F12: "DOM_VK_F12",
+  F13: "DOM_VK_F13",
+  F14: "DOM_VK_F14",
+  F15: "DOM_VK_F15",
+  F16: "DOM_VK_F16",
+  F17: "DOM_VK_F17",
+  F18: "DOM_VK_F18",
+  F19: "DOM_VK_F19",
+  F20: "DOM_VK_F20",
+  F21: "DOM_VK_F21",
+  F22: "DOM_VK_F22",
+  F23: "DOM_VK_F23",
+  F24: "DOM_VK_F24",
+  Space: "DOM_VK_SPACE",
+  Backspace: "DOM_VK_BACK_SPACE",
+  Delete: "DOM_VK_DELETE",
+  Insert: "DOM_VK_INSERT",
+  Return: "DOM_VK_RETURN",
+  Enter: "DOM_VK_RETURN",
+  Up: "DOM_VK_UP",
+  Down: "DOM_VK_DOWN",
+  Left: "DOM_VK_LEFT",
+  Right: "DOM_VK_RIGHT",
+  Home: "DOM_VK_HOME",
+  End: "DOM_VK_END",
+  PageUp: "DOM_VK_PAGE_UP",
+  PageDown: "DOM_VK_PAGE_DOWN",
+  Escape: "DOM_VK_ESCAPE",
+  Esc: "DOM_VK_ESCAPE",
+  Tab: "DOM_VK_TAB",
+  VolumeUp: "DOM_VK_VOLUME_UP",
+  VolumeDown: "DOM_VK_VOLUME_DOWN",
+  VolumeMute: "DOM_VK_VOLUME_MUTE",
+  PrintScreen: "DOM_VK_PRINTSCREEN",
 };
 
 /**
- * Helper to listen for keyboard events decribed in .properties file.
+ * Helper to listen for keyboard events described in .properties file.
  *
  * let shortcuts = new KeyShortcuts({
  *   window
@@ -87,17 +86,24 @@ function KeyShortcuts({ window, target }) {
  * allow efficient match on DOM key event. The normalized object matches DOM
  * API.
  *
- * @param DOMWindow window
- *        Any DOM Window object, just to fetch its `KeyboardEvent` object
  * @param String str
  *        The shortcut string to parse, following this document:
  *        https://github.com/electron/electron/blob/master/docs/api/accelerator.md
  */
-KeyShortcuts.parseElectronKey = function (window, str) {
-  let modifiers = str.split("+");
+KeyShortcuts.parseElectronKey = function (str) {
+  // If a localized string is found but has no value in the properties file,
+  // getStr will return `null`. See Bug 1569572.
+  if (typeof str !== "string") {
+    console.error("Invalid key passed to parseElectronKey, stacktrace below");
+    console.trace();
+
+    return null;
+  }
+
+  const modifiers = str.split("+");
   let key = modifiers.pop();
 
-  let shortcut = {
+  const shortcut = {
     ctrl: false,
     meta: false,
     alt: false,
@@ -107,7 +113,7 @@ KeyShortcuts.parseElectronKey = function (window, str) {
     // Set for non-character keys
     keyCode: undefined,
   };
-  for (let mod of modifiers) {
+  for (const mod of modifiers) {
     if (mod === "Alt") {
       shortcut.alt = true;
     } else if (["Command", "Cmd"].includes(mod)) {
@@ -135,8 +141,16 @@ KeyShortcuts.parseElectronKey = function (window, str) {
   }
 
   if (typeof key === "string" && key.length === 1) {
-    // Match any single character
-    shortcut.key = key.toLowerCase();
+    if (shortcut.alt) {
+      // When Alt is involved, some platforms (macOS) give different printable characters
+      // for the `key` value, like `®` for the key `R`.  In this case, prefer matching by
+      // `keyCode` instead.
+      shortcut.keyCode = KeyCodes[`DOM_VK_${key.toUpperCase()}`];
+      shortcut.keyCodeString = key;
+    } else {
+      // Match any single character
+      shortcut.key = key.toLowerCase();
+    }
   } else if (key in ElectronKeysMapping) {
     // Maps the others manually to DOM API DOM_VK_*
     key = ElectronKeysMapping[key];
@@ -153,7 +167,12 @@ KeyShortcuts.parseElectronKey = function (window, str) {
 };
 
 KeyShortcuts.stringify = function (shortcut) {
-  let list = [];
+  if (shortcut === null) {
+    // parseElectronKey might return null in several situations.
+    return "";
+  }
+
+  const list = [];
   if (shortcut.alt) {
     list.push("Alt");
   }
@@ -176,10 +195,36 @@ KeyShortcuts.stringify = function (shortcut) {
   return list.join("+");
 };
 
+/*
+ * Parse an xul-like key string and return an electron-like string.
+ */
+KeyShortcuts.parseXulKey = function (modifiers, shortcut) {
+  modifiers = modifiers
+    .split(",")
+    .map(mod => {
+      if (mod == "alt") {
+        return "Alt";
+      } else if (mod == "shift") {
+        return "Shift";
+      } else if (mod == "accel") {
+        return "CmdOrCtrl";
+      }
+      return mod;
+    })
+    .join("+");
+
+  if (shortcut.startsWith("VK_")) {
+    shortcut = shortcut.substr(3);
+  }
+
+  return modifiers + "+" + shortcut;
+};
+
 KeyShortcuts.prototype = {
   destroy() {
     this.target.removeEventListener("keydown", this);
     this.keys.clear();
+    this.eventEmitter.off();
   },
 
   doesEventMatchShortcut(event, shortcut) {
@@ -193,11 +238,18 @@ KeyShortcuts.prototype = {
       return false;
     }
     if (shortcut.shift != event.shiftKey) {
-      // Shift is a special modifier, it may implicitely be required if the expected key
+      // Check the `keyCode` to see whether it's a character (see also Bug 1493646)
+      const char = String.fromCharCode(event.keyCode);
+      let isAlphabetical = char.length == 1 && char.match(/[a-zA-Z]/);
+
+      // Shift is a special modifier, it may implicitly be required if the expected key
       // is a special character accessible via shift.
-      let isAlphabetical = event.key && event.key.match(/[a-zA-Z]/);
+      if (!isAlphabetical) {
+        isAlphabetical = event.key && event.key.match(/[a-zA-Z]/);
+      }
+
       // OSX: distinguish cmd+[key] from cmd+shift+[key] shortcuts (Bug 1300458)
-      let cmdShortcut = shortcut.meta && !shortcut.alt && !shortcut.ctrl;
+      const cmdShortcut = shortcut.meta && !shortcut.alt && !shortcut.ctrl;
       if (isAlphabetical || cmdShortcut) {
         return false;
       }
@@ -210,18 +262,20 @@ KeyShortcuts.prototype = {
     }
 
     // get the key from the keyCode if key is not provided.
-    let key = event.key || String.fromCharCode(event.keyCode);
+    const key = event.key || String.fromCharCode(event.keyCode);
 
     // For character keys, we match if the final character is the expected one.
     // But for digits we also accept indirect match to please azerty keyboard,
     // which requires Shift to be pressed to get digits.
-    return key.toLowerCase() == shortcut.key ||
+    return (
+      key.toLowerCase() == shortcut.key ||
       (shortcut.key.match(/[0-9]/) &&
-       event.keyCode == shortcut.key.charCodeAt(0));
+        event.keyCode == shortcut.key.charCodeAt(0))
+    );
   },
 
   handleEvent(event) {
-    for (let [key, shortcut] of this.keys) {
+    for (const [key, shortcut] of this.keys) {
       if (this.doesEventMatchShortcut(event, shortcut)) {
         this.eventEmitter.emit(key, event);
       }
@@ -230,11 +284,12 @@ KeyShortcuts.prototype = {
 
   on(key, listener) {
     if (typeof listener !== "function") {
-      throw new Error("KeyShortcuts.on() expects a function as " +
-                      "second argument");
+      throw new Error(
+        "KeyShortcuts.on() expects a function as " + "second argument"
+      );
     }
     if (!this.keys.has(key)) {
-      let shortcut = KeyShortcuts.parseElectronKey(this.window, key);
+      const shortcut = KeyShortcuts.parseElectronKey(key);
       // The key string is wrong and we were unable to compute the key shortcut
       if (!shortcut) {
         return;
