@@ -1,6 +1,6 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
-/* eslint no-unused-vars: [2, {"vars": "local"}] */
+/* eslint no-unused-vars: [2, {"vars": "local", caughtErrors: "none"}] */
 
 /* import-globals-from ../../inspector/test/shared-head.js */
 
@@ -25,9 +25,8 @@ const { Constructor: CC } = Components;
 // and allocation sites if DEBUG_DEVTOOLS_ALLOCATIONS is set to "verbose".
 const DEBUG_ALLOCATIONS = Services.env.get("DEBUG_DEVTOOLS_ALLOCATIONS");
 if (DEBUG_ALLOCATIONS) {
-  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
-  // as it instantiates custom Debugger API instances and has to be running in a distinct
-  // compartments from DevTools and system scopes (JSMs, XPCOM,...)
+  // Load the allocation tracker from the distinct privileged loader in order
+  // to be able to debug all privileged code (ESMs, XPCOM,...) running in the shared privileged global.
   const {
     useDistinctSystemPrincipalLoader,
     releaseDistinctSystemPrincipalLoader,
@@ -72,9 +71,8 @@ if (DEBUG_ALLOCATIONS) {
 // in any other way on parent process's main thread.
 const DEBUG_STEP = Services.env.get("DEBUG_STEP");
 if (DEBUG_STEP) {
-  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
-  // as it instantiates custom Debugger API instances and has to be running in a distinct
-  // compartments from DevTools and system scopes (JSMs, XPCOM,...)
+  // Load the stepper code from the distinct privileged loader in order
+  // to be able to debug all privileged code (ESMs, XPCOM,...) running in the shared privileged global.
   const {
     useDistinctSystemPrincipalLoader,
     releaseDistinctSystemPrincipalLoader,
@@ -97,9 +95,8 @@ if (DEBUG_STEP) {
 
 const DEBUG_TRACE_LINE = Services.env.get("DEBUG_TRACE_LINE");
 if (DEBUG_TRACE_LINE) {
-  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
-  // as it instantiates custom Debugger API instances and has to be running in a distinct
-  // compartments from DevTools and system scopes (ESMs, XPCOM,...)
+  // Load the tracing code from the distinct privileged loader in order
+  // to be able to debug all privileged code (ESMs, XPCOM,...) running in the shared privileged global.
   const {
     useDistinctSystemPrincipalLoader,
     releaseDistinctSystemPrincipalLoader,
@@ -2536,3 +2533,48 @@ async function _maybeOpenAncestorMenu(menuItem) {
   parentMenu.openMenu(true);
   await shown;
 }
+
+/**
+ * Returns the list of console messages DOM Element display in the Web Console
+ * which contains a given string.
+ *
+ * @param {Toolbox} toolbox
+ * @param {String} query
+ * @return {Array<DOMElement>}
+ */
+async function findConsoleMessages(toolbox, query) {
+  const webConsole = await toolbox.getPanel("webconsole");
+  const win = webConsole._frameWindow;
+  return Array.prototype.filter.call(
+    win.document.querySelectorAll(".message"),
+    e => e.innerText.includes(query)
+  );
+}
+
+/**
+ * Wait for a console message to appear with a given text and a given link to
+ * a specific location in a JS source.
+ * Returns the DOM Element in the Web Console for the link to the JS Source.
+ *
+ * @param {Toolbox} toolbox
+ * @param {String} messageText
+ * @param {String} linkText
+ * @return {DOMElement}
+ */
+async function waitForConsoleMessageLink(toolbox, messageText, linkText) {
+  await toolbox.selectTool("webconsole");
+
+  return waitFor(async () => {
+    // Wait until the message updates.
+    const [message] = await findConsoleMessages(toolbox, messageText);
+    if (!message) {
+      return false;
+    }
+    const linkEl = message.querySelector(".frame-link-source");
+    if (!linkEl || linkEl.textContent !== linkText) {
+      return false;
+    }
+
+    return linkEl;
+  });
+}
