@@ -12,11 +12,9 @@ const {
 const constants = require("resource://devtools/client/webconsole/constants.js");
 const { DEFAULT_FILTERS, FILTERS, MESSAGE_TYPE, MESSAGE_SOURCE } = constants;
 
-loader.lazyRequireGetter(
-  this,
-  "getGripPreviewItems",
-  "resource://devtools/client/shared/components/reps/index.js",
-  true
+const { getGripPreviewItems } = ChromeUtils.importESModule(
+  "resource://devtools/client/shared/components/reps/index.mjs",
+  { global: "current" }
 );
 loader.lazyRequireGetter(
   this,
@@ -290,7 +288,7 @@ function addMessage(newMessage, state, filtersState, prefsState, uiState) {
   }
   state.mutableMessagesById.set(addedMessage.id, addedMessage);
 
-  if (newMessage.type === "trace") {
+  if (newMessage.type === "trace" || newMessage.type === "logPoint") {
     // We want the stacktrace to be open by default.
     state.messagesUiById.push(newMessage.id);
   } else if (isGroupType(newMessage.type)) {
@@ -320,9 +318,8 @@ function addMessage(newMessage, state, filtersState, prefsState, uiState) {
 
       // We loop backward through the warning group's messages to get the latest visible
       // messages in it.
-      const messagesInWarningGroup = state.warningGroupsById.get(
-        warningGroupId
-      );
+      const messagesInWarningGroup =
+        state.warningGroupsById.get(warningGroupId);
       for (let i = messagesInWarningGroup.length - 1; i >= 0; i--) {
         const idx = state.visibleMessages.indexOf(messagesInWarningGroup[i]);
         if (idx > -1) {
@@ -454,15 +451,15 @@ function messages(
       });
 
     case constants.PRIVATE_MESSAGES_CLEAR: {
-      const removedIds = [];
+      const removedIds = new Set();
       for (const [id, message] of mutableMessagesById) {
         if (message.private === true) {
-          removedIds.push(id);
+          removedIds.add(id);
         }
       }
 
       // If there's no private messages, there's no need to change the state.
-      if (removedIds.length === 0) {
+      if (removedIds.size === 0) {
         return state;
       }
 
@@ -475,7 +472,7 @@ function messages(
     }
 
     case constants.TARGET_MESSAGES_REMOVE: {
-      const removedIds = [];
+      const removedIds = new Set();
       for (const [id, message] of mutableMessagesById) {
         // Remove message from the target but not evaluations and their results, so
         // 1. we're consistent with the filtering behavior, i.e. we never hide those
@@ -487,7 +484,7 @@ function messages(
           message.type !== MESSAGE_TYPE.COMMAND &&
           message.type !== MESSAGE_TYPE.RESULT
         ) {
-          removedIds.push(id);
+          removedIds.add(id);
         }
       }
 
@@ -725,7 +722,7 @@ function messages(
         {
           ...state,
         },
-        [action.id]
+        new Set([action.id])
       );
 
     case constants.FILTER_TOGGLE:
@@ -854,7 +851,7 @@ function setVisibleMessages({
  * @param {Array} ignoredIds: An array of ids which can't be the new current group.
  * @returns {String|null} The new current group id, or null if there isn't one.
  */
-function getNewCurrentGroup(currentGroup, groupsById, ignoredIds = []) {
+function getNewCurrentGroup(currentGroup, groupsById, ignoredIds = new Set()) {
   if (!currentGroup) {
     return null;
   }
@@ -865,7 +862,7 @@ function getNewCurrentGroup(currentGroup, groupsById, ignoredIds = []) {
   // If there's at least one parent, make the first one the new currentGroup.
   if (Array.isArray(parents) && parents.length) {
     // If the found group must be ignored, let's search for its parent.
-    if (ignoredIds.includes(parents[0])) {
+    if (ignoredIds.has(parents[0])) {
       return getNewCurrentGroup(parents[0], groupsById, ignoredIds);
     }
 
@@ -914,7 +911,7 @@ function limitTopLevelMessageCount(newState, logLimit) {
     return newState;
   }
 
-  const removedMessagesId = [];
+  const removedMessagesId = new Set();
 
   let cleaningGroup = false;
   for (const id of newState.mutableMessagesOrder) {
@@ -941,7 +938,7 @@ function limitTopLevelMessageCount(newState, logLimit) {
       topLevelCount--;
     }
 
-    removedMessagesId.push(id);
+    removedMessagesId.add(id);
   }
 
   return removeMessagesFromState(newState, removedMessagesId);
@@ -952,11 +949,11 @@ function limitTopLevelMessageCount(newState, logLimit) {
  * Be aware that this function MUTATE the `state` argument.
  *
  * @param {MessageState} state
- * @param {Array} removedMessagesIds
+ * @param {Set} removedMessagesIds
  * @returns {MessageState}
  */
 function removeMessagesFromState(state, removedMessagesIds) {
-  if (!Array.isArray(removedMessagesIds) || removedMessagesIds.length === 0) {
+  if (removedMessagesIds.size === 0) {
     return state;
   }
 
@@ -985,8 +982,15 @@ function removeMessagesFromState(state, removedMessagesIds) {
     state.frontsToRelease = state.frontsToRelease.concat(frontsToRelease);
   }
 
-  const isInRemovedId = id => removedMessagesIds.includes(id);
-  const mapHasRemovedIdKey = map => removedMessagesIds.some(id => map.has(id));
+  const isInRemovedId = id => removedMessagesIds.has(id);
+  const mapHasRemovedIdKey = map => {
+    for (const id of removedMessagesIds) {
+      if (map.has(id)) {
+        return true;
+      }
+    }
+    return false;
+  };
   const objectHasRemovedIdKey = obj =>
     Object.keys(obj).findIndex(isInRemovedId) !== -1;
 
@@ -1437,7 +1441,9 @@ function passSearchFilters(message, filters) {
     // Look for a match in notes.
     isTextInNotes(matchStr, message.notes) ||
     // Look for a match in prefix.
-    isTextInPrefix(matchStr, message.prefix);
+    isTextInPrefix(matchStr, message.prefix) ||
+    // Look for a match in displayName.
+    isTextInDisplayName(matchStr, message.displayName);
 
   return matched ? !exclude : exclude;
 }
@@ -1564,6 +1570,13 @@ function isTextInMessageText(matchStr, messageText) {
   return true;
 }
 
+/**
+ * Returns true if given text is included in JS Trace display name.
+ */
+function isTextInDisplayName(matchStr, displayName) {
+  return displayName && matchStr(displayName);
+}
+
 /**
  * Returns true if given text is included in notes.
  */
