@@ -1,6 +1,5 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
-/* eslint no-unused-vars: [2, {"vars": "local"}] */
 
 "use strict";
 
@@ -42,7 +41,7 @@ const GA_PARAMS = `?${new URLSearchParams({
 
 const wcActions = require("resource://devtools/client/webconsole/actions/index.js");
 
-registerCleanupFunction(async function() {
+registerCleanupFunction(async function () {
   // Reset all cookies, tests loading sjs_slow-response-test-server.sjs will
   // set a foo cookie which might have side effects on other tests.
   Services.cookies.removeAll();
@@ -98,22 +97,24 @@ async function openNewTabWithIframesAndConsole(tabUrl, iframes) {
   // to handle remote frames (we don't support creating frames target when the toolbox
   // is already open).
   await addTab(tabUrl);
-  await ContentTask.spawn(gBrowser.selectedBrowser, iframes, async function(
-    urls
-  ) {
-    const iframesLoadPromises = urls.map((url, i) => {
-      const iframe = content.document.createElement("iframe");
-      iframe.classList.add(`iframe-${i + 1}`);
-      const onLoadIframe = new Promise(resolve => {
-        iframe.addEventListener("load", resolve, { once: true });
+  await ContentTask.spawn(
+    gBrowser.selectedBrowser,
+    iframes,
+    async function (urls) {
+      const iframesLoadPromises = urls.map((url, i) => {
+        const iframe = content.document.createElement("iframe");
+        iframe.classList.add(`iframe-${i + 1}`);
+        const onLoadIframe = new Promise(resolve => {
+          iframe.addEventListener("load", resolve, { once: true });
+        });
+        content.document.body.append(iframe);
+        iframe.src = url;
+        return onLoadIframe;
       });
-      content.document.body.append(iframe);
-      iframe.src = url;
-      return onLoadIframe;
-    });
 
-    await Promise.all(iframesLoadPromises);
-  });
+      await Promise.all(iframesLoadPromises);
+    }
+  );
 
   return openConsole();
 }
@@ -157,7 +158,7 @@ function logAllStoreChanges(hud) {
     );
     info(
       "messages : " +
-        JSON.stringify(debugMessages, function(key, value) {
+        JSON.stringify(debugMessages, function (key, value) {
           if (value && value.getGrip) {
             return value.getGrip();
           }
@@ -397,13 +398,14 @@ async function checkUniqueMessageExists(hud, msg, typeSelector) {
     });
   } catch (e) {
     ok(false, `Message "${msg}" wasn't logged\n`);
-    return;
+    return null;
   }
 
   is(messages.length, 1, `"${msg}" was logged once`);
   const [messageEl] = messages;
   const repeatNode = messageEl.querySelector(".message-repeats");
   is(repeatNode, null, `"${msg}" wasn't repeated`);
+  return messageEl;
 }
 
 /**
@@ -625,8 +627,10 @@ async function checkClickOnNode(
     const column = frameLinkNode.getAttribute("data-column");
     ok(column, `source column found ("${column}")`);
 
+    // Redux location object uses 0-based column, while we display a 1-based one.
     is(
-      parseInt(dbg._selectors.getSelectedLocation(dbg._getState()).column, 10),
+      parseInt(dbg._selectors.getSelectedLocation(dbg._getState()).column, 10) +
+        1,
       parseInt(column, 10),
       "expected source column"
     );
@@ -634,11 +638,12 @@ async function checkClickOnNode(
 
   if (logPointExpr !== undefined && logPointExpr !== "") {
     const inputEl = dbg.panelWin.document.activeElement;
-    is(
-      inputEl.tagName,
-      "TEXTAREA",
-      "The textarea of logpoint panel is focused"
-    );
+
+    const isPanelFocused =
+      inputEl.classList.contains("cm-content") &&
+      inputEl.closest(".conditional-breakpoint-panel.log-point");
+
+    ok(isPanelFocused, "The textarea of logpoint panel is focused");
 
     const inputValue = inputEl.parentElement.parentElement.innerText.trim();
     is(
@@ -865,7 +870,7 @@ async function openDebugger(options = {}) {
     options.tab = gBrowser.selectedTab;
   }
 
-  let toolbox = await gDevTools.getToolboxForTab(options.tab);
+  let toolbox = gDevTools.getToolboxForTab(options.tab);
   const dbgPanelAlreadyOpen = toolbox && toolbox.getPanel("jsdebugger");
   if (dbgPanelAlreadyOpen) {
     await toolbox.selectTool("jsdebugger");
@@ -910,7 +915,7 @@ async function openInspector(options = {}) {
  */
 async function openNetMonitor(tab) {
   tab = tab || gBrowser.selectedTab;
-  let toolbox = await gDevTools.getToolboxForTab(tab);
+  let toolbox = gDevTools.getToolboxForTab(tab);
   if (!toolbox) {
     toolbox = await gDevTools.showToolboxForTab(tab);
   }
@@ -945,92 +950,12 @@ async function openConsole(tab) {
  *         A promise that is resolved once the web console is closed.
  */
 async function closeConsole(tab = gBrowser.selectedTab) {
-  const toolbox = await gDevTools.getToolboxForTab(tab);
+  const toolbox = gDevTools.getToolboxForTab(tab);
   if (toolbox) {
     await toolbox.destroy();
   }
 }
 
-/**
- * Fake clicking a link and return the URL we would have navigated to.
- * This function should be used to check external links since we can't access
- * network in tests.
- * This can also be used to test that a click will not be fired.
- *
- * @param ElementNode element
- *        The <a> element we want to simulate click on.
- * @param Object clickEventProps
- *        The custom properties which would be used to dispatch a click event
- * @returns Promise
- *          A Promise that is resolved when the link click simulation occured or
- *          when the click is not dispatched.
- *          The promise resolves with an object that holds the following properties
- *          - link: url of the link or null(if event not fired)
- *          - where: "tab" if tab is active or "tabshifted" if tab is inactive
- *            or null(if event not fired)
- */
-function simulateLinkClick(element, clickEventProps) {
-  return overrideOpenLink(() => {
-    if (clickEventProps) {
-      // Click on the link using the event properties.
-      element.dispatchEvent(clickEventProps);
-    } else {
-      // Click on the link.
-      element.click();
-    }
-  });
-}
-
-/**
- * Override the browserWindow open*Link function, executes the passed function and either
- * wait for:
- * - the link to be "opened"
- * - 1s before timing out
- * Then it puts back the original open*Link functions in browserWindow.
- *
- * @returns {Promise<Object>}: A promise resolving with an object of the following shape:
- * - link: The link that was "opened"
- * - where: If the link was opened in the background (null) or not ("tab").
- */
-function overrideOpenLink(fn) {
-  const browserWindow = Services.wm.getMostRecentWindow(
-    gDevTools.chromeWindowType
-  );
-
-  // Override LinkIn methods to prevent navigating.
-  const oldOpenTrustedLinkIn = browserWindow.openTrustedLinkIn;
-  const oldOpenWebLinkIn = browserWindow.openWebLinkIn;
-
-  const onOpenLink = new Promise(resolve => {
-    const openLinkIn = function(link, where) {
-      browserWindow.openTrustedLinkIn = oldOpenTrustedLinkIn;
-      browserWindow.openWebLinkIn = oldOpenWebLinkIn;
-      resolve({ link, where });
-    };
-    browserWindow.openWebLinkIn = browserWindow.openTrustedLinkIn = openLinkIn;
-    fn();
-  });
-
-  // Declare a timeout Promise that we can use to make sure openTrustedLinkIn or
-  // openWebLinkIn was not called.
-  let timeoutId;
-  const onTimeout = new Promise(function(resolve) {
-    timeoutId = setTimeout(() => {
-      browserWindow.openTrustedLinkIn = oldOpenTrustedLinkIn;
-      browserWindow.openWebLinkIn = oldOpenWebLinkIn;
-      timeoutId = null;
-      resolve({ link: null, where: null });
-    }, 1000);
-  });
-
-  onOpenLink.then(() => {
-    if (timeoutId) {
-      clearTimeout(timeoutId);
-    }
-  });
-  return Promise.race([onOpenLink, onTimeout]);
-}
-
 /**
  * Open a network request logged in the webconsole in the netmonitor panel.
  *
@@ -1346,13 +1271,55 @@ async function selectNodeWithPicker(toolbox, selector) {
  *
  * @param {HTMLElement} node: Object inspector node (.tree-node)
  */
-function expandObjectInspectorNode(node) {
+async function expandObjectInspectorNode(node) {
+  if (!node.classList.contains("tree-node")) {
+    ok(false, "Node should be a .tree-node");
+    return;
+  }
   const arrow = getObjectInspectorNodeArrow(node);
   if (!arrow) {
     ok(false, "Node can't be expanded");
     return;
   }
+  if (arrow.classList.contains("open")) {
+    ok(false, "Node already expanded");
+    return;
+  }
+  const isLongString = node.querySelector(".node > .objectBox-string");
+  let onMutation;
+  let textContentBeforeExpand;
+  if (!isLongString) {
+    const objectInspector = node.closest(".object-inspector");
+    onMutation = waitForNodeMutation(objectInspector, {
+      childList: true,
+    });
+  } else {
+    textContentBeforeExpand = node.textContent;
+  }
   arrow.click();
+
+  // Long strings are not going to be expanded into children element.
+  // Instead the tree node will update itself to show the long string.
+  // So that we can't wait for the childList mutation.
+  if (isLongString) {
+    // Reps will expand on click...
+    await waitFor(() => arrow.classList.contains("open"));
+    // ...but it will fetch the long string content asynchronously after having expanded the TreeNode.
+    // So also wait for the string to be updated and be longer.
+    await waitFor(
+      () => node.textContent.length > textContentBeforeExpand.length
+    );
+  } else {
+    await onMutation;
+    // Waiting for the object inspector mutation isn't enough,
+    // also wait for the children element, with higher aria-level to be added to the DOM.
+    await waitFor(() => !!getObjectInspectorChildrenNodes(node).length);
+  }
+
+  ok(
+    arrow.classList.contains("open"),
+    "The arrow of the root node of the tree is expanded after clicking on it"
+  );
 }
 
 /**
@@ -1362,7 +1329,7 @@ function expandObjectInspectorNode(node) {
  * @return {HTMLElement|null} the arrow element
  */
 function getObjectInspectorNodeArrow(node) {
-  return node.querySelector(".arrow");
+  return node.querySelector(".theme-twisty");
 }
 
 /**
@@ -1393,7 +1360,7 @@ function getObjectInspectorNodes(oi) {
  *                              deeper than the passed node)
  */
 function getObjectInspectorChildrenNodes(node) {
-  const getLevel = n => parseInt(n.getAttribute("aria-level"), 10);
+  const getLevel = n => parseInt(n.getAttribute("aria-level") || "0", 10);
   const level = getLevel(node);
   const childLevel = level + 1;
   const children = [];
@@ -1519,14 +1486,14 @@ function isConfirmDialogOpened(toolbox) {
 
 async function selectFrame(dbg, frame) {
   const onScopes = waitForDispatch(dbg.store, "ADD_SCOPES");
-  await dbg.actions.selectFrame(dbg.selectors.getThreadContext(), frame);
+  await dbg.actions.selectFrame(frame);
   await onScopes;
 }
 
-async function pauseDebugger(dbg) {
+async function pauseDebugger(dbg, options) {
   info("Waiting for debugger to pause");
-  const onPaused = waitForPaused(dbg);
-  SpecialPowers.spawn(gBrowser.selectedBrowser, [], function() {
+  const onPaused = waitForPaused(dbg, null, options);
+  SpecialPowers.spawn(gBrowser.selectedBrowser, [], function () {
     content.wrappedJSObject.firstCall();
   }).catch(() => {});
   await onPaused;
@@ -1859,6 +1826,7 @@ function checkContextSelectorMenu(hud, expected) {
  * @param {String} expected.tooltip: The tooltip of the target element in the menu
  * @param {Boolean} expected.checked: if the target should be selected or not
  * @param {Boolean} expected.separator: if the element is a simple separator
+ * @param {Boolean} expected.indented: if the element is indented
  */
 function checkContextSelectorMenuItemAt(hud, index, expected) {
   const el = getContextSelectorItems(hud).at(index);
@@ -1871,6 +1839,7 @@ function checkContextSelectorMenuItemAt(hud, index, expected) {
   const elChecked = el.getAttribute("aria-checked") === "true";
   const elTooltip = el.getAttribute("title");
   const elLabel = el.querySelector(".label").innerText;
+  const indented = el.classList.contains("indented");
 
   is(elLabel, expected.label, `The item has the expected label`);
   is(elTooltip, expected.tooltip, `Item "${elLabel}" has the expected tooltip`);
@@ -1879,6 +1848,11 @@ function checkContextSelectorMenuItemAt(hud, index, expected) {
     expected.checked,
     `Item "${elLabel}" is ${expected.checked ? "checked" : "unchecked"}`
   );
+  is(
+    indented,
+    expected.indented ?? false,
+    `Item "${elLabel}" is ${!indented ? " not" : ""} indented`
+  );
 }
 
 /**
@@ -1915,7 +1889,11 @@ async function getImageSizeFromClipboard() {
   trans.init(null);
   trans.addDataFlavor(flavor);
 
-  clip.getData(trans, clipid.kGlobalClipboard);
+  clip.getData(
+    trans,
+    clipid.kGlobalClipboard,
+    SpecialPowers.wrap(window).browsingContext.currentWindowContext
+  );
   const data = {};
   trans.getTransferData(flavor, data);
 
@@ -1955,33 +1933,35 @@ async function getImageSizeFromClipboard() {
   // (which is value of the global `document` here). Doing so might push the
   // toolbox upwards, shrink the content page and fail the fullpage screenshot
   // test.
-  return SpecialPowers.spawn(gBrowser.selectedBrowser, [buffer], async function(
-    _buffer
-  ) {
-    const img = content.document.createElement("img");
-    const loaded = new Promise(r => {
-      img.addEventListener("load", r, { once: true });
-    });
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [buffer],
+    async function (_buffer) {
+      const img = content.document.createElement("img");
+      const loaded = new Promise(r => {
+        img.addEventListener("load", r, { once: true });
+      });
 
-    // Build a URL from the buffer passed to the ContentTask
-    const url = content.URL.createObjectURL(
-      new Blob([_buffer], { type: "image/png" })
-    );
+      // Build a URL from the buffer passed to the ContentTask
+      const url = content.URL.createObjectURL(
+        new Blob([_buffer], { type: "image/png" })
+      );
 
-    // Load the image
-    img.src = url;
-    content.document.documentElement.appendChild(img);
+      // Load the image
+      img.src = url;
+      content.document.documentElement.appendChild(img);
 
-    info("Waiting for the clipboard image to load in the content page");
-    await loaded;
+      info("Waiting for the clipboard image to load in the content page");
+      await loaded;
 
-    // Remove the image and revoke the URL.
-    img.remove();
-    content.URL.revokeObjectURL(url);
+      // Remove the image and revoke the URL.
+      img.remove();
+      content.URL.revokeObjectURL(url);
 
-    return {
-      width: img.width,
-      height: img.height,
-    };
-  });
+      return {
+        width: img.width,
+        height: img.height,
+      };
+    }
+  );
 }
