@@ -47,12 +47,14 @@ const kStateHover = 0x00000004; // ElementState::HOVER
  */
 class TouchSimulator {
   /**
-   * @param {ChromeEventHandler} simulatorTarget: The object we'll use to listen for click
-   *                             and touch events to handle.
+   * @param {WindowGlobalTargetActor} windowTarget: The window object we'll use
+   *                                  to listen for click and touch events to handle.
    */
-  constructor(simulatorTarget) {
-    this.simulatorTarget = simulatorTarget;
+  constructor(windowTarget) {
+    this.windowTarget = windowTarget;
+    this.simulatorTarget = windowTarget.chromeEventHandler;
     this._currentPickerMap = new Map();
+    this.previousScreenY = 0;
   }
 
   enabled = false;
@@ -210,15 +212,12 @@ class TouchSimulator {
 
         this.startX = evt.pageX;
         this.startY = evt.pageY;
-
-        // Capture events so if a different window show up the events
-        // won't be dispatched to something else.
-        evt.target.setCapture(false);
+        this.previousScreenY = this.startY;
 
         type = "touchstart";
         break;
 
-      case "mousemove":
+      case "mousemove": {
         if (!eventTarget) {
           // Don't propagate mousemove event when touchstart event isn't fired
           evt.stopPropagation();
@@ -226,7 +225,11 @@ class TouchSimulator {
         }
 
         type = "touchmove";
+        const deltaY = evt.screenY - this.previousScreenY;
+        this.previousScreenY = evt.screenY;
+        this.windowTarget.emit("contentScrolled", deltaY);
         break;
+      }
 
       case "mouseup":
         if (!eventTarget) {
@@ -260,8 +263,7 @@ class TouchSimulator {
 
   sendContextMenu({ target, clientX, clientY, screenX, screenY }) {
     const view = target.ownerGlobal;
-    const { MouseEvent } = view;
-    const evt = new MouseEvent("contextmenu", {
+    const evt = new view.PointerEvent("contextmenu", {
       bubbles: true,
       cancelable: true,
       view,
