@@ -58,18 +58,26 @@ class CookiesStorageActor extends BaseStorageActor {
     super.destroy();
   }
 
+  static UNIQUE_KEY_INDEXES = { name: 0, host: 1, path: 2, partitionKey: 3 };
+
+  #getCookieUniqueKey(cookie) {
+    return (
+      cookie.name +
+      SEPARATOR_GUID +
+      cookie.host +
+      SEPARATOR_GUID +
+      cookie.path +
+      SEPARATOR_GUID +
+      cookie.originAttributes.partitionKey
+    );
+  }
+
   populateStoresForHost(host) {
     this.hostVsStores.set(host, new Map());
-
-    const originAttributes = this.getOriginAttributesFromHost(host);
-    const cookies = this.getCookiesFromHost(host, originAttributes);
-
+    const cookies = this.getCookiesFromHost(host);
     for (const cookie of cookies) {
       if (this.isCookieAtHost(cookie, host)) {
-        const uniqueKey =
-          `${cookie.name}${SEPARATOR_GUID}${cookie.host}` +
-          `${SEPARATOR_GUID}${cookie.path}`;
-
+        const uniqueKey = this.#getCookieUniqueKey(cookie);
         this.hostVsStores.get(host).set(uniqueKey, cookie);
       }
     }
@@ -84,14 +92,61 @@ class CookiesStorageActor extends BaseStorageActor {
     } else {
       // If we can't find the window by host, fallback to the top window
       // origin attributes.
-      originAttributes = this.storageActor.document?.effectiveStoragePrincipal
-        .originAttributes;
+      originAttributes =
+        this.storageActor.document?.effectiveStoragePrincipal.originAttributes;
     }
 
     return originAttributes;
   }
 
-  getCookiesFromHost(host, originAttributes) {
+  getCookiesFromHost(host) {
+    // Gather originAttributes list from host
+    const hostBrowsingContexts =
+      this.storageActor.getBrowsingContextsFromHost(host);
+    const originAttributesList = [];
+    if (hostBrowsingContexts.length) {
+      // Since we need to get all browsing contexts to get their originAttributes,
+      // we might get "duplicated" objects, which would translate into having the same
+      // cookies multiple times.
+      // To avoid that, we compute a unique key from originAttributes to only have unique ones.
+      const uniqueOriginAttributes = new Set();
+      for (const bc of hostBrowsingContexts) {
+        const { originAttributes } =
+          bc.currentWindowGlobal.documentStoragePrincipal;
+        // The object is small, seems fine to stringify it to compute a unique key
+        const oaKey = JSON.stringify(originAttributes);
+        if (!uniqueOriginAttributes.has(oaKey)) {
+          originAttributesList.push(originAttributes);
+          uniqueOriginAttributes.add(oaKey);
+        }
+
+        // A document might have an empty partitionKey in browsingContext.currentWindowGlobal.documentStoragePrincipal.originAttributes,
+        // (e.g. a top level document), but still have partitioned cookies, in a different jar
+        // (in CHIPS, for top level document that's first-party partitioned cookies).
+        // In order to retrieve those, we create a new originAttribute with the
+        // partitionKey from the window global cookie jar partitionKey
+        if (
+          bc.currentWindowGlobal.cookieJarSettings.partitionKey !==
+          originAttributes.partitionKey
+        ) {
+          const derivedOriginAttributes = {
+            ...originAttributes,
+            partitionKey: bc.currentWindowGlobal.cookieJarSettings.partitionKey,
+          };
+          const derivedOaKey = JSON.stringify(derivedOriginAttributes);
+          if (!uniqueOriginAttributes.has(derivedOaKey)) {
+            originAttributesList.push(derivedOriginAttributes);
+            uniqueOriginAttributes.add(derivedOaKey);
+          }
+        }
+      }
+    } else {
+      // In case of WebExtension or BrowserToolbox, we may pass privileged hosts
+      // which don't relate to any particular window. getOriginAttributesFromHost will
+      // fallback to the top window origin attributes.
+      originAttributesList.push(this.getOriginAttributesFromHost(host));
+    }
+
     // Local files have no host.
     if (host.startsWith("file:///")) {
       host = "";
@@ -99,7 +154,20 @@ class CookiesStorageActor extends BaseStorageActor {
 
     host = trimHttpHttpsPort(host);
 
-    return Services.cookies.getCookiesFromHost(host, originAttributes);
+    // Retrieve cookies all the passed originAttributes so we can get cookies from all jars
+    let cookies;
+    for (const originAttributes of originAttributesList) {
+      const oaCookies = Services.cookies.getCookiesFromHost(
+        host,
+        originAttributes
+      );
+      if (!cookies) {
+        cookies = oaCookies;
+      } else {
+        cookies.push(...oaCookies);
+      }
+    }
+    return cookies || [];
   }
 
   /**
@@ -107,6 +175,9 @@ class CookiesStorageActor extends BaseStorageActor {
    * the cookie belong to.
    */
   getMatchingHosts(cookies) {
+    if (!cookies) {
+      return [];
+    }
     if (!cookies.length) {
       cookies = [cookies];
     }
@@ -147,10 +218,8 @@ class CookiesStorageActor extends BaseStorageActor {
       return null;
     }
 
-    return {
-      uniqueKey:
-        `${cookie.name}${SEPARATOR_GUID}${cookie.host}` +
-        `${SEPARATOR_GUID}${cookie.path}`,
+    const obj = {
+      uniqueKey: this.#getCookieUniqueKey(cookie),
       name: cookie.name,
       host: cookie.host || "",
       path: cookie.path || "",
@@ -171,6 +240,23 @@ class CookiesStorageActor extends BaseStorageActor {
       isHttpOnly: cookie.isHttpOnly,
       sameSite: this.getSameSiteStringFromCookie(cookie),
     };
+
+    if (cookie.isPartitioned) {
+      const rawPartitionKey = cookie.originAttributes.partitionKey;
+      // We need to return the site derived from the partition key.
+      // rawPartitionKey format should be like "(<scheme>,<baseDomain>,[port],[ancestorbit])"
+      // see https://searchfox.org/mozilla-central/rev/23efe2c8c5b3a3182d449211ff9036fb34fe0219/caps/OriginAttributes.h#132-138
+      // We can ignore the `ancestorbit` part.
+      const [scheme, baseDomain, port] = rawPartitionKey
+        .replace(/(?<openingparen>^\()|(?<closingparen>\)$)/g, "")
+        .split(",");
+      const partitionKey = `${scheme}://${baseDomain}${
+        port !== undefined && /^\d+$/.test(port) ? ":" + port : ""
+      }`;
+      obj.partitionKey = partitionKey;
+    }
+
+    return obj;
   }
 
   getSameSiteStringFromCookie(cookie) {
@@ -179,59 +265,55 @@ class CookiesStorageActor extends BaseStorageActor {
         return COOKIE_SAMESITE.LAX;
       case cookie.SAMESITE_STRICT:
         return COOKIE_SAMESITE.STRICT;
+      case cookie.SAMESITE_NONE:
+        return COOKIE_SAMESITE.NONE;
     }
-    // cookie.SAMESITE_NONE
-    return COOKIE_SAMESITE.NONE;
+    // cookie.SAMESITE_UNSET
+    return "";
   }
 
   /**
    * Notification observer for "cookie-change".
    *
-   * @param subject
-   *        {Cookie|[Array]} A JSON parsed object containing either a single
-   *        cookie representation or an array. Array is only in case of
-   *        a "batch-deleted" action.
-   * @param {string} topic
-   *        The topic of the notification.
-   * @param {string} action
-   *        Additional data associated with the notification. Its the type of
-   *        cookie change in the "cookie-change" topic.
-   */
-  onCookieChanged(subject, topic, action) {
-    if (
-      (topic !== "cookie-changed" && topic !== "private-cookie-changed") ||
-      !this.storageActor ||
-      !this.storageActor.windows
-    ) {
-      return null;
+   * @param {(nsICookie|nsICookie[])} cookie - Cookie/s changed. Depending on the action
+   * this is either null, a single cookie or an array of cookies.
+   * @param {nsICookieNotification_Action} action - The cookie operation, see
+   * nsICookieNotification for details.
+   **/
+  onCookieChanged(cookie, action) {
+    const {
+      COOKIE_ADDED,
+      COOKIE_CHANGED,
+      COOKIE_DELETED,
+      COOKIES_BATCH_DELETED,
+      ALL_COOKIES_CLEARED,
+    } = Ci.nsICookieNotification;
+
+    const hosts = this.getMatchingHosts(cookie);
+    if (!hosts.length) {
+      return;
     }
 
-    const hosts = this.getMatchingHosts(subject);
     const data = {};
 
     switch (action) {
-      case "added":
-      case "changed":
+      case COOKIE_ADDED:
+      case COOKIE_CHANGED:
         if (hosts.length) {
           for (const host of hosts) {
-            const uniqueKey =
-              `${subject.name}${SEPARATOR_GUID}${subject.host}` +
-              `${SEPARATOR_GUID}${subject.path}`;
-
-            this.hostVsStores.get(host).set(uniqueKey, subject);
+            const uniqueKey = this.#getCookieUniqueKey(cookie);
+            this.hostVsStores.get(host).set(uniqueKey, cookie);
             data[host] = [uniqueKey];
           }
-          this.storageActor.update(action, "cookies", data);
+          const actionStr = action == COOKIE_ADDED ? "added" : "changed";
+          this.storageActor.update(actionStr, "cookies", data);
         }
         break;
 
-      case "deleted":
+      case COOKIE_DELETED:
         if (hosts.length) {
           for (const host of hosts) {
-            const uniqueKey =
-              `${subject.name}${SEPARATOR_GUID}${subject.host}` +
-              `${SEPARATOR_GUID}${subject.path}`;
-
+            const uniqueKey = this.#getCookieUniqueKey(cookie);
             this.hostVsStores.get(host).delete(uniqueKey);
             data[host] = [uniqueKey];
           }
@@ -239,15 +321,13 @@ class CookiesStorageActor extends BaseStorageActor {
         }
         break;
 
-      case "batch-deleted":
+      case COOKIES_BATCH_DELETED:
         if (hosts.length) {
           for (const host of hosts) {
             const stores = [];
-            for (const cookie of subject) {
-              const uniqueKey =
-                `${cookie.name}${SEPARATOR_GUID}${cookie.host}` +
-                `${SEPARATOR_GUID}${cookie.path}`;
-
+            // For COOKIES_BATCH_DELETED cookie is an array.
+            for (const batchCookie of cookie) {
+              const uniqueKey = this.#getCookieUniqueKey(batchCookie);
               this.hostVsStores.get(host).delete(uniqueKey);
               stores.push(uniqueKey);
             }
@@ -257,7 +337,7 @@ class CookiesStorageActor extends BaseStorageActor {
         }
         break;
 
-      case "cleared":
+      case ALL_COOKIES_CLEARED:
         if (hosts.length) {
           for (const host of hosts) {
             data[host] = [];
@@ -266,11 +346,10 @@ class CookiesStorageActor extends BaseStorageActor {
         }
         break;
     }
-    return null;
   }
 
   async getFields() {
-    return [
+    const fields = [
       { name: "uniqueKey", editable: false, private: true },
       { name: "name", editable: true, hidden: false },
       { name: "value", editable: true, hidden: false },
@@ -285,6 +364,12 @@ class CookiesStorageActor extends BaseStorageActor {
       { name: "creationTime", editable: false, hidden: true },
       { name: "hostOnly", editable: false, hidden: true },
     ];
+
+    if (Services.prefs.getBoolPref("network.cookie.CHIPS.enabled", false)) {
+      fields.push({ name: "partitionKey", editable: false, hidden: false });
+    }
+
+    return fields;
   }
 
   /**
@@ -292,33 +377,50 @@ class CookiesStorageActor extends BaseStorageActor {
    *
    * @param {Object} data
    *        See editCookie() for format details.
+   * @returns {Object} An object with an "errorString" property.
    */
   async editItem(data) {
-    data.originAttributes = this.getOriginAttributesFromHost(data.host);
-    this.editCookie(data);
+    const potentialErrorMessage = this.editCookie(data);
+    return { errorString: potentialErrorMessage };
   }
 
+  /**
+   * Add a cookie on given host
+   *
+   * @param {String} guid
+   * @param {String} host
+   * @returns {Object} An object with an "errorString" property.
+   */
   async addItem(guid, host) {
     const window = this.storageActor.getWindowFromHost(host);
     const principal = window.document.effectiveStoragePrincipal;
-    this.addCookie(guid, principal);
+    const potentialErrorMessage = this.addCookie(guid, principal);
+    return { errorString: potentialErrorMessage };
   }
 
-  async removeItem(host, name) {
-    const originAttributes = this.getOriginAttributesFromHost(host);
-    this.removeCookie(host, name, originAttributes);
+  async removeItem(host, uniqueKey) {
+    if (uniqueKey === undefined) {
+      return;
+    }
+    this._removeCookies(host, { uniqueKey });
   }
 
   async removeAll(host, domain) {
-    const originAttributes = this.getOriginAttributesFromHost(host);
-    this.removeAllCookies(host, domain, originAttributes);
+    this._removeCookies(host, { domain });
   }
 
   async removeAllSessionCookies(host, domain) {
-    const originAttributes = this.getOriginAttributesFromHost(host);
-    this._removeCookies(host, { domain, originAttributes, session: true });
+    this._removeCookies(host, { domain, session: true });
   }
 
+  /**
+   * Add a cookie on given principal
+   *
+   * @param {String} guid
+   * @param {Principal} principal
+   * @returns {String|null} If the cookie couldn't be added (e.g. it's invalid),
+   *          an error string will be returned.
+   */
   addCookie(guid, principal) {
     // Set expiry time for cookie 1 day into the future
     // NOTE: Services.cookies.add expects the time in seconds.
@@ -331,11 +433,10 @@ class CookiesStorageActor extends BaseStorageActor {
     // We check for asciiHost instead, which is always present, and has a
     // value of "" when the host is not available.
     const domain = principal.asciiHost ? principal.host : principal.baseDomain;
-    const path = principal.filePath.startsWith("/") ? principal.filePath : "/";
 
-    Services.cookies.add(
+    const cv = Services.cookies.add(
       domain,
-      path,
+      "/",
       guid, // name
       DEFAULT_VALUE, // value
       false, // isSecure
@@ -348,6 +449,12 @@ class CookiesStorageActor extends BaseStorageActor {
         ? Ci.nsICookie.SCHEME_HTTPS
         : Ci.nsICookie.SCHEME_HTTP
     );
+
+    if (cv.result != Ci.nsICookieValidation.eOK) {
+      return cv.errorString;
+    }
+
+    return null;
   }
 
   /**
@@ -374,6 +481,8 @@ class CookiesStorageActor extends BaseStorageActor {
    *            isHttpOnly: "false"
    *          }
    *        }
+   * @returns {(String|null)} If cookie couldn't be updated (e.g. it's invalid), an error string
+   *          will be returned.
    */
   // eslint-disable-next-line complexity
   editCookie(data) {
@@ -381,17 +490,22 @@ class CookiesStorageActor extends BaseStorageActor {
     const origName = field === "name" ? oldValue : data.items.name;
     const origHost = field === "host" ? oldValue : data.items.host;
     const origPath = field === "path" ? oldValue : data.items.path;
+    // We can't use `data.items.partitionKey` as it's the formatted value and we need
+    // to check against the "raw" one. Its value can't be modified, so we don't need to
+    // look into oldValue.
+    const partitionKey =
+      data.items.uniqueKey.split(SEPARATOR_GUID)[
+        CookiesStorageActor.UNIQUE_KEY_INDEXES.partitionKey
+      ];
     let cookie = null;
 
-    const cookies = Services.cookies.getCookiesFromHost(
-      origHost,
-      data.originAttributes || {}
-    );
+    const cookies = this.getCookiesFromHost(data.host);
     for (const nsiCookie of cookies) {
       if (
         nsiCookie.name === origName &&
         nsiCookie.host === origHost &&
-        nsiCookie.path === origPath
+        nsiCookie.path === origPath &&
+        nsiCookie.originAttributes.partitionKey === partitionKey
       ) {
         cookie = {
           host: nsiCookie.host,
@@ -403,14 +517,16 @@ class CookiesStorageActor extends BaseStorageActor {
           isSession: nsiCookie.isSession,
           expires: nsiCookie.expires,
           originAttributes: nsiCookie.originAttributes,
+          sameSite: nsiCookie.sameSite,
           schemeMap: nsiCookie.schemeMap,
+          isPartitioned: nsiCookie.isPartitioned,
         };
         break;
       }
     }
 
     if (!cookie) {
-      return;
+      return null;
     }
 
     // If the date is expired set it for 10 seconds in the future.
@@ -421,6 +537,8 @@ class CookiesStorageActor extends BaseStorageActor {
       cookie.expires = tenSecondsFromNow;
     }
 
+    let origCookieRemoved = false;
+
     switch (field) {
       case "isSecure":
       case "isHttpOnly":
@@ -446,6 +564,7 @@ class CookiesStorageActor extends BaseStorageActor {
           origPath,
           cookie.originAttributes
         );
+        origCookieRemoved = true;
         break;
     }
 
@@ -457,7 +576,7 @@ class CookiesStorageActor extends BaseStorageActor {
     cookie.isSession = !cookie.expires;
 
     // Add the edited cookie.
-    Services.cookies.add(
+    const cv = Services.cookies.add(
       cookie.host,
       cookie.path,
       cookie.name,
@@ -468,42 +587,59 @@ class CookiesStorageActor extends BaseStorageActor {
       cookie.isSession ? MAX_COOKIE_EXPIRY : cookie.expires,
       cookie.originAttributes,
       cookie.sameSite,
-      cookie.schemeMap
+      cookie.schemeMap,
+      cookie.isPartitioned
     );
+
+    if (cv.result != Ci.nsICookieValidation.eOK) {
+      if (origCookieRemoved) {
+        // Re-add the cookie with the original values if it was removed.
+        Services.cookies.add(
+          origHost,
+          origPath,
+          origName,
+          cookie.value,
+          cookie.isSecure,
+          cookie.isHttpOnly,
+          cookie.isSession,
+          cookie.isSession ? MAX_COOKIE_EXPIRY : cookie.expires,
+          cookie.originAttributes,
+          cookie.sameSite,
+          cookie.schemeMap,
+          cookie.isPartitioned
+        );
+      }
+
+      return cv.errorString;
+    }
+
+    return null;
   }
 
   _removeCookies(host, opts = {}) {
     // We use a uniqueId to emulate compound keys for cookies. We need to
     // extract the cookie name to remove the correct cookie.
-    if (opts.name) {
-      const split = opts.name.split(SEPARATOR_GUID);
+    if (opts.uniqueKey) {
+      const uniqueKeyParts = opts.uniqueKey.split(SEPARATOR_GUID);
 
-      opts.name = split[0];
-      opts.path = split[2];
+      opts.name = uniqueKeyParts[CookiesStorageActor.UNIQUE_KEY_INDEXES.name];
+      opts.path = uniqueKeyParts[CookiesStorageActor.UNIQUE_KEY_INDEXES.path];
+      opts.partitionKey =
+        uniqueKeyParts[CookiesStorageActor.UNIQUE_KEY_INDEXES.partitionKey] ||
+        "";
     }
 
-    host = trimHttpHttpsPort(host);
-
-    function hostMatches(cookieHost, matchHost) {
-      if (cookieHost == null) {
-        return matchHost == null;
-      }
-      if (cookieHost.startsWith(".")) {
-        return ("." + matchHost).endsWith(cookieHost);
-      }
-      return cookieHost == host;
-    }
-
-    const cookies = Services.cookies.getCookiesFromHost(
-      host,
-      opts.originAttributes || {}
-    );
+    const cookies = this.getCookiesFromHost(host);
     for (const cookie of cookies) {
       if (
-        hostMatches(cookie.host, host) &&
+        this.isCookieAtHost(cookie, host) &&
         (!opts.name || cookie.name === opts.name) &&
         (!opts.domain || cookie.host === opts.domain) &&
         (!opts.path || cookie.path === opts.path) &&
+        (!opts.uniqueKey ||
+          // make sure to pick the cookie from the correct jar
+          cookie.originAttributes.partitionKey === opts.partitionKey) &&
+        // for session cookie removal
         (!opts.session || (!cookie.expires && !cookie.maxAge))
       ) {
         Services.cookies.remove(
@@ -526,29 +662,32 @@ class CookiesStorageActor extends BaseStorageActor {
     this._removeCookies(host, { domain, originAttributes });
   }
 
-  observe(subject, topic, data) {
+  observe(subject, topic) {
     if (
       !subject ||
-      (topic != "cookie-changed" && topic != "private-cookie-changed")
+      (topic != "cookie-changed" && topic != "private-cookie-changed") ||
+      !this.storageActor ||
+      !this.storageActor.windows
     ) {
       return;
     }
 
-    if (data === "batch-deleted") {
-      const cookiesNoInterface = subject.QueryInterface(Ci.nsIArray);
-      const cookies = [];
-
+    const notification = subject.QueryInterface(Ci.nsICookieNotification);
+    let cookie;
+    if (notification.action == Ci.nsICookieNotification.COOKIES_BATCH_DELETED) {
+      // Extract the batch deleted cookies from nsIArray.
+      const cookiesNoInterface =
+        notification.batchDeletedCookies.QueryInterface(Ci.nsIArray);
+      cookie = [];
       for (let i = 0; i < cookiesNoInterface.length; i++) {
-        const cookie = cookiesNoInterface.queryElementAt(i, Ci.nsICookie);
-        cookies.push(cookie);
+        cookie.push(cookiesNoInterface.queryElementAt(i, Ci.nsICookie));
       }
-      this.onCookieChanged(cookies, topic, data);
-
-      return;
+    } else if (notification.cookie) {
+      // Otherwise, get the single cookie affected by the operation.
+      cookie = notification.cookie.QueryInterface(Ci.nsICookie);
     }
 
-    const cookie = subject.QueryInterface(Ci.nsICookie);
-    this.onCookieChanged(cookie, topic, data);
+    this.onCookieChanged(cookie, notification.action);
   }
 }
 exports.CookiesStorageActor = CookiesStorageActor;
