@@ -13,16 +13,20 @@ function run_test() {
   );
   const g = createTestGlobal("test1");
 
-  const dbg = makeDebugger();
+  const dbg = makeDebugger({
+    shouldAddNewGlobalAsDebuggee() {
+      return true;
+    },
+  });
   dbg.uncaughtExceptionHook = testExceptionHook;
 
   dbg.addDebuggee(g);
-  dbg.onDebuggerStatement = function(frame) {
+  dbg.onDebuggerStatement = function (frame) {
     Assert.ok(frame === dbg.getNewestFrame());
     // Execute from the nested event loop, dbg.getNewestFrame() won't
     // be working anymore.
 
-    executeSoon(function() {
+    executeSoon(function () {
       try {
         Assert.ok(frame === dbg.getNewestFrame());
       } finally {
