@@ -13,6 +13,8 @@ function cacheKey(resourceType, resourceId) {
 }
 
 class ResourceCommand {
+  #destroyed = false;
+
   /**
    * This class helps retrieving existing and listening to resources.
    * A resource is something that:
@@ -27,12 +29,12 @@ class ResourceCommand {
   constructor({ commands }) {
     this.targetCommand = commands.targetCommand;
 
+    // Public attribute set by tests to disable throttling
+    this.throttlingDisabled = false;
+
     this._onTargetAvailable = this._onTargetAvailable.bind(this);
     this._onTargetDestroyed = this._onTargetDestroyed.bind(this);
 
-    this._onResourceAvailable = this._onResourceAvailable.bind(this);
-    this._onResourceDestroyed = this._onResourceDestroyed.bind(this);
-
     // Array of all the currently registered watchers, which contains object with attributes:
     // - {String} resources: list of all resource watched by this one watcher
     // - {Function} onAvailable: watcher's function to call when a new resource is available
@@ -61,8 +63,17 @@ class ResourceCommand {
     // we don't have listeners registered twice.
     this._offTargetFrontListeners = new Map();
 
+    // Bug 1914386: We used to throttle the resource on client and should try to remove it entirely.
+    const throttleDelay = 0;
     this._notifyWatchers = this._notifyWatchers.bind(this);
-    this._throttledNotifyWatchers = throttle(this._notifyWatchers, 100);
+    this._throttledNotifyWatchers = throttle(
+      this._notifyWatchers,
+      throttleDelay
+    );
+  }
+
+  destroy() {
+    this.#destroyed = true;
   }
 
   get watcherFront() {
@@ -71,6 +82,9 @@ class ResourceCommand {
 
   addResourceToCache(resource) {
     const { resourceId, resourceType } = resource;
+    if (TRANSIENT_RESOURCE_TYPES.includes(resourceType)) {
+      return;
+    }
     this._cache.set(cacheKey(resourceType, resourceId), resource);
   }
 
@@ -176,6 +190,10 @@ class ResourceCommand {
       }
     }
 
+    // Copy the array in order to avoid the callsite to modify the list of watched resources by mutating the array.
+    // You have to call (un)watchResources to update the list of resources being watched!
+    resources = [...resources];
+
     // Pending watchers are used in unwatchResources to remove watchers which
     // are not fully registered yet. Store `onAvailable` which is the unique key
     // for a watcher, as well as the resources array, so that unwatchResources
@@ -192,18 +210,20 @@ class ResourceCommand {
       // Resources watched from the parent process will be emitted on the Watcher Actor.
       // So that we also have to listen for this event on it, in addition to all targets.
       this.watcherFront.on(
-        "resource-available-form",
-        this._onResourceAvailable.bind(this, {
+        "resources-available-array",
+        this._onResourceAvailableArray.bind(this, {
           watcherFront: this.watcherFront,
         })
       );
       this.watcherFront.on(
-        "resource-updated-form",
-        this._onResourceUpdated.bind(this, { watcherFront: this.watcherFront })
+        "resources-updated-array",
+        this._onResourceUpdatedArray.bind(this, {
+          watcherFront: this.watcherFront,
+        })
       );
       this.watcherFront.on(
-        "resource-destroyed-form",
-        this._onResourceDestroyed.bind(this, {
+        "resources-destroyed-array",
+        this._onResourceDestroyedArray.bind(this, {
           watcherFront: this.watcherFront,
         })
       );
@@ -411,9 +431,15 @@ class ResourceCommand {
   async _startLegacyListenersForExistingTargets(resourceType) {
     // If we were already listening to targets, we want to start the legacy listeners
     // for all already existing targets.
+    //
+    // Only try instantiating the legacy listener, if this resource type:
+    //   - has legacy listener implementation
+    // (new resource types may not be supported by old runtime and just not be received without breaking anything)
+    //   - isn't supported by the server, or, the target type requires the a legacy listener implementation.
     const shouldRunLegacyListeners =
-      !this.hasResourceCommandSupport(resourceType) ||
-      this._shouldRunLegacyListenerEvenWithWatcherSupport(resourceType);
+      resourceType in LegacyListeners &&
+      (!this.hasResourceCommandSupport(resourceType) ||
+        this._shouldRunLegacyListenerEvenWithWatcherSupport(resourceType));
     if (shouldRunLegacyListeners) {
       const promises = [];
       const targets = this.targetCommand.getAllTargets(
@@ -495,24 +521,24 @@ class ResourceCommand {
     // We do call Watcher.watchResources, but the events are fired on the target.
     // That's because the Watcher runs in the parent process/main thread, while resources
     // are available from the target's process/thread.
-    const offResourceAvailable = targetFront.on(
-      "resource-available-form",
-      this._onResourceAvailable.bind(this, { targetFront })
+    const offResourceAvailableArray = targetFront.on(
+      "resources-available-array",
+      this._onResourceAvailableArray.bind(this, { targetFront })
     );
-    const offResourceUpdated = targetFront.on(
-      "resource-updated-form",
-      this._onResourceUpdated.bind(this, { targetFront })
+    const offResourceUpdatedArray = targetFront.on(
+      "resources-updated-array",
+      this._onResourceUpdatedArray.bind(this, { targetFront })
     );
-    const offResourceDestroyed = targetFront.on(
-      "resource-destroyed-form",
-      this._onResourceDestroyed.bind(this, { targetFront })
+    const offResourceDestroyedArray = targetFront.on(
+      "resources-destroyed-array",
+      this._onResourceDestroyedArray.bind(this, { targetFront })
     );
 
     const offList = this._offTargetFrontListeners.get(targetFront) || [];
     offList.push(
-      offResourceAvailable,
-      offResourceUpdated,
-      offResourceDestroyed
+      offResourceAvailableArray,
+      offResourceUpdatedArray,
+      offResourceDestroyedArray
     );
 
     if (isTargetSwitching) {
@@ -559,8 +585,8 @@ class ResourceCommand {
     // and only become true when doing a bfcache navigation.
     // (only server side targets follow the WindowGlobal lifecycle)
     // When server side targets are enabled, this will always be true.
-    const isServerSideTarget = this.targetCommand.targetFront.targetForm
-      .followWindowGlobalLifeCycle;
+    const isServerSideTarget =
+      this.targetCommand.targetFront.targetForm.followWindowGlobalLifeCycle;
     if (isServerSideTarget) {
       // For top-level targets created from the server, only restart legacy
       // listeners.
@@ -615,91 +641,93 @@ class ResourceCommand {
     }
   }
 
-  /**
-   * Method called either by:
-   * - the backward compatibility code (LegacyListeners)
-   * - target actors RDP events
-   * whenever an already existing resource is being listed or when a new one
-   * has been created.
-   *
-   * @param {Object} source
-   *        A dictionary object with only one of these two attributes:
-   *        - targetFront: a Target Front, if the resource is watched from the target process or thread
-   *        - watcherFront: a Watcher Front, if the resource is watched from the parent process
-   * @param {Array<json/Front>} resources
-   *        Depending on the resource Type, it can be an Array composed of either JSON objects or Fronts,
-   *        which describes the resource.
-   */
-  async _onResourceAvailable({ targetFront, watcherFront }, resources) {
+  async _onResourceAvailableArray({ targetFront, watcherFront }, array) {
     let includesDocumentEventWillNavigate = false;
     let includesDocumentEventDomLoading = false;
-    for (let resource of resources) {
-      const { resourceType } = resource;
+    for (const [resourceType, resources] of array) {
+      const isAlreadyExistingResource =
+        this._processingExistingResources.has(resourceType);
+      const transformer = ResourceTransformers[resourceType];
 
-      if (watcherFront) {
-        targetFront = await this._getTargetForWatcherResource(resource);
-        // When we receive resources from the Watcher actor,
-        // there is no guarantee that the target front is fully initialized.
-        // The Target Front is initialized by the TargetCommand, by calling TargetFront.attachAndInitThread.
-        // We have to wait for its completion as resources watchers are expecting it to be completed.
-        //
-        // But when navigating, we may receive resources packets for a destroyed target.
-        // Or, in the context of the browser toolbox, they may not relate to any target.
-        if (targetFront) {
-          await targetFront.initialized;
+      for (let i = 0; i < resources.length; i++) {
+        let resource = resources[i];
+        if (!("resourceType" in resource)) {
+          resource.resourceType = resourceType;
+        }
+
+        if (watcherFront) {
+          try {
+            targetFront = await this._getTargetForWatcherResource(resource);
+          } catch (e) {
+            if (this.#destroyed) {
+              // If the resource-command was destroyed while waiting for
+              // _getTargetForWatcherResource, swallow the error.
+              return;
+            }
+            throw e;
+          }
+          // When we receive resources from the Watcher actor,
+          // there is no guarantee that the target front is fully initialized.
+          // The Target Front is initialized by the TargetCommand, by calling TargetFront.attachAndInitThread.
+          // We have to wait for its completion as resources watchers are expecting it to be completed.
+          //
+          // But when navigating, we may receive resources packets for a destroyed target.
+          // Or, in the context of the browser toolbox, they may not relate to any target.
+          if (targetFront) {
+            await targetFront.initialized;
+          }
+        }
+
+        // Put the targetFront on the resource for easy retrieval.
+        // (Resources from the legacy listeners may already have the attribute set)
+        if (!resource.targetFront) {
+          resource.targetFront = targetFront;
+        }
+
+        if (transformer) {
+          resource = transformer({
+            resource,
+            targetCommand: this.targetCommand,
+            targetFront,
+            watcherFront: this.watcherFront,
+          });
+          resources[i] = resource;
+        }
+
+        // isAlreadyExistingResource indicates that the resources already existed before
+        // the resource command started watching for this type of resource.
+        resource.isAlreadyExistingResource = isAlreadyExistingResource;
+
+        if (!resource.resourceId) {
+          resource.resourceId = `auto:${++gLastResourceId}`;
+        }
+
+        // Only consider top level document, and ignore remote iframes top document
+        let isWillNavigate = false;
+        if (resourceType == DOCUMENT_EVENT) {
+          isWillNavigate = resource.name === "will-navigate";
+          if (isWillNavigate && resource.targetFront.isTopLevel) {
+            includesDocumentEventWillNavigate = true;
+            this._onWillNavigate(resource.targetFront);
+          }
+
+          if (
+            resource.name === "dom-loading" &&
+            resource.targetFront.isTopLevel
+          ) {
+            includesDocumentEventDomLoading = true;
+          }
+        }
+
+        // Avoid storing will-navigate resource and consider it as a transcient resource.
+        // We do that to prevent leaking this resource (and its target) on navigation.
+        // We do clear the cache in _onWillNavigate, that we call a few lines before this.
+        if (!isWillNavigate) {
+          this.addResourceToCache(resource);
         }
       }
 
-      // isAlreadyExistingResource indicates that the resources already existed before
-      // the resource command started watching for this type of resource.
-      resource.isAlreadyExistingResource = this._processingExistingResources.has(
-        resourceType
-      );
-
-      // Put the targetFront on the resource for easy retrieval.
-      // (Resources from the legacy listeners may already have the attribute set)
-      if (!resource.targetFront) {
-        resource.targetFront = targetFront;
-      }
-
-      if (ResourceTransformers[resourceType]) {
-        resource = ResourceTransformers[resourceType]({
-          resource,
-          targetCommand: this.targetCommand,
-          targetFront,
-          watcherFront: this.watcherFront,
-        });
-      }
-
-      if (!resource.resourceId) {
-        resource.resourceId = `auto:${++gLastResourceId}`;
-      }
-
-      // Only consider top level document, and ignore remote iframes top document
-      const isWillNavigate =
-        resourceType == ResourceCommand.TYPES.DOCUMENT_EVENT &&
-        resource.name == "will-navigate";
-      if (isWillNavigate && resource.targetFront.isTopLevel) {
-        includesDocumentEventWillNavigate = true;
-        this._onWillNavigate(resource.targetFront);
-      }
-
-      if (
-        resourceType == ResourceCommand.TYPES.DOCUMENT_EVENT &&
-        resource.name == "dom-loading" &&
-        resource.targetFront.isTopLevel
-      ) {
-        includesDocumentEventDomLoading = true;
-      }
-
-      this._queueResourceEvent("available", resourceType, resource);
-
-      // Avoid storing will-navigate resource and consider it as a transcient resource.
-      // We do that to prevent leaking this resource (and its target) on navigation.
-      // We do clear the cache in _onWillNavigate, that we call a few lines before this.
-      if (!isWillNavigate) {
-        this.addResourceToCache(resource);
-      }
+      this._queueResourceEvent("available", resourceType, resources);
     }
 
     // If we receive the DOCUMENT_EVENT for:
@@ -710,7 +738,8 @@ class ResourceCommand {
     if (
       includesDocumentEventWillNavigate ||
       (includesDocumentEventDomLoading &&
-        !this.targetCommand.hasTargetWatcherSupport("service_worker"))
+        !this.targetCommand.hasTargetWatcherSupport("service_worker")) ||
+      this.throttlingDisabled
     ) {
       this._notifyWatchers();
     } else {
@@ -718,14 +747,40 @@ class ResourceCommand {
     }
   }
 
+  async _onResourceUpdatedArray(context, array) {
+    for (const [resourceType, resources] of array) {
+      for (const resource of resources) {
+        if (!("resourceType" in resource)) {
+          resource.resourceType = resourceType;
+        }
+      }
+      await this._onResourceUpdated(context, resources);
+    }
+  }
+
+  async _onResourceDestroyedArray(context, array) {
+    const resources = [];
+    for (const [resourceType, resourceIds] of array) {
+      for (const resourceId of resourceIds) {
+        resources.push({ resourceType, resourceId });
+      }
+    }
+    await this._onResourceDestroyed(context, resources);
+  }
+
   /**
+   * Called every time a resource is updated in the remote target.
+   *
    * Method called either by:
    * - the backward compatibility code (LegacyListeners)
    * - target actors RDP events
-   * Called everytime a resource is updated in the remote target.
    *
    * @param {Object} source
-   *        Please see _onResourceAvailable for this parameter.
+   *        A dictionary object with only one of these two attributes:
+   *        - targetFront: a Target Front, if the resource is watched from the
+   *          target process or thread.
+   *        - watcherFront: a Watcher Front, if the resource is watched from
+   *          the parent process.
    * @param {Array<Object>} updates
    *        Depending on the listener.
    *
@@ -769,12 +824,22 @@ class ResourceCommand {
         console.warn(`Expected resource ${resourceType} to have a resourceId`);
       }
 
-      // See _onResourceAvailable()
+      // See _onResourceAvailableArray()
       // We also need to wait for the related targetFront to be initialized
-      // otherwise we would notify about the udpate *before* the available
+      // otherwise we would notify about the update *before* it's available
       // and the resource won't be in _cache.
       if (watcherFront) {
-        targetFront = await this._getTargetForWatcherResource(update);
+        try {
+          targetFront = await this._getTargetForWatcherResource(update);
+        } catch (e) {
+          if (this.#destroyed) {
+            // If the resource-command was destroyed while waiting for
+            // _getTargetForWatcherResource, swallow the error.
+            return;
+          }
+          throw e;
+        }
+
         // When we receive the navigation request, the target front has already been
         // destroyed, but this is fine. The cached resource has the reference to
         // the (destroyed) target front and it is fully initialized.
@@ -805,46 +870,53 @@ class ResourceCommand {
           target[path[path.length - 1]] = value;
         }
       }
-      this._queueResourceEvent("updated", resourceType, {
-        resource: existingResource,
-        update,
-      });
+      this._queueResourceEvent("updated", resourceType, [
+        {
+          resource: existingResource,
+          update,
+        },
+      ]);
     }
+
     this._throttledNotifyWatchers();
   }
 
   /**
-   * Called everytime a resource is destroyed in the remote target.
-   * See _onResourceAvailable for the argument description.
+   * Called every time a resource is destroyed in the remote target.
+   *
+   * @param {Object} source
+   *        A dictionary object with only one of these two attributes:
+   *        - targetFront: a Target Front, if the resource is watched from the
+   *          target process or thread.
+   *        - watcherFront: a Watcher Front, if the resource is watched from
+   *          the parent process.
+   * @param {Array<json/Front>} resources
+   *        Depending on the resource Type, it can be an Array composed of
+   *        either JSON objects or Fronts, which describes the resource.
    */
-  async _onResourceDestroyed({ targetFront, watcherFront }, resources) {
+  async _onResourceDestroyed({ targetFront }, resources) {
     for (const resource of resources) {
       const { resourceType, resourceId } = resource;
       this._cache.delete(cacheKey(resourceType, resourceId));
-      this._queueResourceEvent("destroyed", resourceType, resource);
+      if (!resource.targetFront) {
+        resource.targetFront = targetFront;
+      }
+      this._queueResourceEvent("destroyed", resourceType, [resource]);
     }
     this._throttledNotifyWatchers();
   }
 
-  _queueResourceEvent(callbackType, resourceType, update) {
+  _queueResourceEvent(callbackType, resourceType, updates) {
     for (const { resources, pendingEvents } of this._watchers) {
       // This watcher doesn't listen to this type of resource
       if (!resources.includes(resourceType)) {
         continue;
       }
-      // If we receive a new event of the same type, accumulate the new update in the last event
-      if (pendingEvents.length) {
-        const lastEvent = pendingEvents[pendingEvents.length - 1];
-        if (lastEvent.callbackType == callbackType) {
-          lastEvent.updates.push(update);
-          continue;
-        }
-      }
-      // Otherwise, pile up a new event, which will force calling watcher
-      // callback a new time
+      // Avoid trying to coalesce with last pending event as mutating `updates` may have side effects
+      // with other watchers as this array is shared between all the watchers.
       pendingEvents.push({
         callbackType,
-        updates: [update],
+        updates,
       });
     }
   }
@@ -856,12 +928,8 @@ class ResourceCommand {
    */
   _notifyWatchers() {
     for (const watcherEntry of this._watchers) {
-      const {
-        onAvailable,
-        onUpdated,
-        onDestroyed,
-        pendingEvents,
-      } = watcherEntry;
+      const { onAvailable, onUpdated, onDestroyed, pendingEvents } =
+        watcherEntry;
       // Immediately clear the buffer in order to avoid possible races, where an event listener
       // would end up somehow adding a new throttled resource
       watcherEntry.pendingEvents = [];
@@ -899,7 +967,7 @@ class ResourceCommand {
     // Server watchers should pass an explicit "-1" value in order to prevent
     // silently ignoring an undefined browsingContextID attribute.
     if (browsingContextID == -1) {
-      return null;
+      return this.targetCommand.targetFront;
     }
 
     if (innerWindowId && this.targetCommand.isServerTargetSwitchingEnabled()) {
@@ -915,7 +983,7 @@ class ResourceCommand {
     return null;
   }
 
-  _onWillNavigate(targetFront) {
+  _onWillNavigate() {
     // Special case for toolboxes debugging a document,
     // purge the cache entirely when we start navigating to a new document.
     // Other toolboxes and additional target for remote iframes or content process
@@ -1066,9 +1134,15 @@ class ResourceCommand {
       return;
     }
 
-    const onAvailable = this._onResourceAvailable.bind(this, { targetFront });
-    const onUpdated = this._onResourceUpdated.bind(this, { targetFront });
-    const onDestroyed = this._onResourceDestroyed.bind(this, { targetFront });
+    const onAvailableArray = this._onResourceAvailableArray.bind(this, {
+      targetFront,
+    });
+    const onUpdatedArray = this._onResourceUpdatedArray.bind(this, {
+      targetFront,
+    });
+    const onDestroyedArray = this._onResourceDestroyedArray.bind(this, {
+      targetFront,
+    });
 
     if (!(resourceType in LegacyListeners)) {
       throw new Error(`Missing legacy listener for ${resourceType}`);
@@ -1093,9 +1167,9 @@ class ResourceCommand {
       await LegacyListeners[resourceType]({
         targetCommand: this.targetCommand,
         targetFront,
-        onAvailable,
-        onDestroyed,
-        onUpdated,
+        onAvailableArray,
+        onDestroyedArray,
+        onUpdatedArray,
       });
     } catch (e) {
       // Swallow the error to avoid breaking calls to watchResources which will
@@ -1141,9 +1215,8 @@ class ResourceCommand {
         this.watcherFront.unwatchResources([resourceType]);
       }
 
-      const shouldRunLegacyListeners = this._shouldRunLegacyListenerEvenWithWatcherSupport(
-        resourceType
-      );
+      const shouldRunLegacyListeners =
+        this._shouldRunLegacyListenerEvenWithWatcherSupport(resourceType);
       if (!shouldRunLegacyListeners) {
         return;
       }
@@ -1190,13 +1263,15 @@ class ResourceCommand {
   }
 }
 
+const DOCUMENT_EVENT = "document-event";
 ResourceCommand.TYPES = ResourceCommand.prototype.TYPES = {
   CONSOLE_MESSAGE: "console-message",
   CSS_CHANGE: "css-change",
   CSS_MESSAGE: "css-message",
+  CSS_REGISTERED_PROPERTIES: "css-registered-properties",
   ERROR_MESSAGE: "error-message",
   PLATFORM_MESSAGE: "platform-message",
-  DOCUMENT_EVENT: "document-event",
+  DOCUMENT_EVENT,
   ROOT_NODE: "root-node",
   STYLESHEET: "stylesheet",
   NETWORK_EVENT: "network-event",
@@ -1211,6 +1286,8 @@ ResourceCommand.TYPES = ResourceCommand.prototype.TYPES = {
   REFLOW: "reflow",
   SOURCE: "source",
   THREAD_STATE: "thread-state",
+  JSTRACER_TRACE: "jstracer-trace",
+  JSTRACER_STATE: "jstracer-state",
   SERVER_SENT_EVENT: "server-sent-event",
   LAST_PRIVATE_CONTEXT_EXIT: "last-private-context-exit",
 };
@@ -1229,15 +1306,21 @@ const WORKER_RESOURCE_TYPES = [
   ResourceCommand.TYPES.THREAD_STATE,
 ];
 
+// List of resource types which aren't stored in the internal ResourceCommand cache.
+// Only the first `watchResources()` call for a given resource type may receive already existing
+// resources. All subsequent call to `watchResources()` for the same resource type will
+// only receive future resource, and not the one already notified in the past.
+// This is typically used for resources with very high throughput.
+const TRANSIENT_RESOURCE_TYPES = [
+  ResourceCommand.TYPES.JSTRACER_TRACE,
+  ResourceCommand.TYPES.JSTRACER_STATE,
+];
+
 // Backward compat code for each type of resource.
 // Each section added here should eventually be removed once the equivalent server
 // code is implement in Firefox, in its release channel.
 const LegacyListeners = {
-  async [ResourceCommand.TYPES.DOCUMENT_EVENT]({
-    targetCommand,
-    targetFront,
-    onAvailable,
-  }) {
+  async [ResourceCommand.TYPES.DOCUMENT_EVENT]({ targetFront, onAvailable }) {
     // DocumentEventsListener of webconsole handles only top level document.
     if (!targetFront.isTopLevel) {
       return;
@@ -1282,48 +1365,6 @@ loader.lazyRequireGetter(
   "resource://devtools/shared/commands/resource/legacy-listeners/root-node.js"
 );
 
-// @backward-compat { version 111 } The WebSocket legacy listener can be removed
-// once 111 is released.
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.WEBSOCKET,
-  "resource://devtools/shared/commands/resource/legacy-listeners/websocket.js"
-);
-
-// @backward-compat { version 111 } All storage legacy listeners can be removed once 111 is released.
-// This codepath was only used when connecting to older servers.
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.COOKIE,
-  "resource://devtools/shared/commands/resource/legacy-listeners/cookie.js"
-);
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.CACHE_STORAGE,
-  "resource://devtools/shared/commands/resource/legacy-listeners/cache-storage.js"
-);
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.LOCAL_STORAGE,
-  "resource://devtools/shared/commands/resource/legacy-listeners/local-storage.js"
-);
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.SESSION_STORAGE,
-  "resource://devtools/shared/commands/resource/legacy-listeners/session-storage.js"
-);
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.EXTENSION_STORAGE,
-  "resource://devtools/shared/commands/resource/legacy-listeners/extension-storage.js"
-);
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.INDEXED_DB,
-  "resource://devtools/shared/commands/resource/legacy-listeners/indexed-db.js"
-);
-// --- end of storage legacy listeners
-
 loader.lazyRequireGetter(
   LegacyListeners,
   ResourceCommand.TYPES.SOURCE,
@@ -1334,13 +1375,7 @@ loader.lazyRequireGetter(
   ResourceCommand.TYPES.THREAD_STATE,
   "resource://devtools/shared/commands/resource/legacy-listeners/thread-states.js"
 );
-// @backward-compat { version 111 } The serverSentEvent legacy listener can be
-// removed once 111 is released.
-loader.lazyRequireGetter(
-  LegacyListeners,
-  ResourceCommand.TYPES.SERVER_SENT_EVENT,
-  "resource://devtools/shared/commands/resource/legacy-listeners/server-sent-events.js"
-);
+
 loader.lazyRequireGetter(
   LegacyListeners,
   ResourceCommand.TYPES.REFLOW,
