@@ -8,7 +8,9 @@
 #include "nss.h"
 #include "ssl.h"
 
+#include "api/audio/builtin_audio_processing_builder.h"
 #include "api/audio_codecs/builtin_audio_decoder_factory.h"
+#include "api/environment/environment_factory.h"
 #include "api/scoped_refptr.h"
 #include "AudioSegment.h"
 #include "Canonicals.h"
@@ -23,11 +25,12 @@
 #include "MediaPipelineFilter.h"
 #include "MediaTrackGraph.h"
 #include "MediaTrackListener.h"
-#include "TaskQueueWrapper.h"
 #include "mtransport_test_utils.h"
 #include "SharedBuffer.h"
 #include "MediaTransportHandler.h"
 #include "WebrtcCallWrapper.h"
+#include "WebrtcEnvironmentWrapper.h"
+#include "WebrtcTaskQueueWrapper.h"
 #include "PeerConnectionCtx.h"
 
 #define GTEST_HAS_RTTI 0
@@ -39,20 +42,52 @@ MOZ_MTLOG_MODULE("transportbridge")
 static MtransportTestUtils* test_utils;
 
 namespace {
-class MainAsCurrent : public TaskQueueWrapper<DeletionPolicy::NonBlocking> {
+class MainAsCurrent : public webrtc::TaskQueueBase {
  public:
   MainAsCurrent()
-      : TaskQueueWrapper(
-            TaskQueue::Create(do_AddRef(GetMainThreadSerialEventTarget()),
-                              "MainAsCurrentTaskQueue"),
-            "MainAsCurrent"_ns),
-        mSetter(this) {
+      : mTaskQueue(CreateWebrtcTaskQueueWrapper(
+            do_AddRef(GetMainThreadSerialEventTarget()), "MainAsCurrent"_ns,
+            false)),
+        mWebrtcTaskQueue(([&] {
+          // Shady but fine, as this raw pointer points to the WebrtcTaskQueue
+          // owned and kept alive by mTaskQueue.
+          webrtc::TaskQueueBase* queue{};
+          MOZ_ALWAYS_SUCCEEDS(mTaskQueue->Dispatch(NS_NewRunnableFunction(
+              "MainAsCurrent::Current",
+              [&] { queue = webrtc::TaskQueueBase::Current(); })));
+          NS_ProcessPendingEvents(nullptr);
+          MOZ_RELEASE_ASSERT(queue);
+          return queue;
+        })()),
+        mSetter(mWebrtcTaskQueue) {
     MOZ_RELEASE_ASSERT(NS_IsMainThread());
   }
 
   ~MainAsCurrent() = default;
 
+  void Delete() override { delete this; }
+
+  void PostTaskImpl(absl::AnyInvocable<void() &&> task,
+                    const PostTaskTraits& traits,
+                    const webrtc::Location& location) override {
+    mWebrtcTaskQueue->PostTask(std::move(task), location);
+  }
+
+  void PostDelayedTaskImpl(absl::AnyInvocable<void() &&> task,
+                           webrtc::TimeDelta delay,
+                           const PostDelayedTaskTraits& traits,
+                           const webrtc::Location& location) override {
+    if (traits.high_precision) {
+      mWebrtcTaskQueue->PostDelayedHighPrecisionTask(std::move(task), delay,
+                                                     location);
+      return;
+    }
+    mWebrtcTaskQueue->PostDelayedTask(std::move(task), delay, location);
+  }
+
  private:
+  RefPtr<TaskQueue> mTaskQueue;
+  webrtc::TaskQueueBase* mWebrtcTaskQueue;
   CurrentTaskQueueSetter mSetter;
 };
 
@@ -69,7 +104,7 @@ class FakeAudioTrack : public ProcessedMediaTrack {
 
   void Destroy() override {
     MutexAutoLock lock(mMutex);
-    MOZ_ASSERT(!mMainThreadDestroyed);
+    MOZ_RELEASE_ASSERT(!mMainThreadDestroyed);
     mMainThreadDestroyed = true;
     mTimer->Cancel();
     mTimer = nullptr;
@@ -79,14 +114,14 @@ class FakeAudioTrack : public ProcessedMediaTrack {
 
   void AddListener(MediaTrackListener* aListener) override {
     MutexAutoLock lock(mMutex);
-    MOZ_ASSERT(!mListener);
+    MOZ_RELEASE_ASSERT(!mListener);
     mListener = aListener;
   }
 
   RefPtr<GenericPromise> RemoveListener(
       MediaTrackListener* aListener) override {
     MutexAutoLock lock(mMutex);
-    MOZ_ASSERT(mListener == aListener);
+    MOZ_RELEASE_ASSERT(mListener == aListener);
     mListener = nullptr;
     return GenericPromise::CreateAndResolve(true, __func__);
   }
@@ -140,7 +175,7 @@ void RunOnSts(Function&& aFunction) {
 
 class LoopbackTransport : public MediaTransportHandler {
  public:
-  LoopbackTransport() : MediaTransportHandler(nullptr) {
+  LoopbackTransport() : MediaTransportHandler() {
     RunOnSts([&] {
       SetState("mux", TransportLayer::TS_INIT);
       SetRtcpState("mux", TransportLayer::TS_INIT);
@@ -224,7 +259,7 @@ class LoopbackTransport : public MediaTransportHandler {
 
   void SendPacket(const std::string& aTransportId,
                   MediaPacket&& aPacket) override {
-    peer_->SignalPacketReceived(aTransportId, aPacket);
+    peer_->LoopbackPacketReceived(aTransportId, aPacket);
   }
 
   void SetState(const std::string& aTransportId, TransportLayer::State aState) {
@@ -236,21 +271,39 @@ class LoopbackTransport : public MediaTransportHandler {
     MediaTransportHandler::OnRtcpStateChange(aTransportId, aState);
   }
 
+  void LoopbackPacketReceived(const std::string& aTransportId,
+                              const MediaPacket& aPacket) {
+    if (aPacket.len() && aPacket.type() == MediaPacket::RTCP) {
+      ++rtcp_packets_received_;
+    }
+    mRtpPacketReceived.Notify(aTransportId, aPacket);
+  }
+
+  int RtcpPacketsReceived() const { return rtcp_packets_received_; }
+
  private:
-  RefPtr<MediaTransportHandler> peer_;
+  RefPtr<LoopbackTransport> peer_;
+  std::atomic<int> rtcp_packets_received_{0};
+};
+
+struct MediaPipelineTestOptions {
+  bool is_rtcp_mux = true;
+  bool activate_receive = true;
+  unsigned int ms_until_filter_update = 500;
+  unsigned int ms_of_traffic_after_answer = 10000;
 };
 
 class TestAgent {
  public:
-  explicit TestAgent(const RefPtr<SharedWebrtcState>& aSharedState)
+  explicit TestAgent(const RefPtr<WebrtcEnvironmentWrapper>& aEnvWrapper,
+                     const RefPtr<SharedWebrtcState>& aSharedState)
       : control_(aSharedState->mCallWorkerThread),
         audio_config_(109, "opus", 48000, 2, false),
         call_(WebrtcCallWrapper::Create(
-            mozilla::dom::RTCStatsTimestampMaker::Create(), nullptr,
-            aSharedState)),
+            aEnvWrapper, mozilla::dom::RTCStatsTimestampMaker::Create(),
+            nullptr, aSharedState)),
         audio_conduit_(
             AudioSessionConduit::Create(call_, test_utils->sts_target())),
-        audio_pipeline_(),
         transport_(new LoopbackTransport) {
     Unused << WaitFor(InvokeAsync(call_->mCallThread, __func__, [&] {
       audio_conduit_->InitControl(&control_);
@@ -262,7 +315,8 @@ class TestAgent {
     LoopbackTransport::InitAndConnect(*client->transport_, *server->transport_);
   }
 
-  virtual void CreatePipeline(const std::string& aTransportId) = 0;
+  virtual void CreatePipeline(const std::string& aTransportId,
+                              const MediaPipelineTestOptions& aOptions) = 0;
 
   void SetState_s(const std::string& aTransportId,
                   TransportLayer::State aState) {
@@ -276,7 +330,7 @@ class TestAgent {
 
   void UpdateTransport_s(const std::string& aTransportId,
                          UniquePtr<MediaPipelineFilter>&& aFilter) {
-    audio_pipeline_->UpdateTransport_s(aTransportId, std::move(aFilter));
+    audio_pipeline_->UpdateTransport_s(aTransportId, std::move(aFilter), false);
   }
 
   void Stop() {
@@ -326,9 +380,7 @@ class TestAgent {
 
   int GetAudioRtcpCountSent() { return audio_pipeline_->RtcpPacketsSent(); }
 
-  int GetAudioRtcpCountReceived() {
-    return audio_pipeline_->RtcpPacketsReceived();
-  }
+  int GetAudioRtcpCountReceived() { return transport_->RtcpPacketsReceived(); }
 
  protected:
   ConcreteControl control_;
@@ -345,20 +397,23 @@ class TestAgent {
 
 class TestAgentSend : public TestAgent {
  public:
-  explicit TestAgentSend(const RefPtr<SharedWebrtcState>& aSharedState)
-      : TestAgent(aSharedState) {
+  explicit TestAgentSend(const RefPtr<WebrtcEnvironmentWrapper>& aEnvWrapper,
+                         const RefPtr<SharedWebrtcState>& aSharedState)
+      : TestAgent(aEnvWrapper, aSharedState) {
     control_.Update([&](auto& aControl) {
       aControl.mAudioSendCodec = Some(audio_config_);
     });
     audio_track_ = new FakeAudioTrack();
   }
 
-  virtual void CreatePipeline(const std::string& aTransportId) {
+  virtual void CreatePipeline(const std::string& aTransportId,
+                              const MediaPipelineTestOptions& aOptions) {
     std::string test_pc;
 
-    auto audio_pipeline = MakeRefPtr<MediaPipelineTransmit>(
-        test_pc, transport_, AbstractThread::MainThread(),
-        test_utils->sts_target(), false, audio_conduit_);
+    RefPtr<MediaPipelineTransmit> audio_pipeline =
+        MediaPipelineTransmit::Create(
+            test_pc, transport_, AbstractThread::MainThread(),
+            test_utils->sts_target(), false, audio_conduit_);
     Unused << WaitFor(InvokeAsync(call_->mCallThread, __func__, [&] {
       audio_pipeline->InitControl(&control_);
       return GenericPromise::CreateAndResolve(true, __func__);
@@ -366,15 +421,16 @@ class TestAgentSend : public TestAgent {
 
     audio_pipeline->SetSendTrackOverride(audio_track_);
     control_.Update([](auto& aControl) { aControl.mTransmitting = true; });
-    audio_pipeline->UpdateTransport_m(aTransportId, nullptr);
+    audio_pipeline->UpdateTransport_m(aTransportId, nullptr, true);
     audio_pipeline_ = audio_pipeline;
   }
 };
 
 class TestAgentReceive : public TestAgent {
  public:
-  explicit TestAgentReceive(const RefPtr<SharedWebrtcState>& aSharedState)
-      : TestAgent(aSharedState) {
+  explicit TestAgentReceive(const RefPtr<WebrtcEnvironmentWrapper>& aEnvWrapper,
+                            const RefPtr<SharedWebrtcState>& aSharedState)
+      : TestAgent(aEnvWrapper, aSharedState) {
     control_.Update([&](auto& aControl) {
       std::vector<AudioCodecConfig> codecs;
       codecs.push_back(audio_config_);
@@ -382,7 +438,8 @@ class TestAgentReceive : public TestAgent {
     });
   }
 
-  virtual void CreatePipeline(const std::string& aTransportId) {
+  virtual void CreatePipeline(const std::string& aTransportId,
+                              const MediaPipelineTestOptions& aOptions) {
     std::string test_pc;
 
     auto audio_pipeline = MakeRefPtr<MediaPipelineReceiveAudio>(
@@ -395,8 +452,11 @@ class TestAgentReceive : public TestAgent {
       return GenericPromise::CreateAndResolve(true, __func__);
     }));
 
-    control_.Update([](auto& aControl) { aControl.mReceiving = true; });
-    audio_pipeline->UpdateTransport_m(aTransportId, std::move(bundle_filter_));
+    control_.Update([activate = aOptions.activate_receive](auto& aControl) {
+      aControl.mReceiving = activate;
+    });
+    audio_pipeline->UpdateTransport_m(aTransportId, std::move(bundle_filter_),
+                                      true);
     audio_pipeline_ = audio_pipeline;
   }
 
@@ -406,7 +466,7 @@ class TestAgentReceive : public TestAgent {
 
   void UpdateTransport_s(const std::string& aTransportId,
                          UniquePtr<MediaPipelineFilter>&& filter) {
-    audio_pipeline_->UpdateTransport_s(aTransportId, std::move(filter));
+    audio_pipeline_->UpdateTransport_s(aTransportId, std::move(filter), false);
   }
 
  private:
@@ -422,28 +482,34 @@ void WaitFor(TimeDuration aDuration) {
       "WaitFor(TimeDuration aDuration)"_ns, [&] { return done; });
 }
 
-webrtc::AudioState::Config CreateAudioStateConfig() {
+webrtc::AudioState::Config CreateAudioStateConfig(
+    const webrtc::Environment& aEnv) {
   webrtc::AudioState::Config audio_state_config;
   audio_state_config.audio_mixer = webrtc::AudioMixerImpl::Create();
-  webrtc::AudioProcessingBuilder audio_processing_builder;
-  audio_state_config.audio_processing = audio_processing_builder.Create();
+
+  webrtc::BuiltinAudioProcessingBuilder audio_processing_builder;
+  audio_state_config.audio_processing = audio_processing_builder.Build(aEnv);
   audio_state_config.audio_device_module = new webrtc::FakeAudioDeviceModule();
   return audio_state_config;
 }
 
 class MediaPipelineTest : public ::testing::Test {
  public:
-  MediaPipelineTest()
+  explicit MediaPipelineTest(MediaPipelineTestOptions options = {})
       : main_task_queue_(
-            WrapUnique<TaskQueueWrapper<DeletionPolicy::NonBlocking>>(
+            std::unique_ptr<webrtc::TaskQueueBase, webrtc::TaskQueueDeleter>(
                 new MainAsCurrent())),
+        options_(options),
+        env_wrapper_(WebrtcEnvironmentWrapper::Create(
+            mozilla::dom::RTCStatsTimestampMaker::Create())),
         shared_state_(MakeAndAddRef<SharedWebrtcState>(
-            AbstractThread::MainThread(), CreateAudioStateConfig(),
+            AbstractThread::MainThread(),
+            CreateAudioStateConfig(env_wrapper_->Environment()),
             already_AddRefed(
                 webrtc::CreateBuiltinAudioDecoderFactory().release()),
-            WrapUnique(new webrtc::NoTrialsConfig()))),
-        p1_(shared_state_),
-        p2_(shared_state_) {}
+            WrapUnique(new webrtc::MozTrialsConfig()))),
+        p1_(env_wrapper_, shared_state_),
+        p2_(env_wrapper_, shared_state_) {}
 
   ~MediaPipelineTest() {
     p1_.Shutdown();
@@ -463,24 +529,22 @@ class MediaPipelineTest : public ::testing::Test {
   }
 
   // Verify RTP and RTCP
-  void TestAudioSend(bool aIsRtcpMux,
+  void TestAudioSend(MediaPipelineTestOptions options,
                      UniquePtr<MediaPipelineFilter>&& initialFilter = nullptr,
-                     UniquePtr<MediaPipelineFilter>&& refinedFilter = nullptr,
-                     unsigned int ms_until_filter_update = 500,
-                     unsigned int ms_of_traffic_after_answer = 10000) {
+                     UniquePtr<MediaPipelineFilter>&& refinedFilter = nullptr) {
     bool bundle = !!(initialFilter);
     // We do not support testing bundle without rtcp mux, since that doesn't
     // make any sense.
-    ASSERT_FALSE(!aIsRtcpMux && bundle);
+    ASSERT_FALSE(!options.is_rtcp_mux && bundle);
 
     p2_.SetBundleFilter(std::move(initialFilter));
 
     // Setup transport flows
     InitTransports();
 
-    std::string transportId = aIsRtcpMux ? "mux" : "non-mux";
-    p1_.CreatePipeline(transportId);
-    p2_.CreatePipeline(transportId);
+    std::string transportId = options.is_rtcp_mux ? "mux" : "non-mux";
+    p1_.CreatePipeline(transportId, options);
+    p2_.CreatePipeline(transportId, options);
 
     // Set state of transports to CONNECTING. MediaPipeline doesn't really care
     // about this transition, but we're trying to simluate what happens in a
@@ -504,7 +568,7 @@ class MediaPipelineTest : public ::testing::Test {
     });
 
     if (bundle) {
-      WaitFor(TimeDuration::FromMilliseconds(ms_until_filter_update));
+      WaitFor(TimeDuration::FromMilliseconds(options.ms_until_filter_update));
 
       // Leaving refinedFilter not set implies we want to just update with
       // the other side's SSRC
@@ -520,7 +584,7 @@ class MediaPipelineTest : public ::testing::Test {
     }
 
     // wait for some RTP/RTCP tx and rx to happen
-    WaitFor(TimeDuration::FromMilliseconds(ms_of_traffic_after_answer));
+    WaitFor(TimeDuration::FromMilliseconds(options.ms_of_traffic_after_answer));
 
     p1_.Stop();
     p2_.Stop();
@@ -546,18 +610,19 @@ class MediaPipelineTest : public ::testing::Test {
   }
 
   void TestAudioReceiverBundle(
-      bool bundle_accepted, UniquePtr<MediaPipelineFilter>&& initialFilter,
+      UniquePtr<MediaPipelineFilter>&& initialFilter,
       UniquePtr<MediaPipelineFilter>&& refinedFilter = nullptr,
-      unsigned int ms_until_answer = 500,
-      unsigned int ms_of_traffic_after_answer = 10000) {
-    TestAudioSend(true, std::move(initialFilter), std::move(refinedFilter),
-                  ms_until_answer, ms_of_traffic_after_answer);
+      MediaPipelineTestOptions options = {}) {
+    TestAudioSend(options, std::move(initialFilter), std::move(refinedFilter));
   }
 
  protected:
   // main_task_queue_ has this type to make sure it goes through Delete() when
   // we're destroyed.
-  UniquePtr<TaskQueueWrapper<DeletionPolicy::NonBlocking>> main_task_queue_;
+  std::unique_ptr<webrtc::TaskQueueBase, webrtc::TaskQueueDeleter>
+      main_task_queue_;
+  const MediaPipelineTestOptions options_;
+  const RefPtr<WebrtcEnvironmentWrapper> env_wrapper_;
   const RefPtr<SharedWebrtcState> shared_state_;
   TestAgentSend p1_;
   TestAgentReceive p2_;
@@ -589,6 +654,15 @@ TEST_F(MediaPipelineFilterTest, TestSSRCFilter) {
   EXPECT_FALSE(Filter(filter, 556, 110));
 }
 
+TEST_F(MediaPipelineFilterTest, TestSSRCFilterOverridesPayloadTypeFilter) {
+  MediaPipelineFilter filter;
+  filter.AddRemoteSSRC(555);
+  filter.AddUniqueReceivePT(110);
+  // We have a configured ssrc but still need to allow payload type matching.
+  EXPECT_TRUE(Filter(filter, 556, 110));
+  EXPECT_TRUE(Filter(filter, 555, 110));
+}
+
 #define SSRC(ssrc)                                                    \
   ((ssrc >> 24) & 0xFF), ((ssrc >> 16) & 0xFF), ((ssrc >> 8) & 0xFF), \
       (ssrc & 0xFF)
@@ -618,9 +692,14 @@ TEST_F(MediaPipelineFilterTest, TestMidFilter) {
 
 TEST_F(MediaPipelineFilterTest, TestPayloadTypeFilter) {
   MediaPipelineFilter filter;
-  filter.AddUniquePT(110);
+  filter.AddUniqueReceivePT(110);
   EXPECT_TRUE(Filter(filter, 555, 110));
   EXPECT_FALSE(Filter(filter, 556, 111));
+  // Matching based on unique payload type causes us to learn the ssrc.
+  EXPECT_TRUE(Filter(filter, 555, 98));
+  // Once we have learned an SSRC we still need to learn new ones
+  // based on payload type.
+  EXPECT_TRUE(Filter(filter, 557, 110));
 }
 
 TEST_F(MediaPipelineFilterTest, TestSSRCMovedWithMid) {
@@ -642,14 +721,14 @@ TEST_F(MediaPipelineFilterTest, TestRemoteSDPNoSSRCs) {
   MediaPipelineFilter filter;
   const auto mid = Some(std::string("mid0"));
   filter.SetRemoteMediaStreamId(mid);
-  filter.AddUniquePT(111);
+  filter.AddUniqueReceivePT(111);
   EXPECT_TRUE(Filter(filter, 555, 110, mid));
   EXPECT_TRUE(Filter(filter, 555, 110));
 
   // Update but remember binding./
   MediaPipelineFilter filter2;
 
-  filter.Update(filter2);
+  filter.Update(filter2, true);
 
   // Ensure that the old SSRC still works.
   EXPECT_TRUE(Filter(filter, 555, 110));
@@ -657,14 +736,18 @@ TEST_F(MediaPipelineFilterTest, TestRemoteSDPNoSSRCs) {
   // Forget the previous binding
   MediaPipelineFilter filter3;
   filter3.SetRemoteMediaStreamId(Some(std::string("mid1")));
-  filter.Update(filter3);
+  filter.Update(filter3, true);
 
   ASSERT_FALSE(Filter(filter, 555, 110));
 }
 
-TEST_F(MediaPipelineTest, TestAudioSendNoMux) { TestAudioSend(false); }
+TEST_F(MediaPipelineTest, TestAudioSendNoMux) {
+  TestAudioSend({.is_rtcp_mux = false});
+}
 
-TEST_F(MediaPipelineTest, TestAudioSendMux) { TestAudioSend(true); }
+TEST_F(MediaPipelineTest, TestAudioSendMux) {
+  TestAudioSend({.is_rtcp_mux = true});
+}
 
 TEST_F(MediaPipelineTest, TestAudioSendBundle) {
   auto filter = MakeUnique<MediaPipelineFilter>();
@@ -673,10 +756,11 @@ TEST_F(MediaPipelineTest, TestAudioSendBundle) {
   // is sometimes sent before the transports are ready, which causes it to
   // be dropped.
   TestAudioReceiverBundle(
-      true, std::move(filter),
+      std::move(filter),
       // We do not specify the filter for the remote description, so it will be
       // set to something sane after a short time.
-      nullptr, 10000, 10000);
+      nullptr,
+      {.ms_until_filter_update = 10000, .ms_of_traffic_after_answer = 10000});
 
   // Some packets should have been dropped, but not all
   ASSERT_GT(p1_.GetAudioRtpCountSent(), p2_.GetAudioRtpCountReceived());
@@ -687,10 +771,21 @@ TEST_F(MediaPipelineTest, TestAudioSendBundle) {
 TEST_F(MediaPipelineTest, TestAudioSendEmptyBundleFilter) {
   auto filter = MakeUnique<MediaPipelineFilter>();
   auto bad_answer_filter = MakeUnique<MediaPipelineFilter>();
-  TestAudioReceiverBundle(true, std::move(filter),
-                          std::move(bad_answer_filter));
+  TestAudioReceiverBundle(std::move(filter), std::move(bad_answer_filter));
   // Filter is empty, so should drop everything.
   ASSERT_EQ(0, p2_.GetAudioRtpCountReceived());
 }
 
+TEST_F(MediaPipelineTest, TestAudioInactiveNoRecv) {
+  auto filter = MakeUnique<MediaPipelineFilter>();
+  TestAudioReceiverBundle(std::move(filter), nullptr,
+                          {.activate_receive = false,
+                           .ms_until_filter_update = 200,
+                           .ms_of_traffic_after_answer = 800});
+
+  // Packets should have been sent but not received.
+  ASSERT_NE(p1_.GetAudioRtpCountSent(), 0);
+  ASSERT_EQ(p2_.GetAudioRtpCountReceived(), 0);
+}
+
 }  // end namespace
