@@ -24,7 +24,7 @@ from mozpack.manifests import InstallManifest
 from mozpack.mozjar import JarWriter
 from reftest import ReftestManifest
 
-from mozbuild.util import ensureParentDir
+from mozbuild.dirutils import ensureParentDir
 
 STAGE = mozpath.join(buildconfig.topobjdir, "dist", "test-stage")
 
@@ -41,16 +41,18 @@ TEST_HARNESS_BINS = [
     "crashinject",
     "geckodriver",
     "http3server",
+    "content_analysis_sdk_agent",
     "minidumpwriter",
     "pk12util",
     "screenshot",
     "screentopng",
     "ssltunnel",
+    "test_stub_installer",
     "xpcshell",
     "plugin-container",
 ]
 
-TEST_HARNESS_DLLS = ["crashinjectdll", "mozglue"]
+TEST_HARNESS_DLLS = ["crashinjectdll", "mozglue", "xul", "nss3", "gkcodecs", "lgpllibs"]
 
 GMP_TEST_PLUGIN_DIRS = ["gmp-fake/**", "gmp-fakeopenh264/**"]
 
@@ -108,7 +110,7 @@ ARCHIVE_FILES = {
             "source": buildconfig.topsrcdir,
             "base": "",
             "manifests": [
-                "testing/marionette/harness/marionette_harness/tests/unit-tests.ini"
+                "testing/marionette/harness/marionette_harness/tests/unit-tests.toml"
             ],
             # We also need the manifests and harness_unit tests
             "pattern": "testing/marionette/harness/marionette_harness/tests/**",
@@ -250,7 +252,7 @@ ARCHIVE_FILES = {
         {
             "source": buildconfig.topsrcdir,
             "base": "testing",
-            "pattern": "cppunittest.ini",
+            "pattern": "cppunittest.toml",
             "dest": "cppunittest",
         },
         {
@@ -294,6 +296,12 @@ ARCHIVE_FILES = {
             "pattern": "IA2Typelib.tlb",
             "dest": "mochitest",
         },
+        {
+            "source": buildconfig.topobjdir,
+            "base": "dist/bin",
+            "pattern": "IA2Marshal.dll",
+            "dest": "mochitest",
+        },
     ],
     "mozharness": [
         {
@@ -336,11 +344,21 @@ ARCHIVE_FILES = {
             "base": "testing/mozbase/mozprocess",
             "pattern": "mozprocess/**",
         },
+        {
+            "source": buildconfig.topsrcdir,
+            "base": "testing/mozbase/mozsystemmonitor",
+            "pattern": "mozsystemmonitor/**",
+        },
         {
             "source": buildconfig.topsrcdir,
             "base": "third_party/python/six",
             "pattern": "six.py",
         },
+        {
+            "source": buildconfig.topsrcdir,
+            "base": "third_party/python/toml",
+            "pattern": "**",
+        },
         {
             "source": buildconfig.topsrcdir,
             "base": "third_party/python/tomlkit",
@@ -454,11 +472,13 @@ ARCHIVE_FILES = {
                 "chrome/**",
                 "chrome.manifest",
                 "components/**",
+                "content_analysis_sdk_agent",
                 "http3server",
                 "*.ini",
+                "*.toml",
                 "localization/**",
                 "modules/**",
-                "update.locale",
+                "default.locale",
                 "greprefs.js",
             ],
             "dest": "bin",
@@ -675,7 +695,10 @@ if buildconfig.substs.get("MOZ_CODE_COVERAGE"):
     )
 
 
-if buildconfig.substs.get("MOZ_ASAN") and buildconfig.substs.get("CLANG_CL"):
+if (
+    buildconfig.substs.get("MOZ_ASAN")
+    and buildconfig.substs.get("CC_TYPE") == "clang-cl"
+):
     asan_dll = {
         "source": buildconfig.topobjdir,
         "base": "dist/bin",
@@ -696,7 +719,7 @@ if buildconfig.substs.get("commtopsrcdir"):
     marionette_comm = {
         "source": commtopsrcdir,
         "base": "",
-        "manifest": "testing/marionette/unit-tests.ini",
+        "manifest": "testing/marionette/unit-tests.toml",
         "dest": "marionette/tests/comm",
     }
     ARCHIVE_FILES["common"].append(marionette_comm)
@@ -783,7 +806,7 @@ def find_files(archive):
             manifests.append(manifest)
         if manifests:
             dirs = find_manifest_dirs(os.path.join(source, base), manifests)
-            patterns.extend({"{}/**".format(d) for d in dirs})
+            patterns.extend({f"{d}/**" for d in dirs})
 
         ignore = list(entry.get("ignore", []))
         ignore.extend(["**/.flake8", "**/.mkdir.done", "**/*.pyc"])
@@ -815,7 +838,7 @@ def find_manifest_dirs(topsrcdir, manifests):
     for p in manifests:
         p = os.path.join(topsrcdir, p)
 
-        if p.endswith(".ini"):
+        if p.endswith(".ini") or p.endswith(".toml"):
             test_manifest = TestManifest()
             test_manifest.read(p)
             dirs |= set([os.path.dirname(m) for m in test_manifest.manifests()])
@@ -827,9 +850,7 @@ def find_manifest_dirs(topsrcdir, manifests):
 
         else:
             raise Exception(
-                '"{}" is not a supported manifest format.'.format(
-                    os.path.splitext(p)[1]
-                )
+                f'"{os.path.splitext(p)[1]}" is not a supported manifest format.'
             )
 
     dirs = {mozpath.normpath(d[len(topsrcdir) :]).lstrip("/") for d in dirs}
