@@ -2,10 +2,10 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+from collections import defaultdict
 from operator import itemgetter
 
 import mozpack.path as mozpath
-import six
 from mozpack.manifests import InstallManifest
 
 from mozbuild.backend.base import PartialBackend
@@ -22,44 +22,39 @@ from mozbuild.frontend.data import (
     XPIDLModule,
 )
 from mozbuild.makeutil import Makefile
-from mozbuild.util import OrderedDefaultDict
 
 
 class FasterMakeBackend(MakeBackend, PartialBackend):
     def _init(self):
         super(FasterMakeBackend, self)._init()
 
-        self._manifest_entries = OrderedDefaultDict(set)
+        self._manifest_entries = defaultdict(set)
 
-        self._install_manifests = OrderedDefaultDict(InstallManifest)
+        self._install_manifests = defaultdict(InstallManifest)
 
-        self._dependencies = OrderedDefaultDict(list)
-        self._l10n_dependencies = OrderedDefaultDict(list)
+        self._dependencies = defaultdict(list)
+        self._l10n_dependencies = defaultdict(list)
 
         self._has_xpidl = False
 
         self._generated_files_map = {}
         self._generated_files = []
 
-    def _add_preprocess(self, obj, path, dest, target=None, **kwargs):
-        if target is None:
-            target = mozpath.basename(path)
-        # This matches what PP_TARGETS do in config/rules.
-        if target.endswith(".in"):
-            target = target[:-3]
-        if target.endswith(".css"):
+    def _add_preprocess(self, obj, src, dest, f, **kwargs):
+        basename = FinalTargetPreprocessedFiles.get_obj_basename(f)
+        if basename.endswith(".css"):
             kwargs["marker"] = "%"
         depfile = mozpath.join(
             self.environment.topobjdir,
             "faster",
             ".deps",
-            mozpath.join(obj.install_target, dest, target).replace("/", "_"),
+            mozpath.join(obj.install_target, dest, basename).replace("/", "_"),
         )
         self._install_manifests[obj.install_target].add_preprocess(
-            mozpath.join(obj.srcdir, path),
-            mozpath.join(dest, target),
+            mozpath.join(obj.srcdir, src),
+            mozpath.join(dest, basename),
             depfile,
-            **kwargs
+            **kwargs,
         )
 
     def consume_object(self, obj):
@@ -69,7 +64,7 @@ class FasterMakeBackend(MakeBackend, PartialBackend):
         elif isinstance(
             obj, (FinalTargetFiles, FinalTargetPreprocessedFiles)
         ) and obj.install_target.startswith("dist/bin"):
-            ab_cd = self.environment.substs["MOZ_UI_LOCALE"][0]
+            ab_cd = self.environment.substs["MOZ_UI_LOCALE"]
             localized = isinstance(obj, (LocalizedFiles, LocalizedPreprocessedFiles))
             defines = obj.defines or {}
             if defines:
@@ -99,9 +94,7 @@ class FasterMakeBackend(MakeBackend, PartialBackend):
                         src = f.full_path
 
                     if isinstance(obj, FinalTargetPreprocessedFiles):
-                        self._add_preprocess(
-                            obj, src, path, target=f.target_basename, defines=defines
-                        )
+                        self._add_preprocess(obj, src, path, f, defines=defines)
                     elif "*" in f:
 
                         def _prefix(s):
@@ -195,7 +188,7 @@ class FasterMakeBackend(MakeBackend, PartialBackend):
         # Add information for chrome manifest generation
         manifest_targets = []
 
-        for target, entries in six.iteritems(self._manifest_entries):
+        for target, entries in self._manifest_entries.items():
             manifest_targets.append(target)
             install_target = mozpath.basedir(target, install_manifests_bases)
             self._install_manifests[install_target].add_content(
@@ -209,19 +202,17 @@ class FasterMakeBackend(MakeBackend, PartialBackend):
         )
 
         # Add dependencies we inferred:
-        for target, deps in sorted(six.iteritems(self._dependencies)):
+        for target, deps in sorted(self._dependencies.items()):
             mk.create_rule([target]).add_dependencies(
                 "$(TOPOBJDIR)/%s" % d for d in sorted(deps)
             )
 
-        # This is not great, but it's better to have some dependencies on these Python files.
+        # This is not great, but it's better to have some dependencies on this Python file.
         python_deps = [
-            "$(TOPSRCDIR)/python/mozbuild/mozbuild/action/l10n_merge.py",
-            "$(TOPSRCDIR)/third_party/python/compare-locales/compare_locales/compare.py",
-            "$(TOPSRCDIR)/third_party/python/compare-locales/compare_locales/paths.py",
+            "$(TOPSRCDIR)/third_party/python/moz.l10n/moz/l10n/bin/build_file.py",
         ]
         # Add l10n dependencies we inferred:
-        for target, deps in sorted(six.iteritems(self._l10n_dependencies)):
+        for target, deps in sorted(self._l10n_dependencies.items()):
             mk.create_rule([target]).add_dependencies(
                 "%s" % d[0] for d in sorted(deps, key=itemgetter(0))
             )
@@ -231,10 +222,8 @@ class FasterMakeBackend(MakeBackend, PartialBackend):
                 )
                 rule.add_commands(
                     [
-                        "$(PYTHON3) -m mozbuild.action.l10n_merge "
-                        "--output {} --ref-file {} --l10n-file {}".format(
-                            merge, ref_file, l10n_file
-                        )
+                        "$(PYTHON3) -m moz.l10n.bin.build_file "
+                        f"--source {ref_file} --l10n {l10n_file} --target {merge}"
                     ]
                 )
                 # Add a dummy rule for the l10n file since it might not exist.
@@ -242,7 +231,7 @@ class FasterMakeBackend(MakeBackend, PartialBackend):
 
         mk.add_statement("include $(TOPSRCDIR)/config/faster/rules.mk")
 
-        for base, install_manifest in six.iteritems(self._install_manifests):
+        for base, install_manifest in self._install_manifests.items():
             with self._write_file(
                 mozpath.join(
                     self.environment.topobjdir,
@@ -255,7 +244,7 @@ class FasterMakeBackend(MakeBackend, PartialBackend):
         # Write a single unified manifest for consumption by |mach watch|.
         # Since this doesn't start 'install_', it's not processed by the build.
         unified_manifest = InstallManifest()
-        for base, install_manifest in six.iteritems(self._install_manifests):
+        for base, install_manifest in self._install_manifests.items():
             # Expect 'dist/bin/**', which includes 'dist/bin' with no trailing slash.
             assert base.startswith("dist/bin")
             base = base[len("dist/bin") :]
