@@ -2,18 +2,17 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-import io
 import logging
 import os
 import re
 from collections import defaultdict, namedtuple
+from functools import reduce
+from io import StringIO
 from itertools import chain
 from operator import itemgetter
 
 import mozpack.path as mozpath
-import six
 from mozpack.manifests import InstallManifest
-from six import StringIO
 
 from mozbuild import frontend
 from mozbuild.frontend.context import (
@@ -69,7 +68,7 @@ from ..frontend.data import (
     XPIDLModule,
 )
 from ..makeutil import Makefile
-from ..util import FileAvoidWrite, OrderedDefaultDict, ensureParentDir, pairwise
+from ..util import FileAvoidWrite, ensureParentDir, pairwise
 from .common import CommonBackend
 from .make import MakeBackend
 
@@ -123,7 +122,7 @@ def make_quote(s):
     return s.replace("#", r"\#").replace("$", "$$")
 
 
-class BackendMakeFile(object):
+class BackendMakeFile:
     """Represents a generated backend.mk file.
 
     This is both a wrapper around a file handle as well as a container that
@@ -167,8 +166,7 @@ class BackendMakeFile(object):
         self.fh.write(buf)
 
     def write_once(self, buf):
-        buf = six.ensure_text(buf)
-        if "\n" + buf not in six.ensure_text(self.fh.getvalue()):
+        if "\n" + buf not in self.fh.getvalue().decode():
             self.write(buf)
 
     # For compatibility with makeutil.Makefile
@@ -194,7 +192,7 @@ class BackendMakeFile(object):
         return self.fh.diff
 
 
-class RecursiveMakeTraversal(object):
+class RecursiveMakeTraversal:
     """
     Helper class to keep track of how the "traditional" recursive make backend
     recurses subdirectories. This is useful until all adhoc rules are removed
@@ -364,7 +362,7 @@ class RecursiveMakeBackend(MakeBackend):
         self._install_manifests["dist_private"]
 
         self._traversal = RecursiveMakeTraversal()
-        self._compile_graph = OrderedDefaultDict(set)
+        self._compile_graph = defaultdict(set)
         self._rust_targets = set()
         self._gkrust_target = None
         self._pre_compile = set()
@@ -478,7 +476,6 @@ class RecursiveMakeBackend(MakeBackend):
         elif isinstance(obj, HostSources):
             suffix_map = {
                 ".c": "HOST_CSRCS",
-                ".mm": "HOST_CMMSRCS",
                 ".cpp": "HOST_CPPSRCS",
             }
             variables = [suffix_map[obj.canonical_suffix]]
@@ -764,7 +761,7 @@ class RecursiveMakeBackend(MakeBackend):
             rule.add_dependencies(["%s/pre-compile" % relobjdir])
 
         all_compile_deps = (
-            six.moves.reduce(lambda x, y: x | y, self._compile_graph.values())
+            reduce(lambda x, y: x | y, self._compile_graph.values())
             if self._compile_graph
             else set()
         )
@@ -776,7 +773,7 @@ class RecursiveMakeBackend(MakeBackend):
         # - nodes that are rust targets.
         compile_roots = [
             t
-            for t, deps in six.iteritems(self._compile_graph)
+            for t, deps in self._compile_graph.items()
             if t in self._rust_targets or t not in all_compile_deps
         ]
 
@@ -810,7 +807,7 @@ class RecursiveMakeBackend(MakeBackend):
                     rule.add_dependencies(sorted(deps))
 
         non_default_roots = defaultdict(list)
-        non_default_graphs = defaultdict(lambda: OrderedDefaultDict(set))
+        non_default_graphs = defaultdict(lambda: defaultdict(set))
 
         for root in compile_roots:
             # If this is a non-default target, separate the root from the
@@ -834,7 +831,7 @@ class RecursiveMakeBackend(MakeBackend):
                 self._no_skip["syms"].remove(dirname)
 
         add_category_rules("compile", compile_roots, self._compile_graph)
-        for category, graph in sorted(six.iteritems(non_default_graphs)):
+        for category, graph in sorted(non_default_graphs.items()):
             add_category_rules(category, non_default_roots[category], graph)
 
         for relobjdir, tier, input in self._post_process_dependencies:
@@ -872,7 +869,7 @@ class RecursiveMakeBackend(MakeBackend):
             "non_default_tiers := %s" % " ".join(sorted(non_default_roots.keys()))
         )
 
-        for category, graphs in sorted(six.iteritems(non_default_graphs)):
+        for category, graphs in sorted(non_default_graphs.items()):
             category_dirs = [mozpath.dirname(target) for target in graphs.keys()]
             root_mk.add_statement("%s_dirs := %s" % (category, " ".join(category_dirs)))
 
@@ -993,7 +990,7 @@ class RecursiveMakeBackend(MakeBackend):
                 obj.topobjdir = bf.environment.topobjdir
                 obj.config = bf.environment
                 self._create_makefile(obj, stub=stub)
-                with io.open(obj.output_path, encoding="utf-8") as fh:
+                with open(obj.output_path, encoding="utf-8") as fh:
                     content = fh.read()
                     # Directories with a Makefile containing a tools target, or
                     # XPI_PKGNAME can't be skipped and must run during the
@@ -1365,8 +1362,12 @@ class RecursiveMakeBackend(MakeBackend):
     def _process_shared_library(self, libdef, backend_file):
         backend_file.write_once("LIBRARY_NAME := %s\n" % libdef.basename)
         backend_file.write("FORCE_SHARED_LIB := 1\n")
-        backend_file.write("IMPORT_LIBRARY := %s\n" % libdef.import_name)
-        backend_file.write("SHARED_LIBRARY := %s\n" % libdef.lib_name)
+        backend_file.write(
+            "IMPORT_LIBRARY := %s\n"
+            % self._pretty_path(libdef.import_path, backend_file)
+        )
+        shared_lib = self._pretty_path(libdef.output_path, backend_file)
+        backend_file.write("SHARED_LIBRARY := %s\n" % shared_lib)
         if libdef.soname:
             backend_file.write("DSO_SONAME := %s\n" % libdef.soname)
         if libdef.symbols_file:
@@ -1375,11 +1376,7 @@ class RecursiveMakeBackend(MakeBackend):
         if not libdef.cxx_link:
             backend_file.write("LIB_IS_C_ONLY := 1\n")
         if libdef.output_category:
-            self._process_non_default_target(libdef, libdef.lib_name, backend_file)
-            # Override the install rule target for this library. This is hacky,
-            # but can go away as soon as we start building libraries in their
-            # final location (bug 1459764).
-            backend_file.write("SHARED_LIBRARY_TARGET := %s\n" % libdef.output_category)
+            self._process_non_default_target(libdef, shared_lib, backend_file)
 
     def _process_static_library(self, libdef, backend_file):
         backend_file.write_once("LIBRARY_NAME := %s\n" % libdef.basename)
@@ -1392,9 +1389,8 @@ class RecursiveMakeBackend(MakeBackend):
         backend_file.write("WASM_ARCHIVE := %s\n" % libdef.basename)
 
     def _process_rust_library(self, libdef, backend_file):
-        backend_file.write_once(
-            "%s := %s\n" % (libdef.LIB_FILE_VAR, libdef.import_name)
-        )
+        rust_lib = self._pretty_path(libdef.import_path, backend_file)
+        backend_file.write_once("%s := %s\n" % (libdef.LIB_FILE_VAR, rust_lib))
         backend_file.write_once("CARGO_FILE := $(srcdir)/Cargo.toml\n")
         # Need to normalize the path so Cargo sees the same paths from all
         # possible invocations of Cargo with this CARGO_TARGET_DIR.  Otherwise,
@@ -1407,7 +1403,7 @@ class RecursiveMakeBackend(MakeBackend):
                 "%s := %s\n" % (libdef.FEATURES_VAR, " ".join(libdef.features))
             )
         if libdef.output_category:
-            self._process_non_default_target(libdef, libdef.import_name, backend_file)
+            self._process_non_default_target(libdef, rust_lib, backend_file)
 
     def _process_host_shared_library(self, libdef, backend_file):
         backend_file.write("HOST_SHARED_LIBRARY = %s\n" % libdef.lib_name)
@@ -1427,15 +1423,10 @@ class RecursiveMakeBackend(MakeBackend):
         )
 
     def _process_linked_libraries(self, obj, backend_file):
-        def pretty_relpath(lib, name):
-            return os.path.normpath(
-                mozpath.join(mozpath.relpath(lib.objdir, obj.objdir), name)
-            )
-
         objs, shared_libs, os_libs, static_libs = self._expand_libs(obj)
 
         obj_target = obj.name
-        if isinstance(obj, Program):
+        if isinstance(obj, (Program, SharedLibrary)):
             obj_target = self._pretty_path(obj.output_path, backend_file)
 
         objs_ref = " \\\n    ".join(os.path.relpath(o, obj.objdir) for o in objs)
@@ -1485,7 +1476,7 @@ class RecursiveMakeBackend(MakeBackend):
         for lib in shared_libs:
             assert obj.KIND != "host" and obj.KIND != "wasm"
             backend_file.write_once(
-                "SHARED_LIBS += %s\n" % pretty_relpath(lib, lib.import_name)
+                "SHARED_LIBS += %s\n" % self._pretty_path(lib.import_path, backend_file)
             )
 
         # We have to link any Rust libraries after all intermediate static
@@ -1497,7 +1488,7 @@ class RecursiveMakeBackend(MakeBackend):
             (l for l in static_libs if isinstance(l, BaseRustLibrary)),
         ):
             backend_file.write_once(
-                "%s += %s\n" % (var, pretty_relpath(lib, lib.import_name))
+                "%s += %s\n" % (var, self._pretty_path(lib.import_path, backend_file))
             )
 
         for lib in os_libs:
@@ -1587,7 +1578,9 @@ class RecursiveMakeBackend(MakeBackend):
                         else:
                             install_manifest.add_pattern_link(f.srcdir, f, dest_dir)
                     elif isinstance(f, AbsolutePath):
-                        if not f.full_path.lower().endswith((".dll", ".pdb", ".so")):
+                        if not f.full_path.lower().endswith(
+                            (".dll", ".pdb", ".so", ".dylib")
+                        ):
                             raise Exception(
                                 "Absolute paths installed to FINAL_TARGET_FILES must"
                                 " only be shared libraries or associated debug"
@@ -1759,7 +1752,7 @@ class RecursiveMakeBackend(MakeBackend):
             for manifest in sorted(manifests):
                 master.write('["include:%s"]\n' % manifest)
 
-    class Substitution(object):
+    class Substitution:
         """BaseConfigSubstitution-like class for use with _create_makefile."""
 
         __slots__ = ("input_path", "output_path", "topsrcdir", "topobjdir", "config")
@@ -1775,8 +1768,6 @@ class RecursiveMakeBackend(MakeBackend):
         with self._get_preprocessor(obj) as pp:
             if extra:
                 pp.context.update(extra)
-            if not pp.context.get("autoconfmk", ""):
-                pp.context["autoconfmk"] = "autoconf.mk"
             pp.handleLine(
                 "# THIS FILE WAS AUTOMATICALLY GENERATED. DO NOT MODIFY BY HAND.\n"
             )
@@ -1786,7 +1777,7 @@ class RecursiveMakeBackend(MakeBackend):
             pp.handleLine("srcdir := @srcdir@\n")
             pp.handleLine("srcdir_rel := @srcdir_rel@\n")
             pp.handleLine("relativesrcdir := @relativesrcdir@\n")
-            pp.handleLine("include $(DEPTH)/config/@autoconfmk@\n")
+            pp.handleLine("include $(DEPTH)/config/autoconf.mk\n")
             if not stub:
                 pp.do_include(obj.input_path)
             # Empty line to avoid failures when last line in Makefile.in ends
@@ -1841,7 +1832,7 @@ class RecursiveMakeBackend(MakeBackend):
         )
 
         ipdl_srcs_path = mozpath.join(ipdl_dir, "ipdlsrcs.txt")
-        mk.add_statement("ALL_IPDLSRCS_FILE := {}".format(ipdl_srcs_path))
+        mk.add_statement(f"ALL_IPDLSRCS_FILE := {ipdl_srcs_path}")
 
         # Preprocessed ipdl files are generated in ipdl_dir.
         mk.add_statement(
@@ -1859,10 +1850,10 @@ class RecursiveMakeBackend(MakeBackend):
         # Windows (32768 bytes) if the checkout path is sufficiently long.
         with self._write_file(ipdl_srcs_path) as srcs:
             for filename in sorted_nonstatic_ipdl_basenames:
-                srcs.write("{}\n".format(filename))
+                srcs.write(f"{filename}\n")
 
             for filename in sorted_static_ipdl_sources:
-                srcs.write("{}\n".format(filename))
+                srcs.write(f"{filename}\n")
 
         with self._write_file(mozpath.join(ipdl_dir, "ipdlsrcs.mk")) as ipdls:
             mk.dump(ipdls, removal_guard=False)
