@@ -2,17 +2,20 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 import json
-import jsonschema
 import os
 import pathlib
 import statistics
+import sys
+
+import jsonschema
 
-from mozperftest.utils import strtobool
 from mozperftest.layers import Layer
+from mozperftest.metrics.common import COMMON_ARGS, filtered_metrics
 from mozperftest.metrics.exceptions import PerfherderValidDataError
-from mozperftest.metrics.common import filtered_metrics, COMMON_ARGS
-from mozperftest.metrics.utils import write_json, is_number
-
+from mozperftest.metrics.notebook.constant import Constant
+from mozperftest.metrics.notebook.transformer import get_transformer
+from mozperftest.metrics.utils import has_callable_method, is_number, write_json
+from mozperftest.utils import strtobool
 
 PERFHERDER_SCHEMA = pathlib.Path(
     "testing", "mozharness", "external_tools", "performance-artifact-schema.json"
@@ -28,24 +31,6 @@ class Perfherder(Layer):
     arguments = COMMON_ARGS
     arguments.update(
         {
-            "app": {
-                "type": str,
-                "default": "firefox",
-                "choices": [
-                    "firefox",
-                    "chrome-m",
-                    "chrome",
-                    "chromium",
-                    "fennec",
-                    "geckoview",
-                    "fenix",
-                    "refbrow",
-                ],
-                "help": (
-                    "Shorthand name of application that is "
-                    "being tested (used in perfherder data)."
-                ),
-            },
             "stats": {
                 "action": "store_true",
                 "default": False,
@@ -105,8 +90,9 @@ class Perfherder(Layer):
             self.warning("No results left after filtering")
             return metadata
 
-        # XXX Add version info into this data
         app_info = {"name": self.get_arg("app", default="firefox")}
+        if metadata.binary_version:
+            app_info["version"] = metadata.binary_version
 
         # converting the metrics list into a mapping where
         # keys are the metrics nane
@@ -125,16 +111,21 @@ class Perfherder(Layer):
             # XXX Instead of just passing replicates here, we should build
             # up a partial perfherder data blob (with options) and subtest
             # overall values.
-            subtests = {}
+            subtests = []
             for r in res:
+                subtest = {}
                 vals = [v["value"] for v in r["data"] if is_number(v["value"])]
                 if vals:
-                    subtests[r["subtest"]] = vals
+                    subtest["name"] = r["subtest"]
+                    subtest["replicates"] = vals
+                    subtest["result"] = r
+                    subtests.append(subtest)
 
             perfherder_data = self._build_blob(
                 subtests,
                 name=name,
-                extra_options=settings.get("extraOptions"),
+                extra_options=settings.get("extraOptions", [])
+                + metadata.get_extra_options(),
                 should_alert=strtobool(settings.get("shouldAlert", False)),
                 application=app_info,
                 alert_threshold=float(settings.get("alertThreshold", 2.0)),
@@ -143,6 +134,7 @@ class Perfherder(Layer):
                 summary=settings.get("value"),
                 framework=settings.get("framework"),
                 metrics_info=metrics,
+                transformer=res[0].get("transformer", None),
             )
 
             if all_perfherder_data is None:
@@ -165,12 +157,19 @@ class Perfherder(Layer):
 
         file = "perfherder-data.json"
         if prefix:
-            file = "{}-{}".format(prefix, file)
-        self.info("Writing perfherder results to {}".format(os.path.join(output, file)))
+            file = f"{prefix}-{file}"
+        self.info(f"Writing perfherder results to {os.path.join(output, file)}")
 
         # XXX "suites" key error occurs when using self.info so a print
         # is being done for now.
-        print("PERFHERDER_DATA: " + json.dumps(all_perfherder_data))
+
+        # print() will produce a BlockingIOError on large outputs, so we use
+        # sys.stdout
+        sys.stdout.write("PERFHERDER_DATA: ")
+        json.dump(all_perfherder_data, sys.stdout)
+        sys.stdout.write("\n")
+        sys.stdout.flush()
+
         metadata.set_output(write_json(all_perfherder_data, output, file))
         return metadata
 
@@ -190,6 +189,7 @@ class Perfherder(Layer):
         unit="ms",
         summary=None,
         metrics_info=None,
+        transformer=None,
     ):
         """Build a PerfHerder data blob from the given subtests.
 
@@ -236,6 +236,10 @@ class Perfherder(Layer):
         :param summary float: The summary value to use in the perfherder
             data blob. By default, the mean of all the subtests will be
             used.
+        :param metrics_info dict: Contains a mapping of metric names to the
+            options that are used on the metric.
+        :param transformer str: The name of a predefined tranformer, a module
+            path to a transform, or a path to the file containing the transformer.
 
         :return dict: The PerfHerder data blob.
         """
@@ -244,17 +248,32 @@ class Perfherder(Layer):
         if subtest_should_alert is None:
             subtest_should_alert = []
         if framework is None:
-            framework = {"name": "browsertime"}
+            framework = {"name": "mozperftest"}
         if application is None:
             application = {"name": "firefox", "version": "9000"}
         if metrics_info is None:
             metrics_info = {}
 
+        # Use the transform to produce a suite value
+        const = Constant()
+        tfm_cls = None
+        transformer_obj = None
+        if transformer and transformer in const.predefined_transformers:
+            # A pre-built transformer name was given
+            tfm_cls = const.predefined_transformers[transformer]
+            transformer_obj = tfm_cls()
+        elif transformer is not None:
+            tfm_cls = get_transformer(transformer)
+            transformer_obj = tfm_cls()
+        else:
+            self.warning(
+                "No transformer found for this suite. Cannot produce a summary value."
+            )
+
         perf_subtests = []
         suite = {
             "name": name,
             "type": test_type,
-            "value": None,
             "unit": unit,
             "extraOptions": extra_options,
             "lowerIsBetter": lower_is_better,
@@ -272,12 +291,14 @@ class Perfherder(Layer):
 
         allvals = []
         alert_thresholds = []
-        for measurement in subtests:
-            reps = subtests[measurement]
+        for subtest_res in subtests:
+            measurement = subtest_res["name"]
+            reps = subtest_res["replicates"]
+            extra_info = subtest_res["result"]
             allvals.extend(reps)
 
             if len(reps) == 0:
-                self.warning("No replicates found for {}, skipping".format(measurement))
+                self.warning(f"No replicates found for {measurement}, skipping")
                 continue
 
             # Gather extra settings specified from within a metric specification
@@ -302,16 +323,38 @@ class Perfherder(Layer):
 
                 break
 
-            perf_subtests.append(
-                {
-                    "name": measurement,
-                    "replicates": reps,
-                    "lowerIsBetter": subtest_lower_is_better,
-                    "value": statistics.mean(reps),
-                    "unit": subtest_unit,
-                    "shouldAlert": should_alert or measurement in subtest_should_alert,
-                }
-            )
+            # The settings specified in the result (from `extra_info`) override
+            # the settings from the metric info which comes from the `--perfherder-metrics`
+            # argument. The `or` check ensures that if the extra_info setting is defined,
+            # but is `None`, then we use the option defined above from the CLI args.
+            subtest = {
+                "name": measurement,
+                "replicates": reps,
+                "value": extra_info.get("value"),
+                "unit": extra_info.get("unit") or subtest_unit,
+            }
+
+            # These two need to be done with if statements since they are boolean
+            # and the `or` check won't work with them (e.g. when a result specified setting
+            # is False, and the CLI arg is True).
+            if extra_info.get("shouldAlert") is not None:
+                subtest["shouldAlert"] = extra_info["shouldAlert"]
+            else:
+                subtest["shouldAlert"] = (
+                    should_alert or measurement in subtest_should_alert
+                )
+
+            if extra_info.get("lowerIsBetter") is not None:
+                subtest["lowerIsBetter"] = extra_info["lowerIsBetter"]
+            else:
+                subtest["lowerIsBetter"] = subtest_lower_is_better
+
+            if has_callable_method(transformer_obj, "subtest_summary"):
+                subtest["value"] = transformer_obj.subtest_summary(subtest)
+            if subtest["value"] is None:
+                subtest["value"] = statistics.mean(reps)
+
+            perf_subtests.append(subtest)
 
         if len(allvals) == 0:
             raise PerfherderValidDataError(
@@ -329,5 +372,12 @@ class Perfherder(Layer):
             suite["alertThreshold"] = alert_thresholds[0]
 
         suite["extraOptions"] = list(set(suite["extraOptions"]))
-        suite["value"] = statistics.mean(allvals)
+
+        if has_callable_method(transformer_obj, "summary"):
+            val = transformer_obj.summary(suite)
+            if val is not None:
+                suite["value"] = val
+        elif summary:
+            suite["value"] = summary
+
         return perfherder
