@@ -10,50 +10,83 @@
 // is used (from service.js).
 /* global Service */
 
-ChromeUtils.import("resource://testing-common/AddonTestUtils.jsm");
-ChromeUtils.import("resource://services-common/async.js");
-ChromeUtils.import("resource://services-common/utils.js");
-ChromeUtils.import("resource://testing-common/PlacesTestUtils.jsm");
-ChromeUtils.import("resource://services-sync/util.js");
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://gre/modules/PlacesUtils.jsm");
-ChromeUtils.import("resource://gre/modules/PlacesSyncUtils.jsm");
-ChromeUtils.import("resource://gre/modules/ObjectUtils.jsm");
-ChromeUtils.import("resource://testing-common/services/sync/utils.js");
-ChromeUtils.defineModuleGetter(this, "AddonManager",
-                               "resource://gre/modules/AddonManager.jsm");
+var { AddonTestUtils, MockAsyncShutdown } = ChromeUtils.importESModule(
+  "resource://testing-common/AddonTestUtils.sys.mjs"
+);
+var { Async } = ChromeUtils.importESModule(
+  "resource://services-common/async.sys.mjs"
+);
+var { CommonUtils } = ChromeUtils.importESModule(
+  "resource://services-common/utils.sys.mjs"
+);
+var { PlacesTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/PlacesTestUtils.sys.mjs"
+);
+var { sinon } = ChromeUtils.importESModule(
+  "resource://testing-common/Sinon.sys.mjs"
+);
+var { SerializableSet, Svc, Utils, getChromeWindow } =
+  ChromeUtils.importESModule("resource://services-sync/util.sys.mjs");
+var { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
+);
+var { PlacesUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/PlacesUtils.sys.mjs"
+);
+var { PlacesSyncUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/PlacesSyncUtils.sys.mjs"
+);
+var { ObjectUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/ObjectUtils.sys.mjs"
+);
+var {
+  MockFxaStorageManager,
+  SyncTestingInfrastructure,
+  configureFxAccountIdentity,
+  configureIdentity,
+  encryptPayload,
+  getLoginTelemetryScalar,
+  makeFxAccountsInternalMock,
+  makeIdentityConfig,
+  promiseNamedTimer,
+  promiseZeroTimer,
+  sumHistogram,
+  syncTestLogging,
+  waitForZeroTimer,
+} = ChromeUtils.importESModule(
+  "resource://testing-common/services/sync/utils.sys.mjs"
+);
+ChromeUtils.defineESModuleGetters(this, {
+  AddonManager: "resource://gre/modules/AddonManager.sys.mjs",
+});
 
-add_task(async function head_setup() {
+add_setup(async function head_setup() {
   // Initialize logging. This will sometimes be reset by a pref reset,
   // so it's also called as part of SyncTestingInfrastructure().
   syncTestLogging();
   // If a test imports Service, make sure it is initialized first.
-  if (this.Service) {
-    await this.Service.promiseInitialized;
+  if (typeof Service !== "undefined") {
+    await Service.promiseInitialized;
   }
 });
 
-// ================================================
-// Load mocking/stubbing library, sinon
-// docs: http://sinonjs.org/releases/v2.3.2/
-ChromeUtils.import("resource://gre/modules/Timer.jsm");
-Services.scriptloader.loadSubScript("resource://testing-common/sinon-2.3.2.js", this);
-/* globals sinon */
-// ================================================
-
-XPCOMUtils.defineLazyGetter(this, "SyncPingSchema", function() {
-  let ns = {};
-  ChromeUtils.import("resource://gre/modules/FileUtils.jsm", ns);
-  ChromeUtils.import("resource://gre/modules/NetUtil.jsm", ns);
-  let stream = Cc["@mozilla.org/network/file-input-stream;1"]
-               .createInstance(Ci.nsIFileInputStream);
+ChromeUtils.defineLazyGetter(this, "SyncPingSchema", function () {
+  let { FileUtils } = ChromeUtils.importESModule(
+    "resource://gre/modules/FileUtils.sys.mjs"
+  );
+  let { NetUtil } = ChromeUtils.importESModule(
+    "resource://gre/modules/NetUtil.sys.mjs"
+  );
+  let stream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(
+    Ci.nsIFileInputStream
+  );
   let schema;
   try {
     let schemaFile = do_get_file("sync_ping_schema.json");
-    stream.init(schemaFile, ns.FileUtils.MODE_RDONLY, ns.FileUtils.PERMS_FILE, 0);
+    stream.init(schemaFile, FileUtils.MODE_RDONLY, FileUtils.PERMS_FILE, 0);
 
-    let bytes = ns.NetUtil.readInputStream(stream, stream.available());
-    schema = JSON.parse((new TextDecoder()).decode(bytes));
+    let bytes = NetUtil.readInputStream(stream, stream.available());
+    schema = JSON.parse(new TextDecoder().decode(bytes));
   } finally {
     stream.close();
   }
@@ -64,11 +97,11 @@ XPCOMUtils.defineLazyGetter(this, "SyncPingSchema", function() {
   return schema;
 });
 
-XPCOMUtils.defineLazyGetter(this, "SyncPingValidator", function() {
-  let ns = {};
-  ChromeUtils.import("resource://testing-common/ajv-4.1.1.js", ns);
-  let ajv = new ns.Ajv({ async: "co*" });
-  return ajv.compile(SyncPingSchema);
+ChromeUtils.defineLazyGetter(this, "SyncPingValidator", function () {
+  const { JsonSchema } = ChromeUtils.importESModule(
+    "resource://gre/modules/JsonSchema.sys.mjs"
+  );
+  return new JsonSchema.Validator(SyncPingSchema);
 });
 
 // This is needed for loadAddonTestFunctions().
@@ -174,24 +207,17 @@ async function generateNewKeys(collectionKeys, collections = null) {
 // and stub part of Service.wm.
 
 function mockShouldSkipWindow(win) {
-  return win.closed ||
-         win.mockIsPrivate;
+  return win.closed || win.mockIsPrivate;
 }
 
 function mockGetTabState(tab) {
   return tab;
 }
 
-function mockGetWindowEnumerator(url, numWindows, numTabs, indexes, moreURLs) {
+function mockGetWindowEnumerator(urls) {
   let elements = [];
 
-  function url2entry(urlToConvert) {
-    return {
-      url: ((typeof urlToConvert == "function") ? urlToConvert() : urlToConvert),
-      title: "title",
-    };
-  }
-
+  const numWindows = 1;
   for (let w = 0; w < numWindows; ++w) {
     let tabs = [];
     let win = {
@@ -203,15 +229,16 @@ function mockGetWindowEnumerator(url, numWindows, numTabs, indexes, moreURLs) {
     };
     elements.push(win);
 
-    for (let t = 0; t < numTabs; ++t) {
-      tabs.push(Cu.cloneInto({
-        index: indexes ? indexes() : 1,
-        entries: (moreURLs ? [url].concat(moreURLs()) : [url]).map(url2entry),
-        attributes: {
-          image: "image",
+    let lastAccessed = 2000;
+    for (let url of urls) {
+      tabs.push({
+        linkedBrowser: {
+          currentURI: Services.io.newURI(url),
+          contentTitle: "title",
         },
-        lastAccessed: 1499,
-      }, {}));
+        lastAccessed,
+      });
+      lastAccessed += 1000;
     }
   }
 
@@ -238,73 +265,83 @@ function mockGetWindowEnumerator(url, numWindows, numTabs, indexes, moreURLs) {
 // Helper function to get the sync telemetry and add the typically used test
 // engine names to its list of allowed engines.
 function get_sync_test_telemetry() {
-  let ns = {};
-  ChromeUtils.import("resource://services-sync/telemetry.js", ns);
-  let testEngines = ["rotary", "steam", "sterling", "catapult"];
+  let { SyncTelemetry } = ChromeUtils.importESModule(
+    "resource://services-sync/telemetry.sys.mjs"
+  );
+  SyncTelemetry.tryRefreshDevices = function () {};
+  let testEngines = ["rotary", "steam", "sterling", "catapult", "nineties"];
   for (let engineName of testEngines) {
-    ns.SyncTelemetry.allowedEngines.add(engineName);
+    SyncTelemetry.allowedEngines.add(engineName);
   }
-  ns.SyncTelemetry.submissionInterval = -1;
-  return ns.SyncTelemetry;
+  SyncTelemetry.submissionInterval = -1;
+  return SyncTelemetry;
 }
 
 function assert_valid_ping(record) {
+  // Our JSON validator does not like `undefined` values, even though they will
+  // be skipped when we serialize to JSON.
+  record = JSON.parse(JSON.stringify(record));
+
   // This is called as the test harness tears down due to shutdown. This
   // will typically have no recorded syncs, and the validator complains about
   // it. So ignore such records (but only ignore when *both* shutdown and
   // no Syncs - either of them not being true might be an actual problem)
-  if (record && (record.why != "shutdown" || record.syncs.length != 0)) {
-    if (!SyncPingValidator(record)) {
-      if (SyncPingValidator.errors.length) {
+  if (record && (record.why != "shutdown" || !!record.syncs.length)) {
+    const result = SyncPingValidator.validate(record);
+    if (!result.valid) {
+      if (result.errors.length) {
         // validation failed - using a simple |deepEqual([], errors)| tends to
         // truncate the validation errors in the output and doesn't show that
         // the ping actually was - so be helpful.
         info("telemetry ping validation failed");
         info("the ping data is: " + JSON.stringify(record, undefined, 2));
-        info("the validation failures: " + JSON.stringify(SyncPingValidator.errors, undefined, 2));
-        ok(false, "Sync telemetry ping validation failed - see output above for details");
+        info(
+          "the validation failures: " +
+            JSON.stringify(result.errors, undefined, 2)
+        );
+        ok(
+          false,
+          "Sync telemetry ping validation failed - see output above for details"
+        );
       }
     }
     equal(record.version, 1);
     record.syncs.forEach(p => {
       lessOrEqual(p.when, Date.now());
-      if (p.devices) {
-        ok(!p.devices.some(device => device.id == record.deviceID));
-        equal(new Set(p.devices.map(device => device.id)).size,
-              p.devices.length, "Duplicate device ids in ping devices list");
-      }
     });
   }
 }
 
+function assert_success_sync(record) {
+  ok(!record.failureReason, JSON.stringify(record.failureReason));
+  equal(undefined, record.status);
+  greater(record.engines.length, 0);
+  for (let e of record.engines) {
+    ok(!e.failureReason);
+    equal(undefined, e.status);
+    if (e.validation) {
+      equal(undefined, e.validation.problems);
+      equal(undefined, e.validation.failureReason);
+    }
+    if (e.outgoing) {
+      for (let o of e.outgoing) {
+        equal(undefined, o.failed);
+        notEqual(undefined, o.sent);
+      }
+    }
+    if (e.incoming) {
+      equal(undefined, e.incoming.failed);
+      equal(undefined, e.incoming.newFailed);
+      notEqual(undefined, e.incoming.applied || e.incoming.reconciled);
+    }
+  }
+}
+
 // Asserts that `ping` is a ping that doesn't contain any failure information
 function assert_success_ping(ping) {
   ok(!!ping);
   assert_valid_ping(ping);
-  ping.syncs.forEach(record => {
-    ok(!record.failureReason, JSON.stringify(record.failureReason));
-    equal(undefined, record.status);
-    greater(record.engines.length, 0);
-    for (let e of record.engines) {
-      ok(!e.failureReason);
-      equal(undefined, e.status);
-      if (e.validation) {
-        equal(undefined, e.validation.problems);
-        equal(undefined, e.validation.failureReason);
-      }
-      if (e.outgoing) {
-        for (let o of e.outgoing) {
-          equal(undefined, o.failed);
-          notEqual(undefined, o.sent);
-        }
-      }
-      if (e.incoming) {
-        equal(undefined, e.incoming.failed);
-        equal(undefined, e.incoming.newFailed);
-        notEqual(undefined, e.incoming.applied || e.incoming.reconciled);
-      }
-    }
-  });
+  ping.syncs.forEach(assert_success_sync);
 }
 
 // Hooks into telemetry to validate all pings after calling.
@@ -318,7 +355,7 @@ function wait_for_pings(expectedPings) {
     let telem = get_sync_test_telemetry();
     let oldSubmit = telem.submit;
     let pings = [];
-    telem.submit = function(record) {
+    telem.submit = function (record) {
       pings.push(record);
       if (pings.length == expectedPings) {
         telem.submit = oldSubmit;
@@ -344,24 +381,68 @@ async function wait_for_ping(callback, allowErrorPings, getFullPing = false) {
   return record.syncs[0];
 }
 
-// Short helper for wait_for_ping
-function sync_and_validate_telem(allowErrorPings, getFullPing = false) {
-  return wait_for_ping(() => Service.sync(), allowErrorPings, getFullPing);
+// Perform a sync and validate all telemetry caused by the sync. If fnValidate
+// is null, we just check the ping records success. If fnValidate is specified,
+// then the sync must have recorded just a single sync, and that sync will be
+// passed to the function to be checked.
+async function sync_and_validate_telem(
+  fnValidate = null,
+  wantFullPing = false
+) {
+  let numErrors = 0;
+  let telem = get_sync_test_telemetry();
+  let oldSubmit = telem.submit;
+  try {
+    telem.submit = function (record) {
+      // This is called via an observer, so failures here don't cause the test
+      // to fail :(
+      try {
+        // All pings must be valid.
+        assert_valid_ping(record);
+        if (fnValidate) {
+          // for historical reasons most of these callbacks expect a "sync"
+          // record, not the entire ping.
+          if (wantFullPing) {
+            fnValidate(record);
+          } else {
+            Assert.equal(record.syncs.length, 1);
+            fnValidate(record.syncs[0]);
+          }
+        } else {
+          // no validation function means it must be a "success" ping.
+          assert_success_ping(record);
+        }
+      } catch (ex) {
+        print("Failure in ping validation callback", ex, "\n", ex.stack);
+        numErrors += 1;
+      }
+    };
+    await Service.sync();
+    Assert.ok(numErrors == 0, "There were telemetry validation errors");
+  } finally {
+    telem.submit = oldSubmit;
+  }
 }
 
 // Used for the (many) cases where we do a 'partial' sync, where only a single
 // engine is actually synced, but we still want to ensure we're generating a
 // valid ping. Returns a promise that resolves to the ping, or rejects with the
 // thrown error after calling an optional callback.
-async function sync_engine_and_validate_telem(engine, allowErrorPings, onError) {
+async function sync_engine_and_validate_telem(
+  engine,
+  allowErrorPings,
+  onError,
+  wantFullPing = false
+) {
   let telem = get_sync_test_telemetry();
   let caughtError = null;
   // Clear out status, so failures from previous syncs won't show up in the
   // telemetry ping.
-  let ns = {};
-  ChromeUtils.import("resource://services-sync/status.js", ns);
-  ns.Status._engines = {};
-  ns.Status.partial = false;
+  let { Status } = ChromeUtils.importESModule(
+    "resource://services-sync/status.sys.mjs"
+  );
+  Status._engines = {};
+  Status.partial = false;
   // Ideally we'd clear these out like we do with engines, (probably via
   // Status.resetSync()), but this causes *numerous* tests to fail, so we just
   // assume that if no failureReason or engine failures are set, and the
@@ -369,32 +450,37 @@ async function sync_engine_and_validate_telem(engine, allowErrorPings, onError)
   // a leftover.
   // This is only an issue since we're triggering the sync of just one engine,
   // without doing any other parts of the sync.
-  let initialServiceStatus = ns.Status._service;
-  let initialSyncStatus = ns.Status._sync;
+  let initialServiceStatus = Status._service;
+  let initialSyncStatus = Status._sync;
 
   let oldSubmit = telem.submit;
   let submitPromise = new Promise((resolve, reject) => {
-    telem.submit = function(ping) {
+    telem.submit = function (ping) {
       telem.submit = oldSubmit;
       ping.syncs.forEach(record => {
         if (record && record.status) {
           // did we see anything to lead us to believe that something bad actually happened
-          let realProblem = record.failureReason || record.engines.some(e => {
-            if (e.failureReason || e.status) {
-              return true;
-            }
-            if (e.outgoing && e.outgoing.some(o => o.failed > 0)) {
-              return true;
-            }
-            return e.incoming && e.incoming.failed;
-          });
+          let realProblem =
+            record.failureReason ||
+            record.engines.some(e => {
+              if (e.failureReason || e.status) {
+                return true;
+              }
+              if (e.outgoing && e.outgoing.some(o => o.failed > 0)) {
+                return true;
+              }
+              return e.incoming && e.incoming.failed;
+            });
           if (!realProblem) {
             // no, so if the status is the same as it was initially, just assume
             // that its leftover and that we can ignore it.
             if (record.status.sync && record.status.sync == initialSyncStatus) {
               delete record.status.sync;
             }
-            if (record.status.service && record.status.service == initialServiceStatus) {
+            if (
+              record.status.service &&
+              record.status.service == initialServiceStatus
+            ) {
               delete record.status.service;
             }
             if (!record.status.sync && !record.status.service) {
@@ -414,6 +500,8 @@ async function sync_engine_and_validate_telem(engine, allowErrorPings, onError)
           onError(ping.syncs[0], ping);
         }
         reject(caughtError);
+      } else if (wantFullPing) {
+        resolve(ping);
       } else {
         resolve(ping.syncs[0]);
       }
@@ -444,9 +532,9 @@ async function sync_engine_and_validate_telem(engine, allowErrorPings, onError)
 
 // Returns a promise that resolves once the specified observer notification
 // has fired.
-function promiseOneObserver(topic, callback) {
-  return new Promise((resolve, reject) => {
-    let observer = function(subject, data) {
+function promiseOneObserver(topic) {
+  return new Promise(resolve => {
+    let observer = function (subject, data) {
       Svc.Obs.remove(topic, observer);
       resolve({ subject, data });
     };
@@ -454,18 +542,10 @@ function promiseOneObserver(topic, callback) {
   });
 }
 
-// Avoid an issue where `client.name2` containing unicode characters causes
-// a number of tests to fail, due to them assuming that we do not need to utf-8
-// encode or decode data sent through the mocked server (see bug 1268912).
-// We stash away the original implementation so test_utils_misc.js can test it.
-Utils._orig_getDefaultDeviceName = Utils.getDefaultDeviceName;
-Utils.getDefaultDeviceName = function() {
-  return "Test device name";
-};
-
 async function registerRotaryEngine() {
-  let {RotaryEngine} =
-    ChromeUtils.import("resource://testing-common/services/sync/rotaryengine.js", {});
+  let { RotaryEngine } = ChromeUtils.importESModule(
+    "resource://testing-common/services/sync/rotaryengine.sys.mjs"
+  );
   await Service.engineManager.clear();
 
   await Service.engineManager.register(RotaryEngine);
@@ -479,10 +559,13 @@ async function registerRotaryEngine() {
 // Set the validation prefs to attempt validation every time to avoid non-determinism.
 function enableValidationPrefs(engines = ["bookmarks"]) {
   for (let engine of engines) {
-    Svc.Prefs.set(`engine.${engine}.validation.interval`, 0);
-    Svc.Prefs.set(`engine.${engine}.validation.percentageChance`, 100);
-    Svc.Prefs.set(`engine.${engine}.validation.maxRecords`, -1);
-    Svc.Prefs.set(`engine.${engine}.validation.enabled`, true);
+    Svc.PrefBranch.setIntPref(`engine.${engine}.validation.interval`, 0);
+    Svc.PrefBranch.setIntPref(
+      `engine.${engine}.validation.percentageChance`,
+      100
+    );
+    Svc.PrefBranch.setIntPref(`engine.${engine}.validation.maxRecords`, -1);
+    Svc.PrefBranch.setBoolPref(`engine.${engine}.validation.enabled`, true);
   }
 }
 
@@ -526,7 +609,7 @@ async function serverForFoo(engine, callback) {
   // do an engine sync only, there's no locking - so we end up with multiple
   // syncs running. Neuter that by making the threshold very large.
   Service.scheduler.syncThreshold = 10000000;
-  return serverForEnginesWithKeys({"foo": "password"}, engine, callback);
+  return serverForEnginesWithKeys({ foo: "password" }, engine, callback);
 }
 
 // Places notifies history observers asynchronously, so `addVisits` might return
@@ -536,37 +619,37 @@ async function promiseVisit(expectedType, expectedURI) {
   return new Promise(resolve => {
     function done(type, uri) {
       if (uri == expectedURI.spec && type == expectedType) {
-        PlacesUtils.history.removeObserver(observer);
-        PlacesObservers.removeListener(["page-visited"],
-                                       observer.handlePlacesEvents);
+        PlacesObservers.removeListener(
+          ["page-visited", "page-removed"],
+          observer.handlePlacesEvents
+        );
         resolve();
       }
     }
     let observer = {
       handlePlacesEvents(events) {
         Assert.equal(events.length, 1);
-        Assert.equal(events[0].type, "page-visited");
-        done("added", events[0].url);
+
+        if (events[0].type === "page-visited") {
+          done("added", events[0].url);
+        } else if (events[0].type === "page-removed") {
+          Assert.ok(events[0].isRemovedFromStore);
+          done("removed", events[0].url);
+        }
       },
-      onBeginUpdateBatch() {},
-      onEndUpdateBatch() {},
-      onTitleChanged() {},
-      onFrecencyChanged() {},
-      onManyFrecenciesChanged() {},
-      onDeleteURI(uri) {
-        done("removed", uri.spec);
-      },
-      onClearHistory() {},
-      onPageChanged() {},
-      onDeleteVisits() {},
     };
-    PlacesUtils.history.addObserver(observer, false);
-    PlacesObservers.addListener(["page-visited"],
-                                observer.handlePlacesEvents);
+    PlacesObservers.addListener(
+      ["page-visited", "page-removed"],
+      observer.handlePlacesEvents
+    );
   });
 }
 
-async function addVisit(suffix, referrer = null, transition = PlacesUtils.history.TRANSITION_LINK) {
+async function addVisit(
+  suffix,
+  referrer = null,
+  transition = PlacesUtils.history.TRANSITION_LINK
+) {
   let uriString = "http://getfirefox.com/" + suffix;
   let uri = CommonUtils.makeURI(uriString);
   _("Adding visit for URI " + uriString);
@@ -592,20 +675,14 @@ function bookmarkNodesToInfos(nodes) {
     if (node.children) {
       info.children = bookmarkNodesToInfos(node.children);
     }
-    if (node.annos) {
-      let orphanAnno = node.annos.find(anno =>
-        anno.name == "sync/parent"
-      );
-      if (orphanAnno) {
-        info.requestedParent = orphanAnno.value;
-      }
-    }
     return info;
   });
 }
 
 async function assertBookmarksTreeMatches(rootGuid, expected, message) {
-  let root = await PlacesUtils.promiseBookmarksTree(rootGuid);
+  let root = await PlacesUtils.promiseBookmarksTree(rootGuid, {
+    includeItemIds: true,
+  });
   let actual = bookmarkNodesToInfos(root.children);
 
   if (!ObjectUtils.deepEqual(actual, expected)) {
@@ -614,3 +691,21 @@ async function assertBookmarksTreeMatches(rootGuid, expected, message) {
     throw new Assert.constructor.AssertionError({ actual, expected, message });
   }
 }
+
+function add_bookmark_test(task) {
+  const { BookmarksEngine } = ChromeUtils.importESModule(
+    "resource://services-sync/engines/bookmarks.sys.mjs"
+  );
+
+  add_task(async function () {
+    _(`Running bookmarks test ${task.name}`);
+    let engine = new BookmarksEngine(Service);
+    await engine.initialize();
+    await engine._resetClient();
+    try {
+      await task(engine);
+    } finally {
+      await engine.finalize();
+    }
+  });
+}
